/*------------------------------------------------------------------------------
#
#	Apple Macintosh Developer Technical Support
#
#	KnowsPICT  by Jon Zap
#
#	CLUTBuilder.c	-	C Source
#
#	Copyright  1989 Apple Computer, Inc.
#	All rights reserved.
#
#
# this file contains functions used to pull colors from a PICT.  You activate it by
# calling CollectColors(PicHandle) with PicHandle set to a PICT that you have already
# loaded into memory.  It will almost invariably return a (handle to a) color table
# (that contains at least black and white) however the color table is not at all clean
# in the current implementation.  It is probably ok if you run it through NewPalette.
# If it runs across a direct pixmap it won't bomb but it won't add any colors to the
# color table.
#
# WARNING: This code has been tested but it has not been tested thoroughly;
# USE AT YOUR OWN RISK!
------------------------------------------------------------------------------*/

#include <QDOffscreen.h>
#include "CLUTBuilder.h"
#include "pictStuff.h"


#define	applec

OSErr		gColorError;		/* to report errors from bottlenecks. */
CTabHandle	gColorTable;		/* to collect colors from bottlenecks. */
short		gNextCSpec;			/* next CSpec entry in color table */
RGBColor	whiteRGB = { 0xFFFF,0xFFFF,0xFFFF };
RGBColor	blackRGB = { 0,0,0 };

PictInfo	gInfo, gInfoClear;

void AddRGBColor		(RGBColor *rgb);
void AddColorTable		(CTabHandle cTab);
void AddRGBForeColor	(void);
void AddRGBBackColor	(void);
void AddPixPat			(PixPatHandle pPat);
void AddPenPixPat		(void);
void AddFillPixPat		(void);
void AddVerb			(GrafVerb verb);

pascal void ColorTextProc	(short byteCount, Ptr textBuf, Point numer, Point denom);
pascal void ColorLineProc	(Point newPt);
pascal void ColorRectProc	(GrafVerb verb, Rect *r);
pascal void ColorRRectProc	(GrafVerb verb, Rect *r, short ovalWidth, short ovalHeight);
pascal void ColorOvalProc	(GrafVerb verb, Rect *r);
pascal void ColorArcProc	(GrafVerb verb, Rect *r, short startAngle, short arcAngle);
pascal void ColorPolyProc	(GrafVerb verb, PolyHandle poly);
pascal void ColorRgnProc	(GrafVerb verb, RgnHandle rgn);
pascal void ColorBitsProc	(BitMap *bitPtr, Rect *srcRect, Rect *dstRect, short mode, RgnHandle maskRgn);


OSErr CollectColors (PicHandle fromPicture,
					 PictInfo *info);



/* Add a color to the color table. */
void AddRGBColor (RGBColor *rgb)
{
	long numSpecs,sizeInBytes;
	int 	i,ctSize;
	CTabPtr TablePtr;
	RGBColor rgbx;
	
	if (gColorError)
		return;
	
	TablePtr = (*gColorTable);
	ctSize = TablePtr -> ctSize;
	for (i = 0; i <= ctSize; i++) {
		rgbx = TablePtr->ctTable[i].rgb;
	    if (rgbx.red==(*rgb).red &&
	    	rgbx.green==(*rgb).green &&
	    	rgbx.blue==(*rgb).blue) 
	    	return;		/* if already there, done */
	}
	numSpecs = (long) (++(**gColorTable).ctSize);	/* add a colorspec to table */
	sizeInBytes = (numSpecs * sizeof(ColorSpec)) + sizeof(ColorTable);
	SetHandleSize((Handle)gColorTable, sizeInBytes);
	if ((gColorError = MemError()) == noErr) {
		(**gColorTable).ctTable[numSpecs].rgb = *rgb;
	    (**gColorTable).ctTable[numSpecs].value = 0;
	}
}


/* Add the contents of another color table to our color table.*/
void AddColorTable (CTabHandle cTab)
{
	short index,size;
	RGBColor color;
	size = (**cTab).ctSize;
	for (index= 0; index <= size; index++)
	{
		color = (**cTab).ctTable[index].rgb;
		AddRGBColor(&color);
	}
}


/* Add the foreground color of the current port to the color table. */
void AddRGBForeColor (void)
{
	AddRGBColor(&((*(CGrafPtr)qd.thePort).rgbFgColor));
}


/* Add the background color of the current port to the color table. */
void AddRGBBackColor (void)
{
	AddRGBColor(&((*(CGrafPtr)qd.thePort).rgbBkColor));
}


/* Add colors from a PixPat to a color table. */
void AddPixPat (PixPatHandle pPat)
{
	switch ((**pPat).patType)
	{
		case 0:		/* one-bit patterns are drawn in the foreground and background color. */
			AddRGBForeColor();
			AddRGBBackColor();
			break;
		case 1:		/* Type 1 PixPats have a color table. */
			AddColorTable((**(**pPat).patMap).pmTable);
			break;
	}
}


/* Add colors from the pen PixPat to the color table. */
void AddPenPixPat (void)
{
	AddPixPat((*(CGrafPtr)qd.thePort).pnPixPat);
}


/* Add colors from the fill PixPat to the color table. */
void AddFillPixPat (void)
{
	AddPixPat((*(CGrafPtr)qd.thePort).fillPixPat);
}


/* Add colors because we are about to draw an object. */
void AddVerb (GrafVerb verb)
{
	switch (verb)
	{
		case frame:
		case paint: /* Framed and painted objects are drawn in the pen PixPat. */
				AddPenPixPat();
				break;
		case erase: /* Erased objects are drawn in the background color. */
				AddRGBBackColor();
				break;
		case fill:	/* Filled objects are drawn in the fill PixPat. */
				AddFillPixPat();
				break;
	}
}


/* bottleneck routines follow . . . */


pascal void ColorTextProc (short byteCount, Ptr textBuf, Point numer, Point denom)
{	/* Text is drawn with the foreground and background colors.*/
#ifdef applec
#pragma unused (byteCount, textBuf, numer, denom)
#endif
	AddRGBForeColor();
	AddRGBBackColor();
	gInfo.textCount++;
}


pascal void ColorLineProc (Point newPt)
{ /* Lines are drawn with the pen PixPat. */
#ifdef applec
#pragma unused (newPt)
#endif
	AddPenPixPat();
	gInfo.lineCount++;
}


pascal void ColorRectProc (GrafVerb verb, Rect *r)
{
#ifdef applec
#pragma unused (r)
#endif
	AddVerb(verb);
	gInfo.rectCount++;
}


pascal void ColorRRectProc (GrafVerb verb, Rect *r, short ovalWidth, short ovalHeight)
{
#ifdef applec
#pragma unused (r, ovalWidth, ovalHeight)
#endif
	AddVerb(verb);
	gInfo.rRectCount++;
}


pascal void ColorOvalProc (GrafVerb verb, Rect *r)
{
#ifdef applec
#pragma unused (r)
#endif
	AddVerb(verb);
	gInfo.ovalCount++;
}


pascal void ColorArcProc (GrafVerb verb, Rect *r, short startAngle, short arcAngle)
{
#ifdef applec
#pragma unused (r, startAngle, arcAngle)
#endif
	AddVerb(verb);
	gInfo.arcCount++;
}


pascal void ColorPolyProc (GrafVerb verb, PolyHandle poly)
{
#ifdef applec
#pragma unused (poly)
#endif
	AddVerb(verb);
	gInfo.polyCount++;
}


pascal void ColorRgnProc (GrafVerb verb, RgnHandle rgn)
{
#ifdef applec
#pragma unused (rgn)
#endif
	AddVerb(verb);
	gInfo.regionCount++;
}


pascal void ColorBitsProc (BitMap *bitPtr, Rect *srcRect, Rect *dstRect, short mode, RgnHandle maskRgn)
{
#ifdef applec
#pragma unused (srcRect, dstRect, mode, maskRgn)
#endif

	PixMapPtr	aPixMap;
	short		tempRB;

	/* Get the PixMap that we are about to draw.  SrcBits might be a BitMap, or
		one of two different kinds of PixMap pointers.  */
		
	tempRB = (*bitPtr).rowBytes;						/* local copy of rowBytes */
	if (tempRB & 0x8000)								/* high bit set? */
	{
		if (tempRB & 0x4000)							/* next to high bit set? */
			aPixMap = (** ( (PixMapHandle *)bitPtr ));	/* ptr to PixMap handle */
		else
			aPixMap = (PixMapPtr) bitPtr;				/* pointer to a PixMap */
			
		gInfo.pixMapCount++;
		
		if ( (aPixMap->pixelSize) > gInfo.depth)		/* deepest pixmap so far? */
			gInfo.depth = (aPixMap->pixelSize);
			
		if ( (aPixMap->hRes) > gInfo.hRes)				/* highest res pixmap so far? */
			gInfo.hRes = (aPixMap->hRes);
		if ( (aPixMap->vRes) > gInfo.vRes)
			gInfo.vRes = (aPixMap->vRes);
			
		if ( (aPixMap->pixelType) == 16 )
			return;										/* direct pixmap?  eek! */

		AddColorTable((*aPixMap).pmTable);				/* it has its own color table. */
	}
	else
	{
		gInfo.bitMapCount++;
		/* It's just a BitMap; it will use the background and foreground colors. */
		AddRGBBackColor();
		AddRGBForeColor();
	}
}


OSErr CollectColors (PicHandle pict,
					 PictInfo *info)
{
	OSErr			err = noErr;
	CQDProcs		procs;
	CTabHandle		colors;

	SetStdCProcs(&procs);

	procs.textProc  = NewQDTextProc ( ColorTextProc  ) ;
	procs.lineProc  = NewQDLineProc ( ColorLineProc  ) ;
	procs.rectProc  = NewQDRectProc ( ColorRectProc  ) ;
	procs.rRectProc = NewQDRRectProc( ColorRRectProc ) ;
	procs.ovalProc  = NewQDOvalProc ( ColorOvalProc  ) ;
	procs.arcProc   = NewQDArcProc  ( ColorArcProc   ) ;
	procs.polyProc  = NewQDPolyProc ( ColorPolyProc  ) ;
	procs.rgnProc   = NewQDRgnProc  ( ColorRgnProc   ) ;
	procs.bitsProc  = NewQDBitsProc ( ColorBitsProc  ) ;

		/* Create a color table containing black and white. */
		gInfo = gInfoClear;
		gInfo.depth = 1;							/* assume we will find a bitmap */
		
		colors = (CTabHandle) NewHandle( sizeof(ColorTable) + sizeof(ColorSpec) );
		if (!colors) return memFullErr;
		
		(**colors).ctSize = 1;					/* 2 entries */
		(**colors).ctFlags = 0x8000;
		(**colors).ctSeed = GetCTSeed();
		(**colors).ctTable[0].rgb = whiteRGB;	/*first entry is white*/
		(**colors).ctTable[1].rgb = blackRGB;	/*second entry is black*/
		
		gColorError = noErr;
		gColorTable = colors;
		
		/* Now play back the picture to get the colors.  The dstRect doesn't
		matter since our bottlenecks will never actually draw. We use global
		variables (gColorError and gColorTable) to communicate with the
		bottlenecks. */

	err = DrawPicHandleUsingBottlenecks ( pict, procs, nil ) ;
		
		/* Fail if error occurred while within the color bottlenecks. */
		if (gColorError != noErr)
		{
			DisposeHandle((Handle)colors);
			colors = 0L;
		}
	
		gInfo.theColorTable = colors ;
	
		*info = gInfo;
		
	DisposeRoutineDescriptor( procs.textProc  ) ;
	DisposeRoutineDescriptor( procs.lineProc  ) ;
	DisposeRoutineDescriptor( procs.rectProc  ) ;
	DisposeRoutineDescriptor( procs.rRectProc ) ;
	DisposeRoutineDescriptor( procs.ovalProc  ) ;
	DisposeRoutineDescriptor( procs.arcProc   ) ;
	DisposeRoutineDescriptor( procs.polyProc  ) ;
	DisposeRoutineDescriptor( procs.rgnProc   ) ;
	DisposeRoutineDescriptor( procs.bitsProc  ) ;
		
	return err;
}
