// Simple framework for Macintosh sample code
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the AppleEvent utility routines.
// 
// 9/16/94	nick	first cut
// 11/16/94	david	added a global gAE_available to aeGlobals.c
//					changeg AE_available to set/clear the global


#include <GestaltEqu.h>
#include <AppleEvents.h>

#include "aeGlobals.h"
#include "aeUtils.h"
#include "aeHandlers.h"

#include "ErrorHandling.h"



//-----------------------------------------------------------------------
// returns true if the platform supports appleevents - we won't run
// if it doesn't
//
// SIDE EFFECTS:
//
// sets the global gAE_available based on the return value

Boolean AE_available (void)
{
	OSErr err;
	long response;
	
	err = Gestalt(gestaltAppleEventsAttr,&response);
	if (err != noErr)
		gAE_available = false;
		
	gAE_available = (response && (response << gestaltAppleEventsPresent));
	
	return gAE_available;
}

//-----------------------------------------------------------------------
// call this to init the AE stuff in this file

void InitAEStuff( void ) 
{
	// Set up the self-addressed descriptor record.
	gSelfPSN.highLongOfPSN = 0;
	gSelfPSN.lowLongOfPSN = kCurrentProcess;		//* Use this instead of GetCurrentProcess *//
	FailIfErr(AECreateDesc(typeProcessSerialNumber,(Ptr)&gSelfPSN,sizeof(ProcessSerialNumber),&gSelfAddress));
	gNullDesc.descriptorType = typeNull;			// Initialize the global null descriptor record.
	gNullDesc.dataHandle = nil;

}

//-----------------------------------------------------------------------
// utility routine to ensure we got all required parameters from the 
// appleevent passed in.

OSErr MyGotRequiredParams(AppleEvent *theAppleEvent)
{
	DescType returnedType;
	Size actualSize;
	OSErr err;
	
	err = AEGetAttributePtr(theAppleEvent,keyMissedKeywordAttr,typeWildCard,
							&returnedType,nil,0,&actualSize);
	if (err == errAEDescNotFound)
		err = noErr;
	else if (err==noErr)
		err = errAEEventNotHandled;
	return err;
}
	

//-----------------------------------------------------------------------
// called to register our appleevent handlers

void RegisterMyEvents(void)
{
	OSErr err;
	
	if (!gAE_available)
		return;
	
	err = AEInstallEventHandler( kCoreEventClass, kAEOpenApplication,
								 NewAEEventHandlerProc(HandleOAPP),
								 0L, false ) ;
	if (err!=noErr)
		return;
				
	err = AEInstallEventHandler( kCoreEventClass, kAEOpenDocuments,
								 NewAEEventHandlerProc(HandleODOC),
								 0L, false ) ;
	if (err!=noErr)
		return;
				
	err = AEInstallEventHandler( kCoreEventClass, kAEPrintDocuments,
								 NewAEEventHandlerProc(HandlePDOC),
								 0L, false ) ;
	if (err!=noErr)
		return;
				
	err = AEInstallEventHandler( kCoreEventClass, kAEQuitApplication,
								 NewAEEventHandlerProc(HandleQUIT),
								 0L, false ) ;
	if (err!=noErr)
		return;			
}


