// Simple framework for Macintosh sample code
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the framework's AppleEvent handlers
// 
// 9/13/94	nick	first cut
// 12/13/94	david	several modifications


#include <Memory.h>
#include <AppleEvents.h>
#include <SegLoad.h>
#include <Errors.h>

#include "aeHandlers.h"
#include "aeGlobals.h"

#include "appGlobals.h"
#include "appAEHandlers.h"

#include "ErrorHandling.h"

#include "win.h"
#include "winPictDoc.h"
#include "winProfile.h"

#include "qdPrintUtils.h"
#include "gxPrintUtils.h"
#include "gxGlobals.h"


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	appHandleOAPP
 *------------------------------------------------------------------------------*
		This routine handles the 'OAPP' AppleEvent for the framework.
		This routine is called by HandleOAPP.
\*------------------------------------------------------------------------------*/
OSErr appHandleOAPP ( void ) 
{
	// put something here ?
	return noErr;
}


/*------------------------------------------------------------------------------*\
	appHandlePDOC
 *------------------------------------------------------------------------------*
		This routine handles the 'PDOC' AppleEvent for the framework.
		This routine is called by HandlePDOC.
\*------------------------------------------------------------------------------*/
OSErr appHandlePDOC ( FSSpec *spec ) 
{
	OSErr			err = noErr ;
	winHandle		win ;
	FInfo			info ;
	AllocProcPtr	allocProc = nil ;
	Boolean			wasAlreadyOpen = false ;
	
	// get the file info for the FSSpec
	err = FSpGetFInfo(spec, &info);
	if ( err != noErr ) return err;
	
	// determine what type of winHandle we need
	if (info.fdType == 'PICT')		// should be factored out
		allocProc = winAllocPictDoc ;
	else
		return 314 ;				// should have a real error number and dialog

	// create a new winHandle of the proper type
	err = NewWinHandle( &win, allocProc ) ;
	FailIfErr( err ) ;

	// set this field of the winHandle so that the doc can be opened
	SetWinFSSpec ( win, spec ) ;
	
	err = CallWinOpenProc ( win ) ;
	if ( err != noErr )						//if an error occured
		DisposeWinHandle( win );

	if ( err == kWasAlreadyOpen )
	{
		wasAlreadyOpen = true ;
		err = noErr;
	}
	
	if ( err ) return err ;
		
	// we know the document window is frontmost because we opened it
	win = GetWindowWinHandle( FrontWindow() ) ;
	if ( win == nil ) return nilHandleErr;
	
	if( !gQDGX_available ) 					// if GX is not installed ...
		DoPrintLoop( win ) ;				// then do the classic page setup dialog
	else									// otherwise ...
		DoGXPrintLoop( win ) ;				// handle the GX print loop
	
	if( !wasAlreadyOpen )					// do we need to close it?
		CallWinCloseProc( win ) ;			// close the window
	
	return err;
}


/*------------------------------------------------------------------------------*\
	appHandleODOC
 *------------------------------------------------------------------------------*
		This routine handles the 'ODOC' AppleEvent for the framework.
		This routine is called by HandleODOC.
\*------------------------------------------------------------------------------*/
OSErr appHandleODOC ( FSSpec *spec ) 
{
	OSErr			err = noErr ;
	winHandle		win ;
	FInfo			info ;
	AllocProcPtr	allocProc = nil ;
	
	// get the file info for the FSSpec
	err = FSpGetFInfo(spec, &info);
	if ( err != noErr ) return err;
	
	// determine what type of winHandle we need
	if (info.fdType == 'PICT')		// should be factored out
		allocProc = winAllocPictDoc ;
	else if (info.fdType == 'prof')
		allocProc = winAllocProfile ;
	else
		return 314 ;				// should have a real error number and dialog
	
	// create a new winHandle of the proper type
	err = NewWinHandle( &win, allocProc ) ;
	FailIfErr( err ) ;

	// set this field of the winHandle so that the doc can be opened
	SetWinFSSpec ( win, spec ) ;
	
	err = CallWinOpenProc ( win ) ;
	if ( err != noErr )						//if an error occured
		DisposeWinHandle( win );
	
	if ( err == kWasAlreadyOpen )
		err = noErr;

	return err ;
}


/*------------------------------------------------------------------------------*\
	appHandleQUIT
 *------------------------------------------------------------------------------*
		This routine handles the 'QUIT' AppleEvent for the framework.
		This routine is called by HandleQUIT.
\*------------------------------------------------------------------------------*/
OSErr appHandleQUIT ( void ) 
{
	// close all windows and signal to Quit
	// ask to save all dirty files here
	
	gQuitFlag = true;
	return noErr;
}


