// Simple framework for Macintosh sample code
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1994, Apple Computer,Inc
//
// Application event dispatching code.
// Most of the stuff in here will not need altering from app to app.
// 
// 9/13/94	nick	first cut
// 12/13/94	david	several modifications


#include <LowMem.h>
#include <Windows.h>
#include <ToolUtils.h>
#include <Errors.h>
#include <AppleEvents.h>
#include <SegLoad.h>
#include <DiskInit.h>

#include "aeGlobals.h"
#include "aeUtils.h"

#include "appGlobals.h"
#include "appMain.h"
#include "appMenus.h"

#include "win.h"

#include "gxGlobals.h"
#include "gxUtils.h"
#include "colorsyncStuff.h"

#include "ErrorHandling.h"


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
void	main				( void ) ;
void	InitToolbox			( void ) ;
void	InitMenuBar			( void ) ;
void	DoEventLoop			( void ) ;
void	DoMouseDownEvent	( EventRecord *event ) ;
void	DoKeyDownEvent		( EventRecord *event ) ;
void	DoUpdateEvent		( EventRecord *event ) ;
void	DoDiskEvent			( EventRecord *event ) ;
void	DoActivateEvent		( EventRecord *event ) ;
void	DoOSEvent			( EventRecord *event ) ;
void	DoNullEvent			( EventRecord *event ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	DoEvent
 *------------------------------------------------------------------------------*
		This routine dispaches one event record.
		This routine is called by DoEventLoop() and also by the
		GX Printing Event Handler.
\*------------------------------------------------------------------------------*/
void DoEvent ( EventRecord *event )
{
	switch (event->what)
	{
		case nullEvent :						/* handle null events */
			DoNullEvent( event ) ;
			break;
		
		case mouseDown :						/* handle mouse clicks */
			DoMouseDownEvent( event ) ;
			break;
		
		case keyDown :							/* handle key hits */
		case autoKey :
			DoKeyDownEvent( event ) ;
			break;
			
		case updateEvt :						/* handle update events */
			DoUpdateEvent( event ) ;
			break;
		
		case diskEvt:							/* handle disk events */
			DoDiskEvent( event ) ;
			break;

		case kHighLevelEvent:					/* handle Apple Events */
			AEProcessAppleEvent( event ) ;
			break;

		case activateEvt:						/* handle activate events */
			DoActivateEvent( event ) ;
			break;

		case osEvt:								/* handle os events */
			DoOSEvent( event ) ;
			break;
	}
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	main
 *------------------------------------------------------------------------------*
		the main routine which initialized the various managers and
		starts the main event loop.
\*------------------------------------------------------------------------------*/
void main ( void )
{
	gxGraphicsClient	myGXClient = nil;		// this is the client reference for this app
	OSErr				theError = noErr ;

	InitToolbox();
	
	gAppResRefNum = CurResFile();

	// we only want to run if we have AppleEvents - i.e. System 7 or later.
	if (!AE_available())
		FatalError( eFatalNeedsAEVTMgr ) ;

	InitAEStuff();
	InitMenuBar() ;
	RegisterMyEvents();
	
	// we'd prefer if ColorSync 2 was available.
	if (!CS2_available())
		AlertUser ( eWarnNoColorSync ) ;

	// check for GX
	if ( QDGX_available() )
		InitGXStuff() ;

	// assuming all went OK then we want to just run
	if( theError == noErr )
		DoEventLoop() ;
	
	// if necessary, clean up after GX
	DisposeGXStuff();

}


/*------------------------------------------------------------------------------*\
	InitToolbox
 *------------------------------------------------------------------------------*
		This routine initializes the standard managers.
		This routine is called by main().
\*------------------------------------------------------------------------------*/
void InitToolbox ( void )
{
	MaxApplZone();

	InitGraf((Ptr) &qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	FlushEvents(everyEvent,0);
	TEInit();
	InitDialogs(0L);
	InitCursor();

	MoreMasters() ; MoreMasters() ; MoreMasters() ; MoreMasters() ; 
	MoreMasters() ; MoreMasters() ; MoreMasters() ; MoreMasters() ; 
	MoreMasters() ; MoreMasters() ; MoreMasters() ; MoreMasters() ; 
}


/*------------------------------------------------------------------------------*\
	InitMenuBar
 *------------------------------------------------------------------------------*
		This routine reads in the MBAR resource, adds the DA to the apple 
		menu, and sets up all the menu states by calling dimAllMenus
		and DoAppAdjustMenus.
		This routine is called by main().
\*------------------------------------------------------------------------------*/
void InitMenuBar ( void )
{
	OSErr 		retCode = noErr ;
	Handle		menuBar = nil;
	
	menuBar = GetNewMBar(128);				// Read menus into menu bar, MBAR res id is 128
		
	if ( menuBar == nil )					// if we dont have it then quit 
	{										// your app needs a dialog here, maybe.
		SysError( dsSysErr ) ; 				// Thing is, if it couldn't get the menu bar,
		ExitToShell(); 						// it may not get the dialog resource.  
	}	 									// I call Syserror here.  This way I
		 									// know there has been a problem
 
	SetMenuBar( menuBar );					// Install menus
	DisposHandle( menuBar );
	
	AddResMenu(GetMHandle(mApple), 'DRVR');	// Add DA names to Apple menu, ID 128
	dimAllMenus() ;
	DoAppAdjustMenus() ;
	DrawMenuBar();
}


/*------------------------------------------------------------------------------*\
	DoEventLoop
 *------------------------------------------------------------------------------*
		This routine processes events until gQuitFlag gets set.
		This routine is called by main().
\*------------------------------------------------------------------------------*/
void DoEventLoop ( void )
{
	EventRecord event;

	while ( !gQuitFlag ) 
	{
		if (WaitNextEvent( everyEvent, &event, 30, nil ))
			DoEvent (&event);
		else
			DoNullEvent (&event);
	}
}


/*------------------------------------------------------------------------------*\
	DoMouseDownEvent
 *------------------------------------------------------------------------------*
		This routine handles mouse down events.
			inDrag clicks are handled here,
			inContent clicks are sent to the window's ClickProcPtr,
			inGoAway clicks are sent to the window's CloseProcPtr
			inMenuBar clicks are handled by calling HandleMenuCommand(),
			inGrow clicks are handled by resizing the window and calling its ResizeProcPtr,
			inZoom clicks are handled by zooming the window and calling its ResizeProcPtr,
		This routine is called by DoEvent().
\*------------------------------------------------------------------------------*/
void DoMouseDownEvent ( EventRecord *event )
{
	WindowPtr		window;
	short			clickArea;
	Rect			screenRect;
	RgnHandle		grayRgn;
	winHandle		win = nil;

	clickArea = FindWindow( event->where, &window );
	win = GetWindowWinHandle( window );
	
	switch (clickArea)
	{
		case inDrag:
			grayRgn = LMGetGrayRgn();
			screenRect = (**grayRgn).rgnBBox;
			SelectWindow( window );
			DragWindow( window, event->where, &screenRect );
			break ;

		case inContent:
			if (window == FrontWindow())
			{
				if (win != nil)
					CallWinClickProc( win, event ) ;
			}
			else
				SelectWindow( window );		/// for d'n'd we wont want to do this...
			break;
		
		case inGoAway:
			if (TrackGoAway( window, event->where ))
				if (win != nil)
					CallWinCloseProc( win ) ;
			break;

		case inMenuBar: 
			HandleMenuCommand(MenuSelect(event->where));
			break;

		case inGrow:
			if (win != nil)
			{
				Rect	sizeRect ;
				long	sizeHV ;

				sizeRect = GetWinSizeRect( win );
		 		sizeHV = GrowWindow( window, event->where, &sizeRect );
				SizeWindow( window, LoWord(sizeHV), HiWord(sizeHV), true);
				CallWinResizeProc( win ) ;
			}
			break;

		case inZoomIn: 
		case inZoomOut: 
			if ( TrackBox( window,event->where,clickArea) )
			{
				EraseRect( &(window->portRect) ) ;
				ZoomWindow( window, clickArea, true) ;
				CallWinResizeProc( win ) ;
			}
			break;

		default:
			break ;
	}
}


/*------------------------------------------------------------------------------*\
	DoKeyDownEvent
 *------------------------------------------------------------------------------*
		This routine handles key down events.
		If the command key is down then the event is handled by calling HandleMenuCommand(),
		Otherwize, the event is sent to the frontmost window by calling its KeyProcPtr,
		This routine is called by DoEvent().
\*------------------------------------------------------------------------------*/
void DoKeyDownEvent ( EventRecord *event )
{
	char			key;
	winHandle		win = nil;

	key = event->message & charCodeMask;
	
	if ( event->modifiers & cmdKey ) 			/* Command key down? */
		HandleMenuCommand( MenuKey(key) );
	else
	{
		// see if the frontmost window can handle the event
		win = GetWindowWinHandle( FrontWindow() );
		if (win != nil)
			CallWinKeyProc( win, event );
	}
}


/*------------------------------------------------------------------------------*\
	DoUpdateEvent
 *------------------------------------------------------------------------------*
		This routine handles update events.
		The event is sent to the appropriate window by calling its UpdateProcPtr,
		This routine is called by DoEvent().
\*------------------------------------------------------------------------------*/
void DoUpdateEvent ( EventRecord *event )
{
	WindowPtr   	window;
	winHandle		win = nil;

	window = (WindowPtr) event->message;
					
	win = GetWindowWinHandle( window );
	if (win != nil)
		CallWinUpdateProc(win, event);
}


/*------------------------------------------------------------------------------*\
	DoDiskEvent
 *------------------------------------------------------------------------------*
		This routine handles disk events.
		It is called by DoEvent().
\*------------------------------------------------------------------------------*/
void DoDiskEvent ( EventRecord *event )
{
	Point aPoint = {100, 100};

	if ( HiWrd(event->message) != noErr ) 
		(void) DIBadMount(aPoint, event->message);
}


/*------------------------------------------------------------------------------*\
	DoActivateEvent
 *------------------------------------------------------------------------------*
		This routine handles activate events.  Whenever a window becomes active,
		it sets up all the menu states by calling dimAllMenus and DoAppAdjustMenus.
		The event is then forwarded to the appropriate window by calling its 
		ActivateProcPtr.
		This routine is called by DoEvent().
\*------------------------------------------------------------------------------*/
void DoActivateEvent ( EventRecord *event )
{
	WindowPtr   	window;
	winHandle		win = nil;
	Boolean			becomingActive ;

	becomingActive = (event->modifiers) & activeFlag ;

	if (becomingActive)
	{
		dimAllMenus() ;				// dim all menus
		DoAppAdjustMenus() ;		// undim app items
	}

	window = (WindowPtr)event->message ;

	win = GetWindowWinHandle( window );
	if (win != nil)
		CallWinActivateProc(win, (event->modifiers) & activeFlag );
}


/*------------------------------------------------------------------------------*\
	DoOSEvent
 *------------------------------------------------------------------------------*
		This routine handles os events such as Suspend/Resume and MouseMoved
		On Suspend/Resume the gInBackground global and the GXJob are updated. 
		Also, the frontmost window is notified by calling its ActivateProcPtr
		This routine is called by DoEvent().
\*------------------------------------------------------------------------------*/
void DoOSEvent ( EventRecord *event )
{
	WindowPtr   	window;
	winHandle		win = nil;

	switch ((event->message >> 24) & 0x00FF) 		// High byte of message
	{
		case suspendResumeMessage:

			gInBackground = (event->message & resumeFlag == 0) ;

			win = GetWindowWinHandle( FrontWindow() );
			if (win != nil)
				CallWinActivateProc(win, !gInBackground);
			
			// check to see if GX is installed, if so then make sure we 
			// update our job object for each window
			
			if( gQDGX_available ) 
			{
				for( window = FrontWindow(); 
				     window != nil;  
				     window = (WindowPtr)((WindowPeek) window)->nextWindow )
				{
					win = GetWindowWinHandle( window ) ;
					if( win != nil )
				        GXUpdateJob( GetWinGXJob(win) );
				}
			}
			break;

		case mouseMovedMessage:
			break;
	}
}


/*------------------------------------------------------------------------------*\
	DoNullEvent
 *------------------------------------------------------------------------------*
		This routine handles null events.
		The event is sent to the frontmost window by calling its NullProcPtr.
		Note: I may change this so that any window with a NullProcPtr will 
		recieve the null event.
		This routine is called by DoEvent().
\*------------------------------------------------------------------------------*/
void DoNullEvent ( EventRecord *event )
{
	winHandle		win = nil;

	win = GetWindowWinHandle( FrontWindow() );
	if (win != nil)
		CallWinNullProc(win, event);
}



