// Simple framework for Macintosh sample code
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1994, Apple Computer,Inc
// 
// Application menu handling code such as dimming menu items and 
// dispatching items.  The dispatcher first calls the the MenuProcPtr
// of the frontmost window to see if it could handle the event.  Any
// items not handled by the front windo are handled here.
//
// 9/13/94	nick	first cut
// 9/13/94	nick	implement commands
// 12/13/94	david	several modifications


#include <StandardFile.h>
#include <SegLoad.h>
#include <Devices.h>

#include "aeSenders.h"

#include "appGlobals.h"
#include "appMain.h"
#include "appMenus.h"

#include "ErrorHandling.h"

#include "win.h"
#include "winAbout.h"
#include "winProfList.h"


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
void DoAppNewCommand	( void ) ;
void DoAppOpenCommand	( void ) ;
void DoAboutBoxCommand	( void ) ;
void DoProfListCommand  ( void ) ;
void DoAppQuitCommand	( void ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	dimAllMenus
 *------------------------------------------------------------------------------*
		This routine dims all menus and menu items in the menu bar
		It is called by:
			InitMenuBar() which is called when the app is intited,
			DoActivateEvent() which is called whenever a window is brought to front,
			AdjustMenusForPrinting() which is called before GX print dialogs, and
			CloseProcPtrs which are called whenever windows are closed 
\*------------------------------------------------------------------------------*/
void dimAllMenus ( void )
{
	register short			m,i,items;
	register MenuHandle		mhdl;

	for ( m = mFirstMenu; m <= mLastMenu; m++ )
	{
		mhdl = GetMHandle(m) ;
		DisableItem( mhdl, 0 );		// 0 ==> the whole menu

		items = (m==mApple) ? (1) : (CountMItems(mhdl)) ;

		for (i = 1; i<=items ; i++)
			DisableItem( mhdl, i );
	}
}
		
		
/*------------------------------------------------------------------------------*\
	DoAppAdjustMenus
 *------------------------------------------------------------------------------*
		This routine enables any menus and menu items in the menu bar
		which the application is responsible for handling.
		It is called by:
			InitMenuBar() which is called when the app is intited,
			DoActivateEvent() which is called whenever a window is brought to front, and
			CloseProcPtrs which are called whenever windows are closed 
\*------------------------------------------------------------------------------*/
void DoAppAdjustMenus ( void ) 
{
	MenuHandle	theMenu ;
	
	// do the file menu
	theMenu = GetMHandle ( mApple ) ;
	EnableItem ( theMenu, 0 );
	EnableItem ( theMenu, iAbout );	
	
	// do the file menu
	theMenu = GetMHandle ( mFile ) ;
	EnableItem ( theMenu, 0 );
	EnableItem ( theMenu, iOpen );
	EnableItem ( theMenu, iQuit );

	// do the profiles menu
	theMenu = GetMHandle ( mProfiles ) ;
	EnableItem ( theMenu, 0 );
	EnableItem ( theMenu, iCSFolder );
}


/*------------------------------------------------------------------------------*\
	HandleMenuCommand
 *------------------------------------------------------------------------------*
		This routine handles all commands generated by mouse event or
		by command key equivalent.  This routine first calls the the MenuProcPtr
		of the frontmost window to see if it could handle the event.  After that
		it hadles any menu items which the application is responsible for.
		It is called by:
			DoMouseDownEvent() which is called in response to mouse events, and
			DoKeyDownEvent() which is called in response to keyboard events
\*------------------------------------------------------------------------------*/
void HandleMenuCommand (long menuResult)
{
	short			menuID;
	short			menuItem;
	Str255			daName;
	winHandle		win = nil;
	
	// see if the frontmost window can handle the event

	win = GetWindowWinHandle( FrontWindow() );
	if (win != nil)
		CallWinMenuProc(win, menuResult);

	// I should check to see if the frontmost window handled it 

	menuID = HiWrd(menuResult);
	menuItem = LoWrd(menuResult);
	switch ( menuID ) {
	
		case mApple:
			switch ( menuItem )
			{
				case iAbout:
					DoAboutBoxCommand() ;
					break;
					
				default:
					GetItem(GetMHandle(mApple), menuItem, daName);
					(void) OpenDeskAcc(daName);
					break;
			}
			break;
			
		case mFile:
			switch ( menuItem )
			{
				case iNew:				// make a new document
					DoAppNewCommand() ;
					break;
					
				case iOpen:				// open an existing file
					DoAppOpenCommand() ;
					break;
					
				case iQuit:
					DoAppQuitCommand() ;
					break;
			}
			break;

		case mProfiles:
			switch ( menuItem )
			{
				case iSystemProf:
					break;
					
				case iCSFolder:
					DoProfListCommand() ;
					break;
			}
			break;
		
			
	}
	HiliteMenu(0);		// Unhighlight whatever MenuSelect or MenuKey hilited
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	DoAppNewCommand
 *------------------------------------------------------------------------------*
		This routine handles the File:New command for the application.
		It is not yet implimented but in principle, it should create a new
		winHandle, and call its OpenProcPtr got get it going.
		It is called by:
			HandleMenuCommand() which dispatches all menu events.
\*------------------------------------------------------------------------------*/
void DoAppNewCommand ( void ) 
{
	return ;
}


/*------------------------------------------------------------------------------*\
	DoAppNewCommand
 *------------------------------------------------------------------------------*
		This routine handles the File:Open command for the application.
		It calls StandardGetFile() and with the resulting FSSpec, calls
		SendOpenDoc() to force the app to send an ODOC event to itself. 
		It is called by:
			HandleMenuCommand() which handles all menu events.
\*------------------------------------------------------------------------------*/
void DoAppOpenCommand ( void ) 
{
	SFTypeList			myTypes = { 'PICT', 'prof' } ;	
	short				numTypes = 2 ;
	StandardFileReply	theSFReply ;	// Get the file name to open

	StandardGetFile( nil, numTypes, myTypes, &theSFReply ) ;
	
	// did the user cancel?
	if(theSFReply.sfGood)
	{
		// if not then package up the selected file into 
		// an appleevent and send it to ourselves:
		SendOpenDoc(&theSFReply.sfFile) ;
	}
	return ;
}



/*------------------------------------------------------------------------------*\
	DoAboutBoxCommand
 *------------------------------------------------------------------------------*
		This routine handles the Apple:About command for the application.
		It creates a new winHandle, sets its NewProcPtr, and then 
		calls the NewProcPtr to get it going.
		It is called by:
			HandleMenuCommand() which handles all menu events.
\*------------------------------------------------------------------------------*/
void DoAboutBoxCommand ( void ) 
{
	OSErr			err = noErr ;
	winHandle		win;

	// create a new winHandle of the propper type
	err = NewWinHandle( &win, winAllocAbout ) ;
	FailIfErr( err ) ;

	err = CallWinNewProc( win ) ;
	if ( err != noErr )
		DisposeWinHandle( win );

	if ( err == kWasAlreadyOpen )
		err = noErr;
}


/*------------------------------------------------------------------------------*\
	DoProfListCommand
 *------------------------------------------------------------------------------*
		This routine handles the Profile:CS Folder command for the application.
		It creates a new winHandle, sets its OpenProcPtr, and then 
		calls the OpenProcPtr to get it going.
		It is called by:
			HandleMenuCommand() which handles all menu events.
\*------------------------------------------------------------------------------*/
void DoProfListCommand ( void ) 
{
	OSErr			err = noErr ;
	winHandle		win;

	// create a new winHandle of the propper type
	err = NewWinHandle( &win, winAllocProfList ) ;
	FailIfErr( err ) ;

	err = CallWinNewProc( win ) ;
	if ( err != noErr )
		DisposeWinHandle( win );

	if ( err == kWasAlreadyOpen )
		err = noErr;
}


/*------------------------------------------------------------------------------*\
	DoAppQuitCommand
 *------------------------------------------------------------------------------*
		This routine handles the File:Quit command for the application.
		It calls StandardGetFile() and with the resulting FSSpec, calls
		It calls SendQuitApp() to force the app to send an QUIT event to itself. 
		It is called by:
			HandleMenuCommand() which handles all menu events.
\*------------------------------------------------------------------------------*/
void DoAppQuitCommand ( void ) 
{
	SendQuitApp() ;				// send ourselves a quit event
	return ;
}
