// ColorSync routines
//
// David Hayward 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains some useful routines for dealing with profiles
// and picts with embedded profiles. 
// 
// 12/13/94	david	first cut

#include <Resources.h>
#include <Gestalt.h>
#include <QDOffscreen.h>

#include "colorsyncStuff.h"
#include "pictStuff.h"
#include "fileStuff.h"
#include "stringStuff.h"
#include "resourceStuff.h"
#include "myDrawMatchedPict.h"


#define MIN(x,y)	( ((x)<(y)) ? (x) : (y) )
#define MAX(x,y)	( ((x)>(y)) ? (x) : (y) )


/**\
|**| ==============================================================================
|**| PUBLIC GLOBALS
|**| ==============================================================================
\**/
Boolean gCS2_available = false;


/**\
|**| ==============================================================================
|**| PRIVATE GLOBALS
|**| ==============================================================================
\**/
static unsigned long gCount ;


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
pascal void		CountProfilesCommentProc	( short kind, short dataSize, Handle dataHandle ) ;
pascal void		UnflatenProfilesCommentProc	( short kind, short dataSize, Handle dataHandle) ;
pascal OSErr	UnflattenProc				( long command, long *sizePtr, void *dataPtr, void *refConPtr) ;

pascal void		TextProc		( short byteCount, Ptr textAddr, Point numer, Point denom ) ;
pascal void		LineProc		( Point endPoint ) ;
pascal void		RectProc		( GrafVerb verb, Rect *r ) ;
pascal void		RRectProc		( GrafVerb verb, Rect *r, short ovalWidth, short ovalHeight ) ;
pascal void		OvalProc		( GrafVerb verb, Rect *r ) ;
pascal void		ArcProc			( GrafVerb verb, Rect *r, short startAngle, short arcAngle ) ;
pascal void		PolyProc		( GrafVerb verb, PolyHandle poly ) ;
pascal void		RgnProc			( GrafVerb verb, RgnHandle rgn ) ;
pascal void		BitsProc		( BitMap *bitPtr, Rect *srcRect, Rect *dstRect, short mode, RgnHandle maskRgn ) ;
pascal void		TxMeasProc		( short byteCount, Ptr textAddr, Point *numer, Point *denom, FontInfo *info ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	CS2_available
 *------------------------------------------------------------------------------*
 		result	true if colorSync 2.0 or greater is installed, otherwise false.
 	
 	Description:
		Sets the global gCS2_available based on the return value.
\*------------------------------------------------------------------------------*/
Boolean	CS2_available ( void ) 
{
   long version;
	
   if (Gestalt(gestaltColorMatchingVersion, &version) == noErr)
   		if ( version >= gestaltColorSync20 )
 			gCS2_available = true ;
 		
	return gCS2_available ;
}


/*------------------------------------------------------------------------------*\
	OpenProfileFSSpec
 *------------------------------------------------------------------------------*
	given an FSSpec, open the profile and return a CMProfileRef

		spec	(In) A valid FSSpec of a version 1 or 2 profile file.

		prof	(out) If result is noErr, then returns the CMProfileRef of the
				opened profile file, otherwise undefined.
		
 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError OpenProfileFSSpec ( FSSpec spec, CMProfileRef *prof ) 
{	
	CMError					cmerr ;
	CMProfileLocation		profLoc ;
	
	profLoc.locType = cmFileBasedProfile;
	profLoc.u.fileLoc.spec = spec;
	cmerr = CMOpenProfile(prof, &profLoc);
	return cmerr ;	
}



/*------------------------------------------------------------------------------*\
	ReOpenProfileRef
 *------------------------------------------------------------------------------*
	given an exiting CMProfileRef, open a new reference to the profile

		newProf	(out) If result is noErr, then returns a new reference to the
				profile, otherwise undefined.  If the location of the profile
				is of type cmNoProfileBase, then make a copy of the profile
				into a temperary file, and return a reference to it.

		prof	(in) The CMProfileRef to reopen
		
 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError ReOpenProfileRef ( CMProfileRef *newProf, CMProfileRef prof ) 
{	
	CMProfileLocation		profLoc ;
	CMError					cmerr ;
	
	if ( prof==nil )
	{
		*newProf = nil ;
		return noErr ;
	}
	
	// get location of profile
	cmerr = CMGetProfileLocation( prof, &profLoc) ;
	if (cmerr) return cmerr ;
	
	switch (profLoc.locType)
	{
		case  cmNoProfileBase :
			cmerr = CMCopyProfile( newProf, nil, prof) ;
			break ;
			
		case cmFileBasedProfile :
			cmerr = CMOpenProfile( newProf, &profLoc ) ;
			break ;
	
		default :
			cmerr = 1 ; // should have a better error code
			break ;
	}
	return cmerr ;	
}


/*------------------------------------------------------------------------------*\
	GetProfileVersion
 *------------------------------------------------------------------------------*
		prof	(in) the CMProfileRef to examine
		
		vers	(out) if result is noErr, then returns the version field of the
				profile's header,  otherwise undefined.
				
 		result	an error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError GetProfVersion ( CMProfileRef prof, unsigned long *vers)
{
	CMError					cmerr ;
	CMAppleProfileHeader	head ;

	cmerr = CMGetProfileHeader(prof, &head) ;
	if (cmerr) return cmerr ;

	*vers = head.cm2.profileVersion ;
	
	return 0 ;
}


/*------------------------------------------------------------------------------*\
	GetProfileName
 *------------------------------------------------------------------------------*
		prof	(in) The CMProfileRef to examine
		
		name	(out) If result is noErr, then returns the name string of the
				profile reference, otherwise undefined.  If the profile reference
				is nil the string in resource 'STR ' 128 is returned.

 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError GetProfName ( CMProfileRef prof, StringPtr name )
{
	CMError			cmerr=nil ;
	ScriptCode		code ;
	
	if (prof==nil)
		GetStringPtr ( 128, name ) ;
	else
		cmerr = CMGetScriptProfileDescription( prof, name, &code) ;
	
	return cmerr ;
}


/*------------------------------------------------------------------------------*\
	EndProfileComment
 *------------------------------------------------------------------------------*
 	Sends a cmEndProfile picComment to the current port.
\*------------------------------------------------------------------------------*/
void EndProfileComment ( void )
{
	PicComment( cmEndProfile, 0, 0 ) ;
}


/*------------------------------------------------------------------------------*\
	PrependProfileToPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which has an embedded profile at the begining.
 	
		pict		(in) PicHandle to prepend the profile to
		
		pictNew		(out) if result is noErr, then returns a new color
					picture which includes the profile, otherwise undefined.

		prof		(in) a profile reference to  embeded profile.
									
 		result		An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError PrependProfileToPicHandle ( PicHandle pict,
									PicHandle *pictNew,
									CMProfileRef prof )
{
	OSErr					err = noErr ;
	CGrafPtr				savePort;
	GDHandle				saveGDev;
	GWorldPtr				smallOff;
	Rect					pictRect;
	CMAppleProfileHeader	head ;
	unsigned long			vers ;
	unsigned long			flags ;
	
	if (prof==nil) return paramErr ;
	
	// get the profile header
	err = CMGetProfileHeader(prof, &head) ;
	if (err) return err ;
	
	// get the version from profile header
	vers = head.cm2.profileVersion ;

	// set the embeded profile flag bit
	if (vers >= cmCS2ProfileVersion)
	{
		flags = head.cm2.flags ;
		head.cm2.flags |= (1<<cmEmbeddedProfile) ;
		err = CMSetProfileHeader(prof, &head) ;
		if (err) return err ;
	}
	
	// create a temporary gworld
	err = NewSmallGWorld( &smallOff ) ;
	if (err) goto restoreProfFlagAndBail ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	pictRect = (**pict).picFrame ;
	ClipRect( &pictRect ) ;						// Important: set clipRgn
	
	// create a new picture
	*pictNew = OpenPicture( &pictRect );		// start recording
	if (vers == cmCS1ProfileVersion)			// if version 1 profile...
		err = paramErr ;						// sorry, can't embed a CS1 profile
	else if (vers >= cmCS2ProfileVersion)		// if version 2 profile...
		err = NCMUseProfileComment(prof,nil) ;	// embed the CS2 profile
	DrawPicture( pict, &pictRect ) ;			// the original picture
	EndProfileComment();						// be sure to to off the prepended profile
	ClosePicture();								// stop recording

	if (err) KillPicture( *pictNew );			// can't embed a CS1 profile
	
	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;

restoreProfFlagAndBail:
	// restore the original flag just in case
	if (vers >= cmCS2ProfileVersion)
	{
		head.cm2.flags = flags ;
		err = CMSetProfileHeader(prof, &head) ;
	}
	return err ;
}


/*------------------------------------------------------------------------------*\
	ProofPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which is soft-proofed version of the input picture.
 	Create a new picture which soft-proofed according to the embeded profiles (if
 	any) and the supplied profiles of the destination and preview device (i.e.
 	the printer and monitor profiles) 
 	This routine calls myDrawProofedPicture() which is in myDrawMatchedPicture.c.
 	
		pict		(in) PicHandle to ckeck
		
		pictProofed	(out) if result is noErr, then returns a new color
					picture which is a matched version of the input picture,
					otherwise undefined.

		srceProf	(in) a profile reference to use when no embeded profile is 
					specified in the pict.  Typically this would be nil (i.e.
					the system profile).
		
		destProf	(in) a profile reference for the destination device.
					Typically this would be profile for the output devive
		
		prevProf	(in) a profile reference for the preview device. 
					Typically this would be profile for the display devive
							
 		result		An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError ProofPicHandle ( PicHandle pict,
						 PicHandle *pictProofed,
						 CMProfileRef srceProf,
						 CMProfileRef destProf,
						 CMProfileRef prevProf )
{
	OSErr			err = noErr ;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	GWorldPtr		smallOff;
	Rect			pictRect;
	
	pictRect = (**pict).picFrame ;

	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	
	ClipRect( &pictRect );						// Important: set clipRgn
	
	*pictProofed = OpenPicture( &pictRect );	// start recording
	err = myDrawProofedPicture( pict, srceProf, destProf, prevProf, &pictRect ) ;
	ClosePicture();			// stop recording

	if (err)
	{
		DisposeHandle( (Handle)*pictProofed ) ;
		*pictProofed = nil ;
	}
	
	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;

	return err ;
}


/*------------------------------------------------------------------------------*\
	MatchPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which matched according to the embeded profiles (if any)
 	and the supplied profile of the preview device (i.e. the monitor profile) 
 	This routine calls myDrawMatchedPicture() which is in myDrawMatchedPicture.c.
 	
		pict		(in) PicHandle to match
		
		pictMatched	(out) if result is noErr, then returns a new color
					picture which is black wherever the input pict is out of
					gamut, otherwise undefined.

		srceProf	(in) a profile reference to use when no embeded profile is 
					specified in the pict.  Typically this would be nil (i.e.
					the system profile).
		
		prevProf	(in) a profile reference for the preview device. 
					Typically this would be profile for the display devive
							
 		result		An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError MatchPicHandle ( PicHandle pict,
						 PicHandle *pictMatched,
						 CMProfileRef srceProf,
						 CMProfileRef prevProf )
{
	OSErr			err = noErr ;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	GWorldPtr		smallOff;
	Rect			pictRect;
	
	pictRect = (**pict).picFrame ;

	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	
	ClipRect( &pictRect );						// Important: set clipRgn
		
	*pictMatched = OpenPicture( &pictRect );	// start recording
	err = myDrawMatchedPicture( pict, srceProf, prevProf, &pictRect ) ;
	ClosePicture();								// stop recording

	if (err)
	{
		DisposeHandle( (Handle)*pictMatched ) ;
		*pictMatched = nil ;
	}

	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;
	
	return err ;
}


/*------------------------------------------------------------------------------*\
	CheckPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which is black wherever the input picture is out of gamut.
 	This routine calls myDrawCheckedPicture() which is in myDrawMatchedPicture.c.
 	
		pict		(in) PicHandle to ckeck
		
		pictChecked	(out) if result is noErr, then returns a new black & white
					picture which is black wherever the input pict is out of
					gamut, otherwise undefined.

		srceProf	(in) a profile reference to use when no embeded profile is 
					specified in the pict.  Typically this would be nil (i.e.
					the system profile).
		
		destProf	(in) a profile reference for the destination device.
					Typically this would be profile for the output devive
		
 		result		An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError CheckPicHandle ( PicHandle pict,
						 PicHandle *pictChecked,
						 CMProfileRef srceProf,
						 CMProfileRef destProf )
{
	OSErr			err = noErr ;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	GWorldPtr		smallOff;
	Rect			pictRect;
	
	pictRect = (**pict).picFrame ;

	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	
	ClipRect( &pictRect );						// Important: set clipRgn
		
	*pictChecked = OpenPicture( &pictRect );	// start recording
	err = myDrawCheckedPicture( pict, srceProf, destProf, &pictRect ) ;
	ClosePicture();								// stop recording

	if (err)
	{
		DisposeHandle( (Handle)*pictChecked ) ;
		*pictChecked = nil ;
	}

	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;
	
	return err ;
}


/**\
|**| ==============================================================================
|**| CountProfilesInPicHandle ROUTINES
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	CountProfilesInPicHandle
 *------------------------------------------------------------------------------*
 	count the number of profiles embedded in a PicHandle.
 	
		pict	(in) PicHandle to count the profile in
		
		count	(out) if result is noErr, then returns a one-based count of the
				profiles in the PicHandle, otherwise undefined.

 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError CountProfilesInPicHandle ( PicHandle pict, unsigned long *count )
{
	OSErr			err = noErr;
	CQDProcs		procs;
		
	// setup bottleneck for piccomments so we can count profiles
	SetStdCProcs(&procs);
	procs.textProc    = NewQDTextProc   ( TextProc    ) ;
	procs.lineProc    = NewQDLineProc   ( LineProc    ) ;
	procs.rectProc    = NewQDRectProc   ( RectProc    ) ;
	procs.rRectProc   = NewQDRRectProc  ( RRectProc   ) ;
	procs.ovalProc    = NewQDOvalProc   ( OvalProc    ) ;
	procs.arcProc     = NewQDArcProc    ( ArcProc     ) ;
	procs.polyProc    = NewQDPolyProc   ( PolyProc    ) ;
	procs.rgnProc     = NewQDRgnProc    ( RgnProc     ) ;
	procs.bitsProc    = NewQDBitsProc   ( BitsProc    ) ;
	procs.commentProc = NewQDCommentProc( CountProfilesCommentProc ) ;
	procs.txMeasProc  = NewQDTxMeasProc ( TxMeasProc  ) ;

	// initialize global which is incrimented by the commentProc
	gCount = 0 ;
	
	// draw picture to count profiles
	err = DrawPicHandleUsingBottlenecks ( pict, procs, nil ) ;
	
	// get the result from the global
	*count = gCount ;
		
	// clean up and return
	DisposeRoutineDescriptor( procs.textProc  ) ;
	DisposeRoutineDescriptor( procs.lineProc  ) ;
	DisposeRoutineDescriptor( procs.rectProc  ) ;
	DisposeRoutineDescriptor( procs.rRectProc ) ;
	DisposeRoutineDescriptor( procs.ovalProc  ) ;
	DisposeRoutineDescriptor( procs.arcProc   ) ;
	DisposeRoutineDescriptor( procs.polyProc  ) ;
	DisposeRoutineDescriptor( procs.rgnProc   ) ;
	DisposeRoutineDescriptor( procs.bitsProc  ) ;
	DisposeRoutineDescriptor( procs.commentProc ) ;
	DisposeRoutineDescriptor( procs.txMeasProc  ) ;
	return err;
}

pascal void CountProfilesCommentProc (short kind, short dataSize, Handle dataHandle)
{
	long selector ;

	switch (kind)
	{
		case cmBeginProfile:
			gCount ++ ;								// we found a version 1 profile
			break ;

		case cmComment:
			if ( dataSize <= 4 ) break;				// dataSize too small for selector so break
			selector = *((long *)(*dataHandle)) ;	// get the selector from 1st long in data
			if (selector == cmBeginProfileSel )
				gCount ++ ;							// we found a version 2 profile
			break ;
	}		
}


/**\
|**| ==============================================================================
|**| GetIndexedProfileFromPicHandle ROUTINES
|**| ==============================================================================
\**/


static Handle			gChunkBaseHndl = nil ;
static unsigned long	gIndex = 0 ;
static long				gChunkIndex = 0 ;
static long				gChunkCount = 0 ;
static long				gChunkOffset = 0 ;
static long				gChunkSize = 0 ;


/*------------------------------------------------------------------------------*\
	GetIndexedProfileFromPicHandle
 *------------------------------------------------------------------------------*
		pict	(in) PicHandle to extract the profile from
		
		index	(in) the one-based index of the profile in the
				pict to unflatten
				
		prof	(out) returns reference to profile if result is noErr
				Otherwise undefined
				
		profLoc	(in) profile location specification.  If null then a temporary
				profile is created and no disk file will remain after
				CMCloseProfile is called
\*------------------------------------------------------------------------------*/
CMError GetIndexedProfileFromPicHandle ( PicHandle pict,
										 unsigned long index,
										 CMProfileRef *prof,
										 CMProfileLocation *profLoc )
{	
	unsigned long		refCon ;
	CMFlattenUPP		flattenUPP ;
	CMError				cmerr = noErr ;
	Boolean				preferredCMMNotFound ;
	FSSpec				tempSpec ;
	CMProfileRef		tempProf ;
	CMProfileLocation 	tempProfLoc ;
	
	flattenUPP = NewCMFlattenProc(UnflattenProc) ;
	
	// should make sure index <= count 
	
	refCon = (unsigned long) pict ;
	gIndex = index ;
	
	cmerr = CMUnflattenProfile( &tempSpec, 
								flattenUPP, 
								(void*)&refCon, 
								&preferredCMMNotFound ) ;

	DisposeRoutineDescriptor( flattenUPP ) ;

	if (cmerr) return cmerr ;
	
	tempProfLoc.locType = cmFileBasedProfile ;
	tempProfLoc.u.fileLoc.spec = tempSpec ;
		
	cmerr = CMOpenProfile( &tempProf, &tempProfLoc ) ;
	if (cmerr) return cmerr ;

	cmerr = CMCopyProfile( prof, profLoc, tempProf ) ;
	cmerr = CMCloseProfile( tempProf ) ;
	cmerr = FSpDelete( &tempSpec ) ;
	
	return cmerr ;
}

pascal OSErr UnflattenProc (long command, long *sizePtr, void *dataPtr, void *refConPtr)
{
	OSErr 				err = noErr ;
	static CQDProcs		procs ;
	static GWorldPtr	offscreen ;
	PicHandle			pict ;
	
	switch (command)
	{
		case cmOpenReadSpool:		
			err = NewSmallGWorld( &offscreen ) ;
			if (err) return err ;
		
			SetStdCProcs( &procs );
			procs.textProc    = NewQDTextProc   ( TextProc    ) ;
			procs.lineProc    = NewQDLineProc   ( LineProc    ) ;
			procs.rectProc    = NewQDRectProc   ( RectProc    ) ;
			procs.rRectProc   = NewQDRRectProc  ( RRectProc   ) ;
			procs.ovalProc    = NewQDOvalProc   ( OvalProc    ) ;
			procs.arcProc     = NewQDArcProc    ( ArcProc     ) ;
			procs.polyProc    = NewQDPolyProc   ( PolyProc    ) ;
			procs.rgnProc     = NewQDRgnProc    ( RgnProc     ) ;
			procs.bitsProc    = NewQDBitsProc   ( BitsProc    ) ;
			procs.commentProc = NewQDCommentProc( UnflatenProfilesCommentProc ) ;
			procs.txMeasProc  = NewQDTxMeasProc ( TxMeasProc  ) ;
		
			gChunkBaseHndl = nil ;
			gChunkIndex = 0 ;
			gChunkOffset = 0 ;
			gChunkSize = 0 ;
			break ;
			
		case cmReadSpool:
			if (gChunkOffset > gChunkSize)			// if we over read the chunk last time
			{										// this shouldn't happen
				return ioErr ;						// is this the best error to return ??
			}
			if (gChunkOffset == gChunkSize)			// if we used up the last chunk
			{
				if ( gChunkBaseHndl != nil )
				{
					HUnlock( gChunkBaseHndl ) ;		// dispose of the previous chunk
					DisposeHandle( gChunkBaseHndl );
					gChunkBaseHndl = nil ;
				}
				gChunkIndex++ ;						// read in a new chunk
				gChunkOffset = 0 ;
				gCount = 0 ;
				gChunkCount = 0 ;
				pict = *((PicHandle *)refConPtr) ;
				err = DrawPicHandleUsingBottlenecks ( pict, procs, offscreen ) ;
				if ( gChunkBaseHndl==nil)			// check to see if we're over-read
					return ioErr ;					// is this the best error to return ??
				HLock( gChunkBaseHndl ) ;
			}
			if (gChunkOffset < gChunkSize)
			{
				*sizePtr = MIN( gChunkSize-gChunkOffset, *sizePtr ) ;
				BlockMove(	(Ptr)( &((*gChunkBaseHndl)[gChunkOffset]) ),
							(Ptr)dataPtr ,
							*sizePtr ) ;
				gChunkOffset += (*sizePtr) ;
			}
			break ;
			
		case cmCloseSpool:
			if ( gChunkBaseHndl != nil )
			{
				HUnlock( gChunkBaseHndl ) ;			// dispose of the previous chunk
				DisposeHandle( gChunkBaseHndl );
				gChunkBaseHndl = nil ;
			}			
			DisposeGWorld( offscreen ) ;
			DisposeRoutineDescriptor( procs.textProc  ) ;
			DisposeRoutineDescriptor( procs.lineProc  ) ;
			DisposeRoutineDescriptor( procs.rectProc  ) ;
			DisposeRoutineDescriptor( procs.rRectProc ) ;
			DisposeRoutineDescriptor( procs.ovalProc  ) ;
			DisposeRoutineDescriptor( procs.arcProc   ) ;
			DisposeRoutineDescriptor( procs.polyProc  ) ;
			DisposeRoutineDescriptor( procs.rgnProc   ) ;
			DisposeRoutineDescriptor( procs.bitsProc  ) ;
			DisposeRoutineDescriptor( procs.commentProc ) ;
			DisposeRoutineDescriptor( procs.txMeasProc  ) ;
			break ;
			
		default:
			break ;
	}
	return err ;
}

pascal void UnflatenProfilesCommentProc ( short kind, short dataSize, Handle dataHandle)
{
	long		selector ;
	OSErr		err ;

	if (gChunkBaseHndl != nil ) return ;			// the handle is in use. shouldnt happen
	if (gCount > gIndex) return ;					// we allready found the profile

	switch (kind)
	{
		case cmBeginProfile:
			gCount ++ ;								// we found a version 1 profile
			gChunkCount = 1 ;						// v1 profiles should only have 1 chunk
			if (gCount != gIndex) break ;			// not the profile we're looking for
			if (gChunkCount != gChunkIndex) break;	// not the chunk we're looking for
			gChunkBaseHndl = dataHandle ;
			err = HandToHand( &gChunkBaseHndl ) ;
			gChunkSize = dataSize ;
			gChunkOffset = 0 ;
			break ;

		case cmComment:
			if ( dataSize <= 4 ) break;				// dataSize too small for selector so break
			selector = *((long *)(*dataHandle)) ;	// get the selector from 1st long in data
			switch (selector)
			{
				case cmBeginProfileSel :
					gCount ++ ;						// we found a version 2 profile
					gChunkCount = 1 ;
					if (gCount != gIndex) break ;	// not the profile we're looking for
					if (gChunkCount!=gChunkIndex) break;// not the chunk we're looking for
					gChunkBaseHndl = dataHandle ;
					err = HandToHand( &gChunkBaseHndl ) ;
					gChunkSize = dataSize ;
					gChunkOffset = 4 ;
					break ;
					
				case cmContinueProfileSel :
					gChunkCount ++ ;
					if (gCount != gIndex) break ;	// not the profile we're looking for
					if (gChunkCount!=gChunkIndex) break;// not the chunk we're looking for
					gChunkBaseHndl = dataHandle ;
					err = HandToHand( &gChunkBaseHndl ) ;
					gChunkSize = dataSize ;
					gChunkOffset = 4 ;
					break ;
					
				case cmEndProfileSel :
					// check to see if we're over-reading ??
					gChunkCount = 0 ;
					break ;
			}
			break ;
	}		
}


/**\
|**| ==============================================================================
|**| NO-OP BOTTLENECK ROUTINES
|**| ==============================================================================
\**/


static pascal void TextProc ( short byteCount, Ptr textAddr, Point numer, Point denom )
{
}
static pascal void LineProc ( Point endPoint )
{
}
static pascal void RectProc ( GrafVerb verb, Rect *r )
{
}
static pascal void RRectProc ( GrafVerb verb, Rect *r, short ovalWidth, short ovalHeight )
{
}
static pascal void OvalProc ( GrafVerb verb, Rect *r )
{
}
static pascal void ArcProc ( GrafVerb verb, Rect *r, short startAngle, short arcAngle )
{
}
static pascal void PolyProc ( GrafVerb verb, PolyHandle poly )
{
}
static pascal void RgnProc ( GrafVerb verb, RgnHandle rgn )
{
}
static pascal void BitsProc ( BitMap *bitPtr, Rect *srcRect, Rect *dstRect, short mode, RgnHandle maskRgn )
{
}
static pascal void TxMeasProc ( short byteCount, Ptr textAddr, Point *numer, Point *denom, FontInfo *info )
{
}
