
#include <GestaltEqu.h>
#include <Errors.h>
#include <GXGraphics.h>
#include <GXEnvironment.h>
#include <GXExceptions.h>
#include <GXPrinting.h>
#include <GXMath.h>

#include "appMain.h"  		// neded for DoEvent
#include "appMenus.h"  		// neded for edit menu items

#include "win.h"

#include "gxGlobals.h"
#include "gxPrintUtils.h"


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	SetupGXEditRecord
 *------------------------------------------------------------------------------*
		This routine fills in the GXEditMenuRec so that print dialogs can
		do cut, copy, and paste.
		This routine is called by InitGXStuff().
\*------------------------------------------------------------------------------*/
void SetupGXEditRecord ( void )
{
	gQDGXEditMenuRec.editMenuID = mEdit ;
	gQDGXEditMenuRec.cutItem    = iCut ;
	gQDGXEditMenuRec.copyItem   = iCopy ;
	gQDGXEditMenuRec.pasteItem  = iPaste ;
	gQDGXEditMenuRec.clearItem  = iClear ;
	gQDGXEditMenuRec.undoItem   = iUndo ;

}


/*------------------------------------------------------------------------------*\
	AdjustMenusForPrinting
 *------------------------------------------------------------------------------*
		This routine enables the Edit menu items so that print dialogs can
		do cut, copy, and paste.
		This routine is called by the menu handling code of any windows that 
		support Page Setup or Print commands.
\*------------------------------------------------------------------------------*/
void AdjustMenusForPrinting ( void ) 
{
	MenuHandle	theMenu ;

	// if we are printing, disable everything except the edit menu

	dimAllMenus() ;
		
	// do the method menu
	theMenu = GetMHandle ( mEdit ) ;
		EnableItem ( theMenu, 0 );		// 0 ==> the whole menu
		EnableItem ( theMenu, iUndo );
		EnableItem ( theMenu, iCut );
		EnableItem ( theMenu, iCopy );
		EnableItem ( theMenu, iPaste );
		EnableItem ( theMenu, iClear );

	DrawMenuBar() ;
}


/*------------------------------------------------------------------------------*\
	GXInstallApplicationOverride
 *------------------------------------------------------------------------------*
		This routine installs the UnivProcPtr so that print dialogs can
		do cut, copy, and paste events.
		This routine is called by the OpenProcPtr for any window that support
		GX printing.
\*------------------------------------------------------------------------------*/
void InstallGXOverrides( winHandle win ) 
{
	gxJob	theJob = GetWinGXJob( win ) ;
	
	GXInstallApplicationOverride(theJob,
								 gxPrintingEventMsg,
								 gQDGXPrintingEventUPP);
}


/*------------------------------------------------------------------------------*\
	PrintingEventProc
 *------------------------------------------------------------------------------*
		This routine handle update and menu events while a GX movable-modal
		printing dialog box is displayed.  
\*------------------------------------------------------------------------------*/
OSErr PrintingEventProc ( EventRecord *anEvent, Boolean filterEvent )
{
	if (!filterEvent)
	{
		switch (anEvent->what)
		{
			case mouseDown:
			case keyDown:
			case autoKey:
				break;
			default:
				DoEvent(anEvent);
		}
	}
	return(noErr);
}


/*------------------------------------------------------------------------------*\
	DoGXStyleDlog
 *------------------------------------------------------------------------------*
		This routine displays the GX Page Setup dialog (a.k.a. Format dialog)
\*------------------------------------------------------------------------------*/
OSErr 	DoGXStyleDlog ( gxJob theJob )
{
	OSErr			err;
	gxDialogResult	result;
	
	result = GXJobDefaultFormatDialog(theJob, &gQDGXEditMenuRec) ;
	if ( result == gxCancelSelected )
		return userCanceledErr ;
		
	// just check we don't have an error	
	err = GXGetJobError(theJob);
	return err ;
}


/*------------------------------------------------------------------------------*\
	DoGXStyleDlog
 *------------------------------------------------------------------------------*
		This routine displays the GX Print dialog (a.k.a. Job dialog)
\*------------------------------------------------------------------------------*/
OSErr 	DoGXJobDlog ( gxJob	theJob )
{
	OSErr			err;
	gxDialogResult 	result;
	
	result = GXJobPrintDialog( theJob, &gQDGXEditMenuRec ) ;
	if ( result == gxCancelSelected )
		return userCanceledErr ;
	
	// just check we don't have an error	
	err = GXGetJobError(theJob);
	return err ;
}


/*------------------------------------------------------------------------------*\
	MyGXPrintAShape
 *------------------------------------------------------------------------------*
		This is a spool proc for the QD-to-QDGX translator routines. In this 
		routine, we simply draw each shape we're passed (printing it).
		You could also capture each shape into one gxPicture shape, and then do
		one draw of that picture afterwards.
\*------------------------------------------------------------------------------*/
OSErr MyGXPrintAShape ( gxShape currentShape, long refCon )
{
	MySpoolDataPtr		spoolData;
	gxShapeType			theShapeType;

	//	Don't waste time spooling the shape if it's being
	//	drawn completely off the page.
	
	spoolData = (MySpoolDataPtr) refCon;
	theShapeType = GXGetShapeType(currentShape);
	
	if ((theShapeType == gxEmptyType)   || 
		(theShapeType == gxFullType)    || 
		(theShapeType == gxPictureType) ||
		(GXTouchesBoundsShape(&spoolData->pageArea, currentShape)))
	{
		GXSetShapeViewPorts(currentShape, 1, &spoolData->printViewPort);
		GXDrawShape(currentShape);
	}
	
	return (OSErr) GXGetGraphicsError(nil);
}


/*------------------------------------------------------------------------------*\
	DoGXPrintLoop
 *------------------------------------------------------------------------------*
		This is a basic GX print loop for QuickDraw-based applications
\*------------------------------------------------------------------------------*/
OSErr 	DoGXPrintLoop ( winHandle win ) 
{
	OSErr				err = noErr;
	long				firstPage, lastPage, numPages, pg;
	gxViewPort			printViewPort;
	Point				patStretch = {1,1};
	gxFormat			pageFormat;
	Rect				everywhereRect;
	gxJob				docJob;
	MySpoolDataRec		spoolData;
	gxRectangle			pageSize, paperSize ;		// used to query each format for the page size
	Rect				qdPageRect ;
	

	// lock the document handle, we pass the dereferenced handle into many routines here
	MoveHHi( (Handle)win ) ;
	HLock( (Handle)win ) ;

	// Get the document format and start printing.  Because we are tiling
	// a PICT we probably don't want to implement a facility for the user to 
	// select different formats for each page.  In another app (a word processor
	// for example) we may want to do this, in which case the jobFormat field
	// of the document record could be implemented as an array with one 
	// slot in the array for each page in the win, or as a list.  We wanted
	// to keep this simple...

	docJob = GetWinGXJob( win ) ;
	pageFormat = GetWinGXFormat( win ) ;

	if (pageFormat == nil)
		pageFormat = GXGetJobFormat(docJob, 1);

	GXGetFormatDimensions( pageFormat, &pageSize, &paperSize ) ;
	
	// convert the pageSize to a regular rect, we use this to image
	// the document with in the routine DoDrawData
	
	qdPageRect.top = FixedToInt( pageSize.top ) ;
	qdPageRect.left = FixedToInt( pageSize.left ) ;
	qdPageRect.bottom = FixedToInt( pageSize.bottom ) ;
	qdPageRect.right = FixedToInt( pageSize.right ) ;
		
	// work out how many pages in the document
	numPages = CallWinPageCountProc ( win, &qdPageRect);

	//	Determine which pages the user selected to print.  If
	//	the user specifies a page range that is greater than
	//	the actual number of pages in the document, only print
	//	those pages that are actually in the document. 

	GXGetJobPageRange(docJob, &firstPage, &lastPage);
	
	if (lastPage > numPages )
		lastPage = numPages;

	//	Calculate the total number of pages to print, and begin
	//	printing if there are no errors.

	numPages = lastPage - firstPage +1;
	err = GXGetJobError(docJob);
	nrequire(err, PageRangeError);

	GXStartJob(docJob, "\p", numPages);		/// should set the doc title
	err = GXGetJobError(docJob);
	nrequire(err, StartJobFailed);

	//	Create a new viewport for printing, and set our translator rects
	//	to "wide open" so they include all data we're drawing.  For each
	//	page we print, call GXStartPage, draw, and call GXFinishPage.

	SetRect(&everywhereRect, 0, 0, 32767, 32767);
	printViewPort = GXNewViewPort(gxScreenViewDevices);

	for (pg = firstPage; (err == noErr) && (pg <= lastPage); pg++)
	{
		CGrafPtr	docWindow = GetWinWindow( win ) ;

		GXStartPage(docJob,
					pg,
					pageFormat,
					1,
					&printViewPort) ;								

		err = GXGetJobError( docJob ) ;

		//	If there were no errors, set up the Translator, draw the QuickDraw
		//	data for the current page and remove the Translator.
		//	
		//	Note that, although it looks like it takes a GrafPtr,
		//	GXInstallQDTranslator actually requires a pointer to a CGrafPort,
		//	and will crash if you pass it a true GrafPtr.

		nrequire(err, StartPageFailed);
		spoolData.printViewPort = printViewPort;
		GXGetFormatDimensions(pageFormat, &spoolData.pageArea, nil);

		GXInstallQDTranslator(	(GrafPtr)docWindow,
								gxDefaultOptionsTranslation,
								&everywhereRect, &everywhereRect,
								patStretch,
								NewgxShapeSpoolProc(MyGXPrintAShape),
								&spoolData);

		SetPort( (GrafPtr)docWindow );
		
		CallWinPagePrintProc( win, &qdPageRect, (GrafPtr)docWindow, pg );

		GXRemoveQDTranslator(	(GrafPtr)docWindow,
								nil ) ;

		// Finish the page.

		GXFinishPage( docJob );
	}

	// Finish printing.

StartPageFailed:

	GXFinishJob( docJob ) ;
	err = GXGetJobError( docJob ) ;
	GXDisposeViewPort( printViewPort ) ;

StartJobFailed:
PageRangeError:

	HUnlock( (Handle)win ) ;
	return err;
}

