// Simple framework for Macintosh sample code
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the QuickDraw GX printing sample code, we want to have
// GX analogs for all the classic printing functions, and we will have some 
// additional routines in here.  This is pretty much based on the recipes in
// the book Inside Macintosh QuickDraw GX: Programmers Overview
// 
// 10/4/94	david	first cut


#include <Gestalt.h>
#include <CodeFragments.h>
#include <GXTypes.h>
#include <GXGraphics.h>
#include <GXEnvironment.h>
#include <GXPrinting.h>
#include <GXErrors.h>
#include <GraphicsLibraries.h>

#include "gxGlobals.h"
#include "gxUtils.h"
#include "gxPrintUtils.h"

#include "ErrorHandling.h"


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	QDGX_available
 *------------------------------------------------------------------------------*
		This function returns true if QuickDraw GX graphics, typography, and
		printing are available to the application and returns false otherwise.
		It also sets the global gQDGX_available based on the return value.
\*------------------------------------------------------------------------------*/
Boolean	QDGX_available ( void ) 
{
   long version;
	
   if (Gestalt(gestaltGraphicsVersion, &version) == noErr)
	 if (Gestalt(gestaltGXPrintingMgrVersion, &version) == noErr)
 		gQDGX_available = true ;
 	
#ifdef powerc
	// This is a sanity check to see if the PowerPC QuickDrawGXLib is installed.
	// Since we are "weak" linked to QuickDrawGXLib, the Process Manager will
	// launch us even if QuickDrawGXLib is missing.  If it's missing, the
	// Code Fragment Manager will leave the address of the functions we call
	// unresolved, and we would crash if we tried to call that function.  So,
	// we do a check here and should be prepared to exit gracefully if the 
	// library is missing.  This could happen if the user has installed a
	// 68K only version of QuickDraw GX.  It could also happen if the user has
	// taken the QuickDraw GX extension out of the Extensions Folder.  
	//
	// Note - We could check this against any function we call in the library.
	// GXNewGraphicsClient is the first function we call, so it's convenient.

	if ( kUnresolvedSymbolAddress == (long)GXNewGraphicsClient) 
		gQDGX_available = false;	// QuickDraw GX is not available in Power Mac
#endif

// 	gQDGX_available = false ;		// uncomment this line if you want 
									// to debug w/ GX-aware printing OFF
 	return gQDGX_available ;
}


/*------------------------------------------------------------------------------*\
	QDGXDebug_available
 *------------------------------------------------------------------------------*
		This function returns true if the debugging version of QuickDraw GX is 
		available to the application and returns false otherwise.
		It also sets the global gQDGXDebug_available based on the return value.
\*------------------------------------------------------------------------------*/
Boolean	QDGXDebug_available ( void ) 
{
	long flags;
	
	if (Gestalt(gestaltGraphicsAttr, &flags) == noErr)
		if (flags & gestaltGraphicsIsDebugging)
			gQDGXDebug_available = true ;

	return gQDGXDebug_available ;
}


/*------------------------------------------------------------------------------*\
	InitGXStuff
 *------------------------------------------------------------------------------*
		This function initializes all the GX stuff.
		It should be called early in the application.
\*------------------------------------------------------------------------------*/
void InitGXStuff ( void ) 
{
	OSErr		err = noErr ;

	if ( QDGX_available() )
	{
		// set up the graphics client for the application
		gQDGXClient = GXNewGraphicsClient(nil,
											gQDGXGraphicsHeapSize * 1024L,
											(gxClientAttribute)0L);
		// sanity check
		if ( gQDGXClient == nil )
			FatalError( eFatalNoGXClient ) ;


		// If gDebugging = TRUE, you will receive graphics library errors & notices will be posted.
		// This functionality will only work with the "debugging" version of QuickDraw GX.  If you
		// don't have the debugging version installed, these functions will not work. 
	
		if (gQDGXDebugging)
		{
			SetGraphicsLibraryErrors();
			SetGraphicsLibraryNotices();	
		}
	
		// Set gGiveMeValidation to TRUE if you want run-time validation. As you increase the amount
		// of validation, The drawing speed will SLOW down due to all of the internal checking. 
			
		// gxPublicValidation will check parameters to public routines. For additional details
		// regarding the various levels of validation, see "Inside Macintosh: QuickDraw GX
		// Environment and Utilties."
	
		if (gQDGXValidation)
			GXSetValidation( gxPublicValidation ); 

		
		// initialise the gx heap
		GXEnterGraphics() ;
		
		// check we were ok - no errors
		if ( GXGetGraphicsError( nil ) == out_of_memory )
			FatalError( eFatalNoGXClientMemory ) ;
		
		// init printing
		err = GXInitPrinting() ;
		if ( err  ) AlertUser( err ) ;

		// Initialize our printing event override UPP
		gQDGXPrintingEventUPP = NewGXPrintingEventProc(PrintingEventProc);

		SetupGXEditRecord() ;
	}
}


/*------------------------------------------------------------------------------*\
	InitToolbox
 *------------------------------------------------------------------------------*
		This function disposes all the GX stuff.
		It should be called before the application exits.
\*------------------------------------------------------------------------------*/
void DisposeGXStuff ( void ) 
{
	if( gQDGX_available )
	{
		DisposeRoutineDescriptor(gQDGXPrintingEventUPP);
		GXExitPrinting() ;
		GXExitGraphics() ;
		GXDisposeGraphicsClient( gQDGXClient ) ;
	}
}

