
#include "pictStuff.h"

OSErr DoReadPICT ( FSSpec spec, PicHandle *pict ) 
{
	long			fileSize ;
//	PicHandle		pict = nil ;
	short			ref ;
	OSErr			err;
	
	// try to open the file
	
	err = FSpOpenDF( &spec, fsRdPerm, &ref ) ;
	if ( err ) return nil;

	// pict files have a 512 byte header at the front - we dont care about this
	// we can find the size of the pict by subtracting 512 bytes from the length
	// of the file.  We then want to size a handle to that and read the data
	// into the handle.
	
	err = GetEOF( ref, &fileSize ) ;
	if ( err ) goto bail;
	
	err = SetFPos( ref, fsFromStart, 512) ;
	if ( err ) goto bail;

	fileSize -= 512 ;
	
	*pict = (PicHandle)NewHandle( fileSize ) ;
	if( *pict == nil )
	{
		err = MemError() ;
		goto bail;
	}
	
	HLock( (Handle)*pict ) ;
	err = FSRead( ref, &fileSize, (Ptr)**pict ) ;
	HUnlock( (Handle)*pict ) ;
	if( err )
	{
		DisposHandle( (Handle)*pict ) ;
		*pict = nil ;
	}

bail:
	FSClose( ref ) ;
	return err ;	
}


OSErr DoWritePICT ( FSSpec spec, PicHandle pict, OSType creator )
{
	short			refNum, count;
	long			inOutCount;
	unsigned char	header[512];
	OSErr			err;

	for ( count=0; count<512; count++ )
		header[count] = 0x00;
	
	err = FSpCreate( &spec, creator, 'PICT', 0 /*reply.sfScript*/ ) ;
	
	/* this is quick 'n' dirty or there'd be more robust handling here */
	if (err == dupFNErr)
	{
		err = FSpDelete( &spec ) ;
		err = FSpCreate( &spec, creator, 'PICT', 0 /*reply.sfScript*/ ) ;
	}
	
	err = FSpOpenDF( &spec, fsWrPerm, &refNum);

	if ( err == noErr )
	{ 
		inOutCount = 512;
		err = FSWrite( refNum, &inOutCount, header);	/* write the header */
		
		if (err == noErr)								/* don't write if error  */
		{
			inOutCount = GetHandleSize( (Handle)pict ) ;
			HLock((Handle)pict);
			err = FSWrite( refNum, &inOutCount, *pict ) ;
			HUnlock( (Handle)pict );
		}
		FSClose( refNum );								/* close it */
	}
		
	return err ;
}


QDErr NewSmallGWorld ( GWorldPtr *offscreen )
{
	/* 2 x 2 x 1bit should be sufficient.
	/* We only need to watch the StdComment bottleneck proc  */
	Rect rect = { 0, 0, 2, 2 } ;
	return NewGWorld( offscreen, 1, &rect, nil, nil, nil ) ;
}


QDErr NewGWorldClear (	GWorldPtr *offscreen, short depth, const Rect *rect,
						CTabHandle cTable, GDHandle aGDevice, GWorldFlags flags )
{
	QDErr		err ;
	CGrafPtr	savePort;
	GDHandle	saveGDev;

	err = NewGWorld( offscreen, depth, rect, cTable, aGDevice, flags ) ;
	if (err) return err ;
					 
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( *offscreen, nil ) ;
	EraseRect( rect ) ;
	SetGWorld( savePort, saveGDev ) ;
	return err ;
}


OSErr DrawPicHandleUsingBottlenecks ( PicHandle pict, CQDProcs procs, CGrafPtr port )
{
	OSErr			err = noErr ;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	GWorldPtr		offscreen;
	Rect			pictRect;
	
	
	if ( port==nil )
	{
		err = NewSmallGWorld( &offscreen ) ;
		if (err) return err ;
	}
	else
		offscreen = port ;

	
	offscreen->grafProcs = &procs ;			// must be a color port
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( offscreen, nil ) ;

	pictRect = (**pict).picFrame ;
	DrawPicture( pict, &pictRect ) ;
	
	SetGWorld( savePort, saveGDev ) ;
	
	if ( port==nil )
		DisposeGWorld( offscreen ) ;

	return err;
}