// Simple framework for Macintosh sample code
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the classic quickdraw printing sample code
// 
// 10/4/94	nick	first cut


#include <Resources.h>
#include <Errors.h>

#include "qdPrintUtils.h"
#include "ErrorHandling.h"
#include "cursorStuff.h"


static const Str255  resIDStr = "\pPrint Record" ;


/**\
|**| ==============================================================================
|**| PUBLIC GLOBALS
|**| ==============================================================================
\**/
Boolean				gPrGeneral_available = false ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	PrGeneral_available
 *------------------------------------------------------------------------------*
		This function returns true if PrGeneral is available and returns false
		otherwise.  It also sets the global gPrGeneral_available based on the
		return value.
\*------------------------------------------------------------------------------*/
Boolean PrGeneral_available ( void )
{
	TGetRslBlk		getResRec ;
	OSErr			myPrintErr = noErr ;

	getResRec.iOpCode = getRslDataOp;
	
	PrGeneral((Ptr) &getResRec);
	
	myPrintErr = PrError() ;
	PrSetError( noErr ) ;
	
	gPrGeneral_available = (myPrintErr == noErr) ;
	return  gPrGeneral_available;
}


/*------------------------------------------------------------------------------*\
	SetupPrintHdl
 *------------------------------------------------------------------------------*
		This routine returns a default print handle for the current printer driver.
		The printer driver is expected to be closed upon entry, and is therefore
		opened and closed in this routine.
\*------------------------------------------------------------------------------*/
OSErr SetupPHdl(THPrint *hPrint)
{
	OSErr err;
	
	// Create a handle, open the printer driver, set the
	// default values for the handle, and close the driver.
	// Be on the lookout for errors.

	*hPrint = (THPrint) NewHandle(sizeof(TPrint));

	if (*hPrint != nil)
	{
		PrOpen();

		err = PrError();
		
		if (err == fnfErr)	// Here, "file not found" really means "no printer chosen."
			err = noPrinterChosenErr;
		
		if (err) return err;

		PrintDefault(*hPrint);
		err = PrError();

		PrClose();
	}
	else
		err = MemError();
	
	return err;
}


/*------------------------------------------------------------------------------*\
	SavePHdl
 *------------------------------------------------------------------------------*
		This routine saves the passed print handle in the indicated resource
		fork, using a name which identifies it uniquely for the printer driver
		and version being used. If there is already a print record resource with
		this name, it's deleted and replaced.  The printer driver is expected to
		be closed upon entry, and is therefore opened and closed in this routine.
\*------------------------------------------------------------------------------*/
OSErr SavePHdl (THPrint hPrint, short resRefNum)
{
	short		savedRefNum, uniqID;
	OSErr		err = noErr;
	Handle		oldHdl;
	Str255		resIDStr;

	// Save the current resource file refNum, open the printer
	// driver, get our unique name for the passed print handle
	// and close the driver.
	
	savedRefNum = CurResFile();
	UseResFile(resRefNum);
	
	if (!err)
	{
		oldHdl = GetNamedResource(r_printHdlType, resIDStr);

		if (oldHdl)
		{
			RmveResource(oldHdl);
			DisposHandle(oldHdl);
		}

		// Save the new resource with a unique ID and our coded name.
	
		uniqID = UniqueID(r_printHdlType);
		AddResource((Handle) hPrint, r_printHdlType, uniqID, resIDStr);
		WriteResource((Handle) hPrint);
		err = ResError();

		
		// Detach the resource, update the resource file and switch back
		// to the original resource file.
	

		if (!err)
			DetachResource((Handle) hPrint);
	}
	
	UpdateResFile(resRefNum);
	UseResFile(savedRefNum);
	return err;
}


/*------------------------------------------------------------------------------*\
	LoadPHdl
 *------------------------------------------------------------------------------*
		This routine loads a print handle resource which matches the name we
		uniquely encode.  If there is no print record resource in the indicated
		resource file which matches our uniquely encoded name, the function
		returns resNotFound.
		Upon entry, hPrint is expected to be a valid print handle or nil.
		The printer driver is expected to be closed upon entry,and is therefore
		opened and closed in this routine.
\*------------------------------------------------------------------------------*/
OSErr LoadPHdl(THPrint *hPrint, short resRefNum)
{
	short		savedRefNum;
	OSErr		err;
	Handle		newHdl;
	Str255		resIDStr;

	// Save the current resource file refNum, open the printer driver,
	// get the name that we would have encoded this resource with and
	// close the driver.

	savedRefNum = CurResFile();
	
	// Get our resource, if there is one, out of the indicated resource
	// file.  If there isn't one, return resNotFound.  Finally, switch
	// back to the original resource file.

	UseResFile(resRefNum);
	
	newHdl = GetNamedResource(r_printHdlType, resIDStr);

	if (newHdl)
	{

		// Got one.  Detach it from the resource map.  If we previously had
		// a print handle, dispose of that before passing this new one back.

		DetachResource(newHdl);
		if (*hPrint) DisposHandle(*(Handle *) hPrint);
		*hPrint = (THPrint) newHdl;
		err = noErr;
	}
	else
		err = resNotFound;
	
	UseResFile(savedRefNum);
	return err;
}


/*------------------------------------------------------------------------------*\
	DoCreatePrintRecord
 *------------------------------------------------------------------------------*
\*------------------------------------------------------------------------------*/
void DoCreatePrintRecord ( winHandle win )
{
	Boolean			changed = false ;
	char			theState = 0 ;
	short			theResRef ;
	FSSpec 			spec ;
	THPrint			thePrintRec;
	
	spec = GetWinFSSpec( win ) ;
	
	// create a new print record
	theState = HGetState( (Handle)win ) ;
	MoveHHi( (Handle)win ) ;
	HLock( (Handle)win ) ;
	
	FailIfErr( SetupPHdl( &thePrintRec )) ;
	
	SetWinPrintRec ( win, thePrintRec );
	
	theResRef = FSpOpenResFile(&spec, fsRdPerm);
//	theResRef = HOpenResFile( spec.vRefNum, spec.parID, spec.name, fsRdPerm );
	
	if ( theResRef != -1)
	{
	//	THPrint		oldTHP, newTHP ;
	//	oldTHP = GetWinPrintRec( win ) ;
	//	newTHP = LoadPHdl( oldTHP, theResRef ) ;
	//	GetWinPrintRec( win, newTHP ) ;
	//	DisposeHandle(oldTHP) ;
		
		(void)LoadPHdl( &(**win).PrintRec, theResRef ) ; /// this should be changed
														 /// to use Get/SetWinPrintRec()
	}
	
	// check we have a "good" print record
	changed = PrValidate( thePrintRec ) ;
	
	HSetState( (Handle)win, theState ) ;
}


/*------------------------------------------------------------------------------*\
	DoStyleDlog
 *------------------------------------------------------------------------------*
		This routine displays the standard Page Setup dialog
\*------------------------------------------------------------------------------*/
OSErr DoStyleDlog(THPrint hPrint)
{
	OSErr	err = noErr ;
	Boolean	changed = false ;
	
	PrOpen() ;
	
	if((err = PrError()) == noErr )
	{
		changed = PrStlDialog(hPrint);
		err = PrError();
		if (!err && !changed) err = iPrAbort;
	}

	PrClose();
	return err;
}


/*------------------------------------------------------------------------------*\
	DoJobDlog
 *------------------------------------------------------------------------------*
		This routine displays the standard Print dialog
\*------------------------------------------------------------------------------*/
OSErr DoJobDlog (THPrint hPrint)
{
	OSErr	err = noErr ;
	Boolean	changed = false ;
	
	PrOpen() ;
	
	if((err = PrError()) == noErr )
	{
		changed = PrJobDialog(hPrint);
		err = PrError();
		if (!err && !changed) err = iPrAbort;
	}

	PrClose();
	return err;
}


/*------------------------------------------------------------------------------*\
	IdleProc
 *------------------------------------------------------------------------------*
		This routine is a PrIdleProcPtr that spins our cursor and looks for
		command-period presses during printing.
\*------------------------------------------------------------------------------*/
pascal void IdleProc (void)
{
	OSErr		err;
	BumpCursor();
	err = CommandPeriod();
	if (err) PrSetError(err);
}


/*------------------------------------------------------------------------------*\
	CommandPeriod
 *------------------------------------------------------------------------------*
		This routine checks to see if the user has pressed command-period.
		The Printing Manager does this for us, but a routine like this is helpful
		to allow users to cancel in the middle of a rendering routine, rather
		than only Printing Manager operations.
		If there's a command-period in the event queue, the routine returns
		iPrAbort just like the Printing Manager.  Otherwise, it returns noErr
\*------------------------------------------------------------------------------*/
OSErr CommandPeriod (void)
{
	Boolean		cancel = false;
	EventRecord	evtRec;
	
	while (!cancel && (WaitNextEvent(keyDownMask | autoKeyMask, &evtRec, 0, nil)))
		cancel = (evtRec.modifiers & cmdKey) &&
				 ((evtRec.message & charCodeMask) == '.');

	return ((cancel)? iPrAbort: noErr);
}


/*------------------------------------------------------------------------------*\
	DoPrintLoop
 *------------------------------------------------------------------------------*
		This is a basic classic QD print loop
\*------------------------------------------------------------------------------*/
OSErr DoPrintLoop ( winHandle win ) 
{
	OSErr				err = noErr;
	short				firstPage, lastPage, numPages, pg;
	short				copies, numCopies;
	GrafPtr				savedPort ;
	TPPrPort			printingPort ;
	THPrint				thePrintRec = GetWinPrintRec(win) ;
	PrIdleUPP			myIdleProc = NewPrIdleProc(IdleProc) ; 
	Rect				qdPageRect ;
	
	thePrintRec = GetWinPrintRec(win);
	myIdleProc = NewPrIdleProc(IdleProc);
	
	GetPort( &savedPort ) ;
	PrOpen() ;
	err = PrError();
	
	if (err != noErr) goto PROpenFailed ;
	

	numCopies = (**thePrintRec).prJob.iCopies ;
	
	// we don't assign this directly because NewPrIdleProc may move memory,
	// so assign to an intermediate - myIdleProc
	(**thePrintRec).prJob.pIdleProc =  myIdleProc; // install the cursor spinning idle proc

	// convert the pageSize to a regular rect, we use this to image
	// the document with in the routine DoDrawData
	
	qdPageRect = (**thePrintRec).prInfo.rPage;
		
	// work out how many pages in the document
	numPages = CallWinPageCountProc ( win, &qdPageRect);
	
	//	Determine which pages the user selected to print.  If
	//	the user specifies a page range that is greater than
	//	the actual number of pages in the document, only print
	//	those pages that are actually in the document. 

	firstPage = (**thePrintRec).prJob.iFstPage ;
	lastPage = (**thePrintRec).prJob.iLstPage;
	
	if (lastPage > numPages )
		lastPage = numPages;


	for( copies = 1; copies <= numCopies ; copies++ )
	{
	
		printingPort = PrOpenDoc( thePrintRec, nil, nil ) ;
		SetPort( (GrafPtr)printingPort ) ;
		err = PrError();
		
		if (err != noErr) goto PROpenFailed;
		
		for ( pg = firstPage; pg <= lastPage; pg++ )
		{
		
			PrOpenPage( printingPort, nil ) ;
			err = PrError();
			if ( err == noErr )
			{
				// image the page
				CallWinPagePrintProc ( win, &qdPageRect, (GrafPtr)printingPort, pg );
				PrClosePage(printingPort) ;
			}
		
		}
		err = PrError();
		if ( err == noErr )
		{
			TPrStatus 		theStatus ;

			PrCloseDoc(printingPort) ;
			err = PrError() ;

			// if we're printing to our good friend the ImageWriter, or similar, despool
			if( (err == noErr) && ((**thePrintRec).prJob.bJDocLoop == bSpoolLoop))
				PrPicFile( thePrintRec, nil, nil, nil, &theStatus ) ; 
		}
		
	}
	
PrintCopyFailed:

	PrClose() ;
	
PROpenFailed:

	DisposeRoutineDescriptor ((UniversalProcPtr)myIdleProc) ;
	SetPort( savedPort ) ;
	return err ;
	
}
