#include <TextEdit.h>
#include <Errors.h>
#include <TextUtils.h>
#include <Resources.h>
#include <Script.h>

#include "stringStuff.h"


void StringToString ( StringPtr src, StringPtr dest )
{
	BlockMove(src, dest, src[0]+1) ;
}


void OSTypeToString ( OSType type, StringPtr dest )
{
	dest[0] = 6 ;
	dest[1] = dest[6] = '\'' ;
	BlockMove(&type, &dest[2], 4);
}


void FSSpecToString ( FSSpec spec, StringPtr dest, Boolean appendName )
{
	CInfoPBRec	pb;
	Str255		dirName;
	OSErr		err;

	dest[0] = 0;									// initialize full pathname
	
	if (appendName)
		pStrCat(dest, spec.name, 255);

	pb.dirInfo.ioNamePtr = dirName;
	pb.dirInfo.ioVRefNum = spec.vRefNum;			// indicat target volume
	pb.dirInfo.ioDrParID = spec.parID;				// initialize parent directory ID
	pb.dirInfo.ioFDirIndex = -1;					// get info about a directory

	do
	{
		pb.dirInfo.ioDrDirID = pb.dirInfo.ioDrParID;
		err = PBGetCatInfo(&pb, false);				// needs error cheching...
		pStrCat(dirName, "\p:", 255);
		pStrIns(dest, dirName, 255);				// assume fullPathName is a Str255
	} while (pb.dirInfo.ioDrDirID != fsRtDirID);
}


void LongHexToString ( long num, StringPtr dest)
{
	long  n ;
	short i ;
	
	dest[0] = 10 ;
	dest[1] = '0' ;
	dest[2] = 'x' ;
	
	for (i=0; i<8; i++)
	{
		n = ( num & 15 ) ;
		dest[10-i] = (n<=9) ? ('0'+n) : ('A'+n-10) ;
		num >>= 4 ;
	}
}


void VersionToString ( long vers, StringPtr dest)
{
	register unsigned char len = 0 ;
	register unsigned char digit ;
	
	digit = (vers & 0xF0000000) >> 28 ;
	if (digit)
		dest[ ++len ] = '0' + digit ;
	
	digit = (vers & 0x0F000000) >> 24 ;
	dest[ ++len ] = '0' + digit ;
	
	dest[ ++len ] = '.' ;
	
	digit = (vers & 0x00F00000) >> 20 ;
	dest[ ++len ] = '0' + digit ;
	
	
	digit = (vers & 0x000F0000) >> 16 ;
	if (digit)
	{
		dest[ ++len ] = '.' ;
		dest[ ++len ] = '0' + digit ;
	}
	
	digit = (vers & 0x0000FF00) >> 8 ; // this doesn look right
	switch (digit)
	{
		case 0x20: dest[ ++len ] = 'd'; break;
		case 0x40: dest[ ++len ] = 'a'; break;
		case 0x60: dest[ ++len ] = 'b'; break;
	}
	
	digit = (vers & 0x000000F0) >> 4 ;
	if (digit)
		dest[ ++len ] = '0' + digit ;
		
	digit = (vers & 0x0000000F) >> 0 ;
	if (digit)
		dest[ ++len ] = '0' + digit ;
	
	dest[0] = len;
}


short MyReplaceText ( Handle dest,  Str15 find, StringPtr rplc )
{
	Size	rplcLen ;
	Handle	rplcHandle ;
	short	returnVal ;
	OSErr	err ;
	
	rplcLen = rplc[0];
	err = PtrToHand( &rplc[1], &rplcHandle, rplcLen );
	if (err) return err;
	
	returnVal = ReplaceText(dest, rplcHandle, find ) ;
	DisposeHandle( rplcHandle ) ;
	return returnVal ;
}



//////////////////////


void pStrCat (StringPtr dst, StringPtr src, unsigned char size)
{
	if (*dst + *src > size)								// make sure were not too big
		*src = size - *dst;
	BlockMove(src + 1, dst + *dst + 1, *src);			// copy string in
	*dst += *src;										// adjust length byte
}


void pStrIns (StringPtr dst, StringPtr src, unsigned char size)
{
	if (*dst + *src > size)								// make sure were not too big
		*dst = size - *src;
	BlockMove(dst + 1, dst + *src + 1, *dst);			// make room for new string
	BlockMove(src + 1, dst + 1, *src);					// copy new string in
	*dst += *src;										// adjust length byte
}


//////////////////////


void myTETextBox ( Ptr text, long length, Rect *box, short justType)
{
	TEHandle hTE ;
	
	hTE = TENew( box, box );
	
	TESetJust( justType, hTE ) ;
	TESetText( text, length, hTE ) ;
	
	(**hTE).txMode = (qd.thePort)->txMode ;
	
	TEUpdate( box, hTE ) ;
	TEDispose(hTE);
}




OSErr FormatExtd80 ( extended80 *x, short formatID, StringPtr dest )
{
	NumberParts			parts;
	NumFormatString		**format;
	Intl0Hndl			itl4;
	long				offset, length;
	FormatStatus		status;

	/* Just in case! */
	Length(dest) = 0;

	/* Get default Number Parts resource/offset/length */
	GetIntlResourceTable( iuCurrentScript, iuNumberPartsTable,
							(Handle*)&itl4, &offset, &length );
    if ( itl4==nil ) return paramErr;

	/* Retrieve default Number Parts Table */
	parts = *( NumberPartsPtr )( *((Handle)itl4) + offset );

	/* Retrieve canonical format */
	format = ( NumFormatString ** )GetResource( 'FMAT', formatID );
	if ( format==nil ) return resNotFound;

	/* Lock it */
	HLock(( Handle )format );

	/* Format it! */
	status = ExtendedToString( x, *format, &parts, dest );

	/* Unlock and release the canonical format */
	HUnlock( (Handle)format ) ;
	ReleaseResource( (Handle)format ) ; 
	
	return status;
}


	typedef struct xtnd80
	{
		short							exp;
		short							man[4];
	} xtnd80 ;


OSErr FormatLong ( long num, short formatID, StringPtr dest)
{
	xtnd80		x80 = {0,{0,0,0,0}} ;
	x80.man[0] = HiWord(num);
	x80.man[1] = LoWord(num);
	x80.exp = 0x4000 + 32 - 2 ;		// 16414
	return FormatExtd80( (extended80*)&x80, formatID, dest ) ;
}


OSErr FormatFixed ( Fixed num, short formatID, StringPtr dest)
{
	xtnd80		x80 = {0,{0,0,0,0}} ;
	x80.man[0] = HiWord(num);
	x80.man[1] = LoWord(num);
	x80.exp = 0x4000 + 16 - 2 ;		// 16398
	return FormatExtd80( (extended80*)&x80, formatID, dest ) ;
}


OSErr FormatSmallFract ( short num, short formatID, StringPtr dest)
{
	xtnd80		x80 = {0,{0,0,0,0}} ;
	x80.man[0] = num;
	x80.exp = 0x4000 + 1 - 2 ;		// 16384
	return FormatExtd80( (extended80*)&x80, formatID, dest ) ;
}


