// Simple framework for Macintosh sample code
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// Access routines for the winHandle structure that is used this
// simple application framework
// 
// 9/2/94	nick	first cut
// 10/4/94	nick	add field for the doc FSSpec
// 11/16/94	david	in the DocumentRecord structure
//						added fields dPicHandle, dPictPalette, dPictCTab
//						changed field dDocFsSp to dFSSpec
//						changed field dPageFormat to dGXFormat
//						changed field dDocumentJob to dGXJob
//					added several getters/setters routines
//					changed some code to use the new getters/setters
//					changed the #define SetWinumentHandleForWindowRefcon to a function SetWindowDocumentHandle
//					changed the #define GetWinumentHandleForWindowRefcon to a function GetWindowDocumentHandle
//					changed varaible names from "theDoc" to "doc" for brevity
//					added some lines to DisposeDocumentHandle so doc's palette and color table are disposed
// 3/10/95	david	fixed bugs in FindWinHandle


#include <Palettes.h>
#include <Files.h>

#include "win.h"

#include "ErrorHandling.h"
#include "fileStuff.h"

#include "gxGlobals.h"
#include "gxPrintUtils.h"


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	GetWindowWinHandle
 *------------------------------------------------------------------------------*
		This routine gets the winHandle from the refcon field of a window
		and returns it.
\*------------------------------------------------------------------------------*/
winHandle GetWindowWinHandle ( WindowPtr w )
{
	winHandle	win;
	short		kind;
	
	if (w==nil)
		return nil ;

	kind = ((WindowPeek)w)->windowKind;
	if ((kind < userKind) && (kind != dialogKind))
		return nil ;

	win = (winHandle) GetWRefCon( w ) ;
	
	if(	win != nil )				// validate - check for nil
	{
		// validate - check for the document creator 
		// as the first long of the struct
		if( GetWinCreator(win) != kDocumentCreator )
			win = nil;
	}
	return win ;					// assert: this will either be valid or nil
}


/*------------------------------------------------------------------------------*\
	GetWindowWinHandle
 *------------------------------------------------------------------------------*
		This routine puts the winHandle into the refcon field of a window.
\*------------------------------------------------------------------------------*/
void SetWindowWinHandle ( WindowPtr w, winHandle win )
{	SetWRefCon( w, (long)win );
}


/*------------------------------------------------------------------------------*\
	FindWinHandle
 *------------------------------------------------------------------------------*
		This routine searches through the active window list to locate those that
		have winHandle's with a given FSSpec and/or type.  It can be used either 
		to count the number of matches or to return the matches in the result buffer.
		The logic is basically the following:
		  if result buffer is nil
			return num of winHandles that match the specPtr and/or type parameters
			the index and count fields are ignored
		  else
			copy into result buffer up to count winHandles
			that match the specPtr and/or type parameters
			starting with the index-th match
			return the the number of winHandles copied into the buffer
\*------------------------------------------------------------------------------*/
short	FindWinHandle ( FSSpec	*specPtr,
						OSType type,
						short index,
						short count,
						winHandle *result )
{
	WindowPeek		wp;
	FSSpec			docSpec ;
	winHandle		win = nil;
	short			n = 0;
	short			stopCount, foundCount;
	Boolean			foundOne, sameSpec, sameType;
	
	stopCount = count ;
	foundCount = 0 ;

	wp = (WindowPeek)FrontWindow();

	if (wp==nil) return 0 ;		// no window are open

	do {
		foundOne = false;		
		
		win = GetWindowWinHandle( (WindowPtr)wp );
		// if (win==nill) skip to next

		if ( (specPtr==nil) && (type==0L) )
			foundOne = true;
		else
		{
			sameType = ( type==GetWinType(win) );

			if ( (specPtr==nil) && sameType )
				foundOne = true;
			else
			{
				docSpec = GetWinFSSpec( win ) ;
				sameSpec = SameFSSpec( &docSpec, specPtr);
				if ( sameSpec && (sameType || (type==0L))  )
					foundOne = true;
			}
		}

		if (foundOne)
		{
			if  ( result==nil )
				foundCount++ ;
			else
			{
				n++;
				if (n>=index)
				{
					foundCount++ ;
					result[foundCount-1] = win;
				}
			}
		}
		wp =  wp->nextWindow;

	} while ( (wp!=nil)  &&  ((foundCount<stopCount) || (result==nil)) );
	
	return foundCount;
}


/*------------------------------------------------------------------------------*\
	NewWinHandle
 *------------------------------------------------------------------------------*
		This routine allocates a new winRecord, sets record's AllocProc, and
		then calls that AllocProc to do the rest of the work.
\*------------------------------------------------------------------------------*/
OSErr	NewWinHandle ( winHandle *win, AllocProcPtr allocProc )
{
	OSErr		err = noErr ;
	
	*win = (winHandle)NewHandleClear( sizeof(winRecord) ) ;	
	if( *win == nil )
	{
		err = MemError() ;
		if (err==noErr)  err=memFullErr;	// should only need this when memerror is noErr
		return err ;
	}

	// init the creator field
	SetWinCreator( *win, kDocumentCreator ) ;

	// allocate any custom data for the win
	SetWinAllocProc( *win, allocProc );
	err = CallWinAllocProc( *win );
	if ( err != noErr)
		DisposeHandle( (Handle)(*win) );
		
	return err ;
}


/*------------------------------------------------------------------------------*\
	NewWinHandle
 *------------------------------------------------------------------------------*
		This routine calls the winHandle's DisposePrce, disposes of the structures
		associated with the winHandle, and lastly, disposed of the winHandle 
		itself.
\*------------------------------------------------------------------------------*/
OSErr	DisposeWinHandle ( winHandle win )
{
	OSErr	err = noErr;
	
	// deallocate the fields

	// dispose of any custom data for the win
	CallWinDisposeProc( win );

	// dispose of the window we used
	if ( GetWinWindow(win) != nil )
	{
		DisposeWindow( (WindowPtr)GetWinWindow(win) );
		err =  QDError() ;
	}

	// get rid of the memory allocated for the document job
	if ( gQDGX_available ) 
	{
		GXDisposeJob( GetWinGXJob(win) );
	//	GXDisposeFormat( GetWinGXFormat(win) );		// for some reason, this crashes
	}

	// get rid of the memory allocated for the print record
	if ( GetWinPrintRec(win) != nil )
	{
		DisposeHandle((Handle)GetWinPrintRec(win)) ;
		err =  MemError() ;
	}
	
	// get rid of the memory for the document handle  - DO THIS LAST
	if ( err == noErr )
	{
		DisposeHandle( (Handle)win );
		err =  MemError() ;
	}
	
	return	err ;
}


/*------------------------------------------------------------------------------*\
	GetWinXxxx / SetWinXxxx
 *------------------------------------------------------------------------------*
		These routines get/set the data fields of a winHandle.
\*------------------------------------------------------------------------------*/


OSType		GetWinCreator ( winHandle win )
{	return (**win).Creator ;
}
void		SetWinCreator ( winHandle win, OSType creator )
{	(**win).Creator = creator ;
}


OSType		GetWinType ( winHandle win )
{	return (**win).Type ;
}
void		SetWinType ( winHandle win, OSType type )
{	(**win).Type = type ;
}


FSSpec		GetWinFSSpec ( winHandle win )
{	return (**win).FSSpec ;
}
void		SetWinFSSpec ( winHandle win, FSSpec *theFSSpec )
{	(**win).FSSpec = *theFSSpec ;
}


CGrafPtr	GetWinWindow ( winHandle win )
{	return (**win).Window ;
}
void		SetWinWindow ( winHandle win, CGrafPtr theWindow )
{	(**win).Window = theWindow ;
}


Rect		GetWinRect ( winHandle win )
{	return (**win).Rect ;
}
void		SetWinRect ( winHandle win, Rect theRect )
{	(**win).Rect = theRect ;
}


Rect		GetWinSizeRect ( winHandle win )
{	return (**win).SizeRect ;
}
void		SetWinSizeRect ( winHandle win, Rect theRect )
{	(**win).SizeRect = theRect ;
}


THPrint		GetWinPrintRec ( winHandle win )
{	return (**win).PrintRec ;
}
void	SetWinPrintRec ( winHandle win, THPrint thePrintRec )
{	(**win).PrintRec = thePrintRec ;
}


Boolean		GetWinDirty ( winHandle win )
{	return (**win).Dirty ;
}
void		SetWinDirty ( winHandle win, Boolean dirty )
{	(**win).Dirty = dirty ;
}


gxJob		GetWinGXJob ( winHandle win )
{	return (**win).GXJob ;
}
void		SetWinGXJob ( winHandle win, gxJob theGXJob )
{	(**win).GXJob = theGXJob ;
}


gxFormat	GetWinGXFormat ( winHandle win )
{	return (**win).GXFormat ;
}
void		SetWinGXFormat ( winHandle win, gxFormat theGXFormat )
{	(**win).GXFormat = theGXFormat ;
}


Handle		GetWinData ( winHandle win )
{	return (**win).Data ;
}
void		SetWinData ( winHandle win, Handle h )
{	(**win).Data = h ;
}


void	CallWinActivateProc (winHandle win, Boolean b)
{	if ( ((**win).ActivateProc) != nil)
		((**win).ActivateProc)(win, b);
}
void	SetWinActivateProc (winHandle win, ActivateProcPtr f)
{	(**win).ActivateProc = f;
}


void	CallWinUpdateProc (winHandle win, EventRecord *e)
{	if ( ((**win).UpdateProc) != nil)
		((**win).UpdateProc)(win, e);
}
void	SetWinUpdateProc (winHandle win, UpdateProcPtr f)
{	(**win).UpdateProc = f;
}


void	CallWinClickProc (winHandle win, EventRecord *e)
{	if ( ((**win).ClickProc) != nil)
		((**win).ClickProc)(win, e);
}
void	SetWinClickProc (winHandle win, ClickProcPtr f)
{	(**win).ClickProc = f;
}


void	CallWinKeyProc (winHandle win, EventRecord *e)
{	if ( ((**win).KeyProc) != nil)
		((**win).KeyProc)(win, e);
}
void	SetWinKeyProc (winHandle win, KeyProcPtr f)
{	(**win).KeyProc = f;
}


void	CallWinResumeProc (winHandle win, Boolean b)
{	if ( ((**win).ResumeProc) != nil)
		((**win).ResumeProc)(win, b);
}
void	SetWinResumeProc (winHandle win, ResumeProcPtr f)
{	(**win).ResumeProc = f;
}


void	CallWinNullProc (winHandle win, EventRecord *e)
{	if ( ((**win).NullProc) != nil)
		((**win).NullProc)(win, e);
}
void	SetWinNullProc (winHandle win, NullProcPtr f)
{	(**win).NullProc = f;
}


void		CallWinMenuProc (winHandle win, long m)
{	if ( ((**win).MenuProc) != nil)
		((**win).MenuProc)(win, m);
}
void		SetWinMenuProc (winHandle win, MenuProcPtr f)
{	(**win).MenuProc = f;
}


void		CallWinResizeProc (winHandle win)
{	if ( ((**win).ResizeProc) != nil)
		((**win).ResizeProc)(win);
}
void		SetWinResizeProc (winHandle win, ResizeProcPtr f)
{	(**win).ResizeProc = f;
}


OSErr		CallWinAllocProc (winHandle win )
{	if ( ((**win).AllocProc) != nil)
		return ((**win).AllocProc)(win);
	else return eFatalNoProcPtr ;
}
void		SetWinAllocProc (winHandle win, AllocProcPtr f)
{	(**win).AllocProc = f;
}


OSErr		CallWinNewProc (winHandle win )
{	if ( ((**win).NewProc) != nil)
		return ((**win).NewProc)(win);
	else return eFatalNoProcPtr ;
}
void		SetWinNewProc (winHandle win, NewProcPtr f)
{	(**win).NewProc = f;
}


OSErr		CallWinOpenProc (winHandle win )
{	if ( ((**win).OpenProc) != nil)
		return((**win).OpenProc)(win);
	else return eFatalNoProcPtr ;
}
void		SetWinOpenProc (winHandle win, OpenProcPtr f)
{	(**win).OpenProc = f;
}


void		CallWinCloseProc (winHandle win)
{	if ( ((**win).CloseProc) != nil)
		((**win).CloseProc)(win);
}
void		SetWinCloseProc (winHandle win, CloseProcPtr f)
{	(**win).CloseProc = f;
}


void		CallWinDisposeProc	( winHandle win )
{	if ( ((**win).DisposeProc) != nil)
		((**win).DisposeProc)(win);
}
void		SetWinDisposeProc	( winHandle win, DisposeProcPtr f )
{	(**win).DisposeProc = f;
}


short		CallWinPageCountProc (winHandle win, Rect *page)
{	if ( ((**win).PageCountProc) != nil)
		return ((**win).PageCountProc)(win,page);
	else return 0;
}
void		SetWinPageCountProc (winHandle win, PageCountProcPtr f)
{	(**win).PageCountProc = f;
}


void		CallWinPagePrintProc (winHandle win, Rect *page, GrafPtr imagingPort, short pageNum)
{	if ( ((**win).PagePrintProc) != nil)
		((**win).PagePrintProc)(win,page,imagingPort,pageNum);
}
void		SetWinPagePrintProc (winHandle win, PagePrintProcPtr f)
{	(**win).PagePrintProc = f;
}

