
#ifndef _WIN_
#define _WIN_


#ifndef __PRINTING__
#include <Printing.h>
#endif

#ifndef __GXPRINTING__
#include <GXPrinting.h>
#endif


/**\
|**| ==============================================================================
|**| PUBLIC DEFINES
|**| ==============================================================================
\**/
#define kDocumentCreator	'ASDF'
#define kWasAlreadyOpen		128


/**\
|**| ==============================================================================
|**| PUBLIC TYPEDEFS
|**| ==============================================================================
\**/
typedef	struct	_winRecord  winRecord, *winPtr, **winHandle ;

/*------------------------------------------------------------------------------*\
	typedefs for the procptrs used by the framework.
\*------------------------------------------------------------------------------*/
typedef	void  (*ActivateProcPtr)    ( winHandle win, Boolean b) ;
typedef	void  (*UpdateProcPtr)      ( winHandle win, EventRecord *e) ;
typedef	void  (*ClickProcPtr)       ( winHandle win, EventRecord *e) ;
typedef	void  (*KeyProcPtr)         ( winHandle win, EventRecord *e) ;
typedef	void  (*ResumeProcPtr)      ( winHandle win, Boolean b) ;
typedef	void  (*NullProcPtr)        ( winHandle win, EventRecord *e) ; 
typedef	void  (*MenuProcPtr)        ( winHandle win, long m) ;
typedef	void  (*ResizeProcPtr)      ( winHandle win ) ;
typedef	OSErr (*AllocProcPtr)		( winHandle win ) ; 
typedef	OSErr (*NewProcPtr)         ( winHandle win ) ; 
typedef	OSErr (*OpenProcPtr)        ( winHandle win ) ; 
typedef	void  (*CloseProcPtr)       ( winHandle win ) ; 
typedef	void  (*DisposeProcPtr)		( winHandle win ) ; 
typedef	short (*PageCountProcPtr)   ( winHandle win, Rect *pageRect ) ; 
typedef	void  (*PagePrintProcPtr)   ( winHandle win, Rect *pageRect, GrafPtr imagingPort, short pageNum ) ;

/*------------------------------------------------------------------------------*\
	winRecord structure used by the framework.
\*------------------------------------------------------------------------------*/
struct	_winRecord {
// type & creator of the windows
	OSType				Creator ;			// always 'ASDF' - helps to verify that it is our window
	OSType				Type ;				// this is the type of window (eg 'Pict' 'Abou', etc.)

// pocedure pointers
	ActivateProcPtr		ActivateProc;
	UpdateProcPtr		UpdateProc;
	ClickProcPtr		ClickProc;
	KeyProcPtr			KeyProc;
	ResumeProcPtr		ResumeProc;
	NullProcPtr			NullProc;
	MenuProcPtr			MenuProc;
	ResizeProcPtr		ResizeProc;
	AllocProcPtr		AllocProc;
	NewProcPtr			NewProc;
	OpenProcPtr			OpenProc;
	CloseProcPtr		CloseProc;
	DisposeProcPtr		DisposeProc;
	PageCountProcPtr	PageCountProc;
	PagePrintProcPtr	PagePrintProc;

// basic necessities
	FSSpec				FSSpec ;			// file spec for the win, we don't keep it open
	CGrafPtr			Window ;			// the graf port/window for the document
	Rect				Rect ;				// the bounds of the document
	Rect				SizeRect ;			// the max/min size of the document
	Boolean				Dirty ; 			// true if the file needs to be saved
	
// print data record (classic & GX)
	THPrint				PrintRec ;			// a print record for the document
	gxJob				GXJob ;			// stores a reference to the win's job format
	gxFormat			GXFormat ;			// the format for the pict

//content data
	Handle				Data;				// handle to whatever data this document needs

} ;



/**\
|**| ==============================================================================
|**| PUBLIC FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
OSErr			NewWinHandle		( winHandle *win, AllocProcPtr allocProc ) ;
OSErr			DisposeWinHandle	( winHandle win ) ;

winHandle		GetWindowWinHandle	( WindowPtr w ) ;
void			SetWindowWinHandle	( WindowPtr w, winHandle win ) ;

short			FindWinHandle		( FSSpec	*specPtr,
									  OSType type,
									  short index,
									  short count,
									  winHandle *result ) ;

static			Boolean	SameFSSpec	( FSSpec *fs1, FSSpec *fs2 ) ;

OSType		GetWinCreator		( winHandle win ) ;
void		SetWinCreator		( winHandle win, OSType creator ) ;

OSType		GetWinType			( winHandle win ) ;
void		SetWinType			( winHandle win, OSType type ) ;

FSSpec		GetWinFSSpec		( winHandle win ) ;
void		SetWinFSSpec		( winHandle win, FSSpec *theFSSpec ) ; 

CGrafPtr	GetWinWindow		( winHandle win ) ;
void		SetWinWindow		( winHandle win, CGrafPtr theWindow ) ;

Rect		GetWinRect			( winHandle win ) ;
void		SetWinRect			( winHandle win, Rect theRect ) ;

Rect		GetWinSizeRect		( winHandle win ) ;
void		SetWinSizeRect		( winHandle win, Rect theRect ) ;

Boolean		GetWinDirty			( winHandle win ) ;
void		SetWinDirty			( winHandle win, Boolean dirty ) ;

THPrint		GetWinPrintRec		( winHandle win ) ;
void		SetWinPrintRec		( winHandle win, THPrint thePrintRec ) ;

gxJob		GetWinGXJob			( winHandle win ) ;
void		SetWinGXJob			( winHandle win, gxJob theGXJob ) ;

gxFormat	GetWinGXFormat		( winHandle win ) ;
void		SetWinGXFormat		( winHandle win, gxFormat theGXFormat ) ;

Handle		GetWinData			( winHandle win ) ;
void		SetWinData			( winHandle win, Handle h ) ;


void		CallWinActivateProc	( winHandle win, Boolean b ) ;
void		SetWinActivateProc	( winHandle win, ActivateProcPtr f ) ;

void		CallWinUpdateProc	( winHandle win, EventRecord *e ) ;
void		SetWinUpdateProc	( winHandle win, UpdateProcPtr f ) ;

void		CallWinClickProc	( winHandle win, EventRecord *e ) ;
void		SetWinClickProc		( winHandle win, ClickProcPtr f ) ;

void		CallWinKeyProc		( winHandle win, EventRecord *e ) ;
void		SetWinKeyProc		( winHandle win, KeyProcPtr f ) ;

void		CallWinResumeProc	( winHandle win, Boolean b ) ;
void		SetWinResumeProc	( winHandle win, ResumeProcPtr f ) ;

void		CallWinNullProc		( winHandle win, EventRecord *e ) ;
void		SetWinNullProc		( winHandle win, NullProcPtr f ) ;

void		CallWinMenuProc		( winHandle win, long m ) ;
void		SetWinMenuProc		( winHandle win, MenuProcPtr f ) ;

void		CallWinResizeProc	( winHandle win ) ;
void		SetWinResizeProc	( winHandle win, ResizeProcPtr f ) ;

OSErr		CallWinAllocProc	( winHandle win ) ;
void		SetWinAllocProc		( winHandle win, AllocProcPtr f ) ;

OSErr		CallWinNewProc		( winHandle win ) ;
void		SetWinNewProc		( winHandle win, NewProcPtr f ) ;

OSErr		CallWinOpenProc		( winHandle win ) ;
void		SetWinOpenProc		( winHandle win, OpenProcPtr f ) ;

void		CallWinCloseProc	( winHandle win ) ;
void		SetWinCloseProc		( winHandle win, CloseProcPtr f ) ;

void		CallWinDisposeProc	( winHandle win ) ;
void		SetWinDisposeProc	( winHandle win, DisposeProcPtr f ) ;


short		CallWinPageCountProc	( winHandle win, Rect *pageRect ) ;
void		SetWinPageCountProc		( winHandle win, PageCountProcPtr f ) ;

void		CallWinPagePrintProc	( winHandle win, Rect *pageRect, GrafPtr imagingPort, short pageNum ) ;
void		SetWinPagePrintProc		( winHandle win, PagePrintProcPtr f ) ;


#endif