// An about box window
//
// David Hayward 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the handlers procs for an about box window.
// 
// 12/13/94	david	first cut


#include <PictUtil.h>
#include <Types.h>
#include <Files.h>
#include <QDOffScreen.h>
#include <Resources.h>
#include <GXTypes.h>

#include "appGlobals.h"
#include "appMain.h"
#include "appMenus.h"

#include "win.h"
#include "winAbout.h"
#include "winAboutMenus.h"

#include "gxGlobals.h"
#include "gxPrintUtils.h"



/**\
|**| ==============================================================================
|**| PRIVATE DEFINES
|**| ==============================================================================
\**/
#define rAboutBoxStringID	2000			// 'STR ' resource id
#define rAboutDocWindID		2000			// 'WIND' resource id


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
OSErr		DoCreateAboutWindow		( winHandle win ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	winActivateAbout
 *------------------------------------------------------------------------------*
		This is a ActivateProcPtr for the About window.
		So far all this does is update menus when its window becomes active.
		Other things, such as activating/deactivation controls should also go here
		This ProcPtr is envoked by CallWinActivateProc() which is called by:
			DoActivateEvent() which dispaches activate events and
			DoOSEvent() which handles events such as Suspend/Resume.
\*------------------------------------------------------------------------------*/
void	winActivateAbout ( winHandle win, Boolean becomingActive ) 
{
	if (becomingActive)
	{
		SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port
		winAdjustMenusAbout() ;
	}
}


/*------------------------------------------------------------------------------*\
	winUpdateAbout
 *------------------------------------------------------------------------------*
		This is a UpdateProcPtr for the About window.  
		It handles all drawing into the window.
		For the About window, this means that this routine is responible for
		drawing the text into the About box.
		This ProcPtr is envoked by CallWinUpdateProc() which is called by:
			DoUpdateEvent() which dispaches update events.
\*------------------------------------------------------------------------------*/
void	winUpdateAbout ( winHandle win, EventRecord *e )
{
	long			length ;
	Rect			aboutRect ;
	Handle			aboutText ;
	Handle			aboutRectHndl ;
	WindowPtr		whichWindow = (WindowPtr)e->message;
	GrafPtr			savedPort ;
	
	GetPort(&savedPort ) ;	
	SetPort( whichWindow );

	BeginUpdate (whichWindow);

	aboutRectHndl = GetResource( 'RECT', rAboutBoxStringID ) ;
	aboutRect = **((Rect**)(aboutRectHndl)) ;
	ReleaseResource( aboutRectHndl ) ;

	aboutText = GetResource( 'TEXT', rAboutBoxStringID ) ;
	HLock( aboutText ) ;
	length = GetHandleSize( aboutText ) ;
	TETextBox( *aboutText, length, &aboutRect, teCenter);
	HUnlock( aboutText ) ;
	ReleaseResource( aboutText ) ;
	
	// signal that we finished drawing
	EndUpdate (whichWindow);
	SetPort( savedPort ) ;
}


/*------------------------------------------------------------------------------*\
	winCloseAbout
 *------------------------------------------------------------------------------*
		This is a CloseProcPtr for the About window.
		So far, all this does is call DisposeWinHandle() and return all the menus
		to the default state by calling dimAllMenus() and DoAppAdjustMenus().
		This is needed because, if the window being closed is the only window,
		then there is not another window to trigger an activate event. 
		This ProcPtr is envoked by CallWinCloseProc() which is called by:
			app_aePDOC_handler() which handles PDOC AppleEvents,  
			DoMouseDownEvent() which dispached click events (e.g. close box), and
			MenuProcPtrs which are called whenever menu events occur.
\*------------------------------------------------------------------------------*/
void	winCloseAbout ( winHandle win ) 
{
	// could do something like ask the user to save if dirty
	DisposeWinHandle( win ) ;
	
	// we have to update the menu bar just in case
	// this window is the only window
	
	dimAllMenus() ;				// dim all menus
	DoAppAdjustMenus() ;		// undim app items
	DrawMenuBar() ;
	
	return ;
}


/*------------------------------------------------------------------------------*\
	winMenuAbout
 *------------------------------------------------------------------------------*
		This is a MenuProcPtr for the About window.
		This routine dispatches any menu commands that the window can handle
		to the appropriate function.
		For the About window, the only need menu command is File:Close 
		This ProcPtr is envoked by CallWinMenuProc() which is called by:
			HandleMenuCommand() which dispatches all menu events.
\*------------------------------------------------------------------------------*/
void winMenuAbout ( winHandle win, long menuResult )
{
	short			menuID;
	short			menuItem;
	

	menuID   = HiWrd(menuResult);
	menuItem = LoWrd(menuResult);
	switch ( menuID )
	{
		case mFile:
			switch ( menuItem )
			{
				case iClose:			// close an open window/document
					if (win != nil)
						CallWinCloseProc( win ) ;
					break;
			}
			break;
	}
	HiliteMenu(0);		// Unhighlight whatever MenuSelect or MenuKey hilited
}



/*------------------------------------------------------------------------------*\
	winAllocAbout
 *------------------------------------------------------------------------------*
		This is a winAllocAbout for the About window.
		This routine is responsible for filling in all the needed fields of the
		winHandle structure. This routine shouldn't be called directly.
		Instead, the code which needs to create a document of this type should
		call NewWinHandle(win,winAllocAbout) to envoke this function.
		This ProcPtr is envoked by NewWinHandle() which is called by:
			DoAboutBoxCommand() which handles the Apple:About menu command.
\*------------------------------------------------------------------------------*/
OSErr winAllocAbout ( winHandle win )
{
	OSErr			err = noErr ;
	
	// set the window type
	SetWinType( win, kAboutDocType );
	
	//  stuff winHandle fields
	SetWinActivateProc		( win,    (ActivateProcPtr) winActivateAbout );
	SetWinUpdateProc		( win,      (UpdateProcPtr) winUpdateAbout );
	SetWinClickProc			( win,       (ClickProcPtr) nil );
	SetWinKeyProc			( win,         (KeyProcPtr) nil );
	SetWinResumeProc		( win,      (ResumeProcPtr) nil );
	SetWinNullProc			( win,        (NullProcPtr) nil );
	SetWinMenuProc			( win,        (MenuProcPtr) winMenuAbout );
	SetWinResizeProc		( win,      (ResizeProcPtr) nil );
	
	SetWinNewProc			( win,         (NewProcPtr) winNewAbout );
	SetWinOpenProc			( win,        (OpenProcPtr) nil );
	SetWinCloseProc			( win,       (CloseProcPtr) winCloseAbout );
	SetWinDisposeProc		( win,     (DisposeProcPtr) nil );
	
	SetWinPageCountProc		( win,   (PageCountProcPtr) nil );
	SetWinPagePrintProc		( win,   (PagePrintProcPtr) nil );
	
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winNewAbout
 *------------------------------------------------------------------------------*
		This is a NewProcPtr for the About window.
		This routine is responsible for building a new About window after all the
		needed fields of the winHandle structure have been filled in by winAllocAbout.
		If an About window is already open, then this routine will bring it
		to the front instead of opening a second window.
		This routine shouldn't be called directly.  Instead, the code which needs
		to create a document of this type should call NewWinHandle(win,winAllocAbout),
		and then CallWinNewProc(win) to envoke this function.
		This ProcPtr is envoked by CallWinNewProc() which is called by:
			DoAboutBoxCommand() which handles the Apple:About menu command.
\*------------------------------------------------------------------------------*/
OSErr winNewAbout ( winHandle win )
{
	OSErr			err = noErr ;
	winHandle		existingDoc ;
	
	// see if we are already open
	
 	if ( FindWinHandle( nil, kAboutDocType, 1, 1, &existingDoc ) == 1 )
	{
		// bring it to the front
		SelectWindow( (WindowPtr) GetWinWindow(existingDoc) ) ;		
		return kWasAlreadyOpen ;
	}
	
	err = DoCreateAboutWindow( win ) ;
	if ( err ) return err;
	
	return err ;	
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	DoCreateAboutWindow
 *------------------------------------------------------------------------------*
		This routine actually creates the WindowPtr for a winHandle.
		After creating the window, it:
			reference the window and the DocumentRecord to each other,
			makes the window the current port,
			makes it visible .
		This routine is called by:
			winOpenAbout() which fills in the winHandle structure
\*------------------------------------------------------------------------------*/
OSErr DoCreateAboutWindow ( winHandle win )
{

	OSErr		err = noErr;
	WindowPtr	window ;
	
	window = GetNewCWindow(rAboutDocWindID, nil, (WindowPtr)-1 );
	
	SetWinWindow( win, (CGrafPtr)window );	// save a reference to the window in the winRecord
	SetWindowWinHandle( window, win );		// save a reference to the winRecord in the window

	SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port

	SetWinRect( win, ((CGrafPtr)window)->portRect ) ;

	// make sure it is visible
	ShowWindow( window ) ;

	return err ;
}


