// A pict file window
//
// David Hayward and Nick Thompson 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the handlers procs for a pict file window.
// 
// 9/13/94	nick	first cut
// 12/13/94	david	several modifications


#include <PictUtil.h>
#include <Types.h>
#include <Fonts.h>
#include <Files.h>
#include <QDOffScreen.h>
#include <Resources.h>
#include <GXTypes.h>

#include "appGlobals.h"
#include "appMain.h"
#include "appMenus.h"

#include "win.h"
#include "winPictDoc.h"
#include "winPictDocGetSet.h"
#include "winPictDocMenus.h"
#include "winProfile.h"
#include "winProfileGetSet.h"

#include "aeSenders.h"

#include "ErrorHandling.h"
#include "stringStuff.h"
#include "pictStuff.h"
#include "faux3DStuff.h"
#include "resourceStuff.h"
#include "colorsyncStuff.h"
#include "cursorStuff.h"

#include "gxGlobals.h"
#include "gxPrintUtils.h"
#include "qdPrintUtils.h"



/**\
|**| ==============================================================================
|**| PRIVATE DEFINES
|**| ==============================================================================
\**/
#define rPictDocWindID			3000				// 'WIND' resource id
#define rPictSrceProfRectID		3001
#define rPictDestProfRectID		3002
#define rPictPrevProfRectID		3003
#define rPictEmbdProfRectID		3004
#define kScrollDelta			16
#define kWindowWidthMin			500
#define kWindowHeightMin		200
#define kHeaderHeightBig		120
#define kHeaderHeightSmall		30
#define kStatusWidth			100
#define kCellHeight				16


static	RGBColor		ltGray = {0xDDDD, 0xDDDD, 0xDDDD};
static	RGBColor		white  = {0xFFFF, 0xFFFF, 0xFFFF};

/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
void		DoDrawPictData		( winHandle win, GrafPtr destPort, Rect *srcRect, Rect *dstRect ) ;
OSErr		DoCreatePalettes	( winHandle win ) ;
ListHandle	BuildProfList		( winHandle win ) ;
OSErr		DoCreatePictWindow	( winHandle win ) ;
pascal void	PictDocActionProc	( ControlHandle cntl, short part) ;
void		UpdatePane			( winHandle win, Rect updateRect ) ;
void		ScrollPane			( winHandle win, ControlHandle cntl, short oldCntlVal ) ;
void		ChangeCSOnOff		( winHandle win ) ;
void		ChangeCSMode		( winHandle win ) ;
Boolean		SetWinContentColor	( WindowPtr window, RGBColor theColor ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	winActivatePictDoc
 *------------------------------------------------------------------------------*
		This is a ActivateProcPtr for PictDoc windows.
		So far all this does is update menus when its window becomes active.
		Other things, such as activating/deactivation controls should also go here
		This ProcPtr is envoked by CallWinActivateProc() which is called by:
			DoActivateEvent() which dispaches activate events and
			DoOSEvent() which handles events such as Suspend/Resume.
\*------------------------------------------------------------------------------*/
void	winActivatePictDoc ( winHandle win, Boolean becomingActive ) 
{
	if (becomingActive)
	{
		SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port
		winAdjustMenusPictDoc( win ) ;
	}
}


/*------------------------------------------------------------------------------*\
	winUpdatePictDoc
 *------------------------------------------------------------------------------*
		This is a UpdateProcPtr for PictDoc windows.  
		It handles all drawing into the window.
		For PictDoc windows, this means that this routine is responible for
		drawing the scroll bars, grow box, the header area, and the Pict itself.
		The Pict is drawn in the space provided by the current window size and
		in accordance with the scroll bar values.  It is drawn by calling 
		DoDrawPictData() which is also used by the PagePrintProc.
		This ProcPtr is envoked by CallWinUpdateProc() which is called by:
			DoUpdateEvent() which dispaches update events.
\*------------------------------------------------------------------------------*/
void	winUpdatePictDoc ( winHandle win, EventRecord *e )
{
	Rect			docRect ;
	Rect			pictRect ;
	Rect			headRect, headUpdateRect ;
	Rect			growRect, growUpdateRect ;
	Rect			statRect, statUpdateRect ;
	WindowPtr		window = (WindowPtr)e->message;
	Rect 			updateRect = ((**(window->visRgn)).rgnBBox);
	GrafPtr			savedPort ;
	
	GetPort(&savedPort ) ;	
	SetPort( window );

	BeginUpdate( window );


	docRect  = GetWinRect( win ) ;					// check if ok?
	pictRect = GetPictDocPictRect( win ) ;			// check if ok?
	
// draw header	
	headRect = docRect ;
	headRect.bottom = kHeaderHeightBig;
	headRect.bottom -= 3;
	if (SectRect(&headRect,&updateRect,&headUpdateRect))
	{
		faux3DFillRect ( &headRect ) ;
		ForeColor(whiteColor);
		MoveTo (headRect.left, headRect.bottom+1); LineTo (headRect.right, headRect.bottom+1);
		ForeColor(blackColor);
		MoveTo (headRect.left, headRect.bottom  ); LineTo (headRect.right, headRect.bottom  );
		MoveTo (headRect.left, headRect.bottom+2); LineTo (headRect.right, headRect.bottom+2);
		{
			Rect		embdRect,    srceRect,    destRect,    prevRect ;
			Str255		embdProfStr, srceProfStr, destProfStr, prevProfStr ;
			CMError		cmerr ;
			ListHandle	list ;

			// get rects from resources
			srceRect = GetRect ( rPictSrceProfRectID ) ;
			destRect = GetRect ( rPictDestProfRectID ) ;
			prevRect = GetRect ( rPictPrevProfRectID ) ;
			embdRect = GetRect ( rPictEmbdProfRectID ) ;
			
			TextSize(9) ;
			TextFace(bold) ;
			TextFont(geneva) ;
			ForeColor(blackColor);
			
			GetStringPtr ( rPictSrceProfRectID, srceProfStr ) ;
			GetStringPtr ( rPictDestProfRectID, destProfStr ) ;
			GetStringPtr ( rPictPrevProfRectID, prevProfStr ) ;
			GetStringPtr ( rPictEmbdProfRectID, embdProfStr ) ;
			
			MoveTo (srceRect.left +4, srceRect.top -4); DrawString( srceProfStr ) ;
			MoveTo (destRect.left +4, destRect.top -4); DrawString( destProfStr ) ;
			MoveTo (prevRect.left +4, prevRect.top -4); DrawString( prevProfStr ) ;
			MoveTo (embdRect.left +4, embdRect.top -4); DrawString( embdProfStr ) ;

			faux3DFillRectIn( &srceRect ) ;
			faux3DFillRectIn( &destRect ) ;
			faux3DFillRectIn( &prevRect ) ;
			faux3DFillRectIn( &embdRect ) ;
			
			TextSize(9) ;
			TextFace(0) ;
			TextFont(geneva) ;
			ForeColor(blackColor);

			cmerr = GetProfName ( GetPictDocProfile( win, kSrceProf ), srceProfStr ) ;
			cmerr = GetProfName ( GetPictDocProfile( win, kDestProf ), destProfStr ) ;
			cmerr = GetProfName ( GetPictDocProfile( win, kPrevProf ), prevProfStr ) ;
			
			list = GetPictDocProfList( win );
			LDoDraw( true, list );
			LUpdate( window->visRgn, list);		// not sure this is the rgn
			
			MoveTo (srceRect.left +6, srceRect.top + 14); DrawString( srceProfStr ) ;
			MoveTo (destRect.left +6, destRect.top + 14); DrawString( destProfStr ) ;
			MoveTo (prevRect.left +6, prevRect.top + 14); DrawString( prevProfStr ) ;
		}
	}

// update controls
	TextSize(9) ;
	TextFace(bold) ;
	TextFont(geneva) ;
	SetWinContentColor ( window, ltGray ) ;
	DrawControls( window );
	SetWinContentColor ( window, white ) ;		// should restore previous
	{
	short			val;
	val = GetPictDocCntlValue ( win, rPictDocMatchOnOffID ) ;
	SetPictDocCntlValue ( win, rPictDocMatchOnOffID, !val ) ;
	SetPictDocCntlValue ( win, rPictDocMatchOnOffID,  val ) ;
	}
	
// draw GrowIcon
	growRect = docRect ;
	growRect.top += kHeaderHeightBig ;
	if (SectRect(&growRect,&updateRect,&growUpdateRect))
	{
		RgnHandle saveRgn = NewRgn() ;
		GetClip( saveRgn ) ;
		ClipRect( &growRect ) ;
		DrawGrowIcon( window ) ;
		SetClip( saveRgn ) ;
		DisposeRgn( saveRgn ) ;
	}
	
// startdraw status box	
	statRect = docRect ;
	statRect.top = statRect.bottom +1 - kScrollWidth ;
	statRect.right = kStatusWidth - 1 ;
	if (SectRect(&statRect,&updateRect,&statUpdateRect))
	{
		faux3DFillRect ( &statRect ) ;
		InsetRect( &statRect, -1, -1 ) ;
		ForeColor(blackColor);
		FrameRect( &statRect ) ;
	}

// draw scrolling pane
	UpdatePane ( win, updateRect ) ;
	
// finish drawing status box
	if (SectRect(&statRect,&updateRect,&statUpdateRect))
	{
		Str255		string ;
		long		bytes ;
		long		ticks ;
		Fixed		fix ;
		short		mode, cson ;
		
		ForeColor(blackColor);
		TextSize(9) ;
		TextFace(0) ;
		TextFont(geneva) ;
		
		cson = GetPictDocCntlValue ( win, rPictDocMatchOnOffID ) ;
		mode = GetPictDocCntlValue ( win, rPictDocMatchPopupID ) ;
		if (!cson) mode = kOriginal ;
		
		bytes = GetHandleSize( (Handle)GetPictDocPict(win,mode) ) ;
		fix = (Fixed)(bytes<<6) ;
		FormatFixed( fix, rKBytesFMAT, string) ;
		MoveTo( statRect.left+10, statRect.bottom-4 ) ;
		DrawString( string ) ;

		if (mode!= kOriginal)
		{
			ticks = GetPictDocDrawTime(win,mode) ;
			fix = FixRatio( (short)ticks, 60 ) ;
			FormatFixed( fix, rSecsFMAT, string) ;
			MoveTo( statRect.left+50, statRect.bottom-4 ) ;
			DrawString( string ) ;
		}
	}

// signal that we finished drawing
	EndUpdate( window );
	SetPort( savedPort ) ;
}


/*------------------------------------------------------------------------------*\
	winClickPictDoc
 *------------------------------------------------------------------------------*
		This is a ClickProcPtr for PictDoc windows.
		So far, all this contains is a crude handling of the scroll controls
		Other things, such as initiating a drag-and-drop should also go here
		This ProcPtr is envoked by CallWinClickProc() which is called by:
			DoMouseDownEvent() which dispaches mouse down events 
\*------------------------------------------------------------------------------*/
void	winClickPictDoc ( winHandle win, EventRecord *e ) 
{
	Point				pt = e->where;
	ControlHandle		hCtl, vCtl, csonCtl, modeCtl ;
	WindowPtr			window ;
	Boolean				dblClk ;
	ListHandle			list ;
	Point				cell = {0,0} ;
	unsigned long		index ;
	ControlHandle		cntl ;
	short				oldCntlVal ;
	short				cntlPart ;
	ControlActionUPP	caUPP ;
	
	window = (WindowPtr) GetWinWindow( win ) ;
	SetPort (window ) ;
	GlobalToLocal( &pt );

	hCtl = GetPictDocCntl( win, rPictDocHorzScrollID ) ;
	vCtl = GetPictDocCntl( win, rPictDocVertScrollID ) ;
	csonCtl = GetPictDocCntl( win, rPictDocMatchOnOffID ) ;
	modeCtl = GetPictDocCntl( win, rPictDocMatchPopupID ) ;
	
	list = GetPictDocProfList( win ) ;
	
	dblClk = LClick( pt, e->modifiers, list ) ;
	
	if ( dblClk )
	{
		while ( LGetSelect(true, &cell, list))
		{
			OSErr			err = noErr ;
			CMError			cmerr = noErr ;
			CMProfileRef	prof ;
			CMProfileRef	profCopy ;
			winHandle		profWin ;
			FSSpec			spec ;
			
			index = cell.v +1 ;

			cmerr = GetIndexedProfileFromPicHandle( GetPictDocPict(win,kOriginal), index, &prof, nil ) ;
			cmerr = ReOpenProfileRef ( &profCopy, prof ) ;
			CMCloseProfile( prof ) ;

			// create a new winHandle of the proper type
			err = NewWinHandle( &profWin, winAllocProfile ) ;
			FailIfErr( err ) ;
			
			spec = GetWinFSSpec(win) ;

			SetWinFSSpec( profWin, &spec ) ;
			SetProfileRef( profWin, profCopy ) ;
			SetProfileSubType( profWin, kEmbededSubType ) ;
			SetProfileIndex( profWin, index ) ;
			
			err = CallWinOpenProc ( profWin ) ;
		
			if ( err != noErr )			// if an error occured
				DisposeWinHandle( profWin );
			if ( err == kWasAlreadyOpen )
				err = noErr;
				
			LNextCell( false, true, &cell, list ) ;
		}

	}

	cntlPart = FindControl(pt, window, &cntl);
	
	if ( cntl==hCtl	|| cntl==vCtl )
	{
		switch (cntlPart)
		{
			case inUpButton :
			case inDownButton :
			case inPageUp :
			case inPageDown :
					caUPP = NewControlActionProc(PictDocActionProc) ;
					cntlPart = TrackControl( cntl, pt, (ControlActionUPP)caUPP ) ;
					DisposeRoutineDescriptor( caUPP ) ;
					break;
			case inThumb :
					oldCntlVal = GetCtlValue( cntl ) ;
					cntlPart = TrackControl( cntl, pt ,nil) ;
					if ( cntlPart != 0 )
						ScrollPane( win, cntl, oldCntlVal ) ;
					break;
		}
	}
	if ( cntl==csonCtl )
	{
		TextSize(9) ;
		TextFace(bold) ;
		TextFont(geneva) ;
		cntlPart = TrackControl( cntl, pt ,nil) ;
		if ( cntlPart != 0 )
			ChangeCSOnOff( win ) ;
	}
	if ( cntl==modeCtl )
	{
		TextSize(9) ;
		TextFace(bold) ;
		TextFont(geneva) ;
		cntlPart = TrackControl( cntl, pt ,(ControlActionUPP)-1) ;
		if ( cntlPart != 0 )
			ChangeCSMode( win ) ;
	}
}


/*------------------------------------------------------------------------------*\
	winResizePictDoc
 *------------------------------------------------------------------------------*
		This is a ResizeProcPtr for PictDoc windows.
		This routine is responsible for re-arranging the contents of the window if
		it is resized.  On entry to this routine, the new window size can be found
		in the document's window's portRect.  At the very least, this routine 
		should call SetWinRect() to put the new window size in the winHandle.
		So far, the only other thing this routine does is repostion the scroll controls
		and issue the appropriate update events so everything is re-drawn correctly.
		This ProcPtr is envoked by CallWinResizeProc() which is called by:
			DoMouseDownEvent() which dispaches mouse down events and
			DoCreatePictWindow() which initially creates the window.
\*------------------------------------------------------------------------------*/
void	winResizePictDoc ( winHandle win ) 
{
	ControlHandle	hCtl, vCtl ;
	WindowPtr		window ;
	Rect			horzInval, vertInval ;
	Rect			oldDocRect, newDocRect, pictRect ;
	Rect			paneRect ;

	oldDocRect = GetWinRect( win );
	pictRect = GetPictDocPictRect( win );

	window = (WindowPtr) GetWinWindow ( win ) ;
	newDocRect = window->portRect ;
	SetWinRect( win, newDocRect );

	horzInval = vertInval = newDocRect ;
	horzInval.top  = MIN( oldDocRect.bottom, newDocRect.bottom) - kScrollWidth;
	vertInval.left = MIN( oldDocRect.right, newDocRect.right)   - kScrollWidth;
	InvalRect( &horzInval ) ;
	InvalRect( &vertInval ) ;
	EraseRect( &horzInval ) ;
	EraseRect( &vertInval ) ;
	
	hCtl = GetPictDocCntl( win, rPictDocHorzScrollID ) ;
	vCtl = GetPictDocCntl( win, rPictDocVertScrollID ) ;

	paneRect = newDocRect ;
	paneRect.top += kHeaderHeightBig ;
	paneRect.left = 0;
	paneRect.bottom -= kScrollWidth ;
	paneRect.right -= kScrollWidth ;
	SetPictDocPaneRect( win, paneRect ) ;
	
	MoveControl(hCtl, (paneRect.left-1)+kStatusWidth,
					  paneRect.bottom ) ;	
	MoveControl(vCtl, paneRect.right,
					  paneRect.top-1) ;
		
	SizeControl(hCtl, (2+paneRect.right-paneRect.left)-kStatusWidth,
					  kScrollWidth+1) ;	
	SizeControl(vCtl, kScrollWidth+1,
					  2+paneRect.bottom-paneRect.top) ;

	{
		short pV, wV, max, oldCntlVal ;
		
		pV = pictRect.bottom - pictRect.top ;
		wV = paneRect.bottom - paneRect.top ;
		if (wV>=pV)
			max = 0 ;
		else
			max = ((MAX (0, pV-wV )) / kScrollDelta) +1 ;
		oldCntlVal = GetControlValue(vCtl) ;
		SetControlMinimum(vCtl, 0);
		SetControlMaximum(vCtl, max) ;
		SetControlValue  (vCtl, oldCntlVal ) ;
		
		HiliteControl(vCtl, (max==0)?(255):(0) );	
		ScrollPane( win, vCtl, oldCntlVal ) ;
	}
	{
		short pH, wH, max, oldCntlVal ;
		
		pH = pictRect.right - pictRect.left ;
		wH = paneRect.right - paneRect.left ;
		if (wH>=pH)
			max = 0 ;
		else
			max = ((MAX (0, pH-wH )) / kScrollDelta) +1 ;
		oldCntlVal = GetControlValue(hCtl) ;
		SetControlMinimum(hCtl, 0);
		SetControlMaximum(hCtl, max) ;
		SetControlValue  (hCtl, oldCntlVal );
		
		HiliteControl(hCtl, (max==0)?(255):(0) );	
		ScrollPane( win, hCtl, oldCntlVal ) ;
	}

	return ;
}


/*------------------------------------------------------------------------------*\
	winClosePictDoc
 *------------------------------------------------------------------------------*
		This is a CloseProcPtr for PictDoc windows.
		So far, all this does is call DisposeWinHandle() and return all the menus
		to the default state by calling dimAllMenus() and DoAppAdjustMenus().
		This is needed because, if the window being closed is the only window,
		then there is not another window to trigger an activate event. 
		Other things, such as prompting the user to save the file, should go here.
		This ProcPtr is envoked by CallWinCloseProc() which is called by:
			app_aePDOC_handler() which handles PDOC AppleEvents,  
			DoMouseDownEvent() which dispached click events (e.g. close box), and
			MenuProcPtrs which are called whenever menu events occur.
\*------------------------------------------------------------------------------*/
void	winClosePictDoc ( winHandle win ) 
{
	// could do something like ask the user to save if dirty
	DisposeWinHandle( win ) ;
	
	// we have to update the menu bar just in case
	// this window is the only window
	
	dimAllMenus() ;				// dim all menus
	DoAppAdjustMenus() ;		// undim app items
	DrawMenuBar() ;
	
	return ;
}


/*------------------------------------------------------------------------------*\
	winMenuPictDoc
 *------------------------------------------------------------------------------*
		This is a MenuProcPtr for PictDoc windows.
		This routine dispatches any menu commands that the window can handle
		to the appropriate function.
		This ProcPtr is envoked by CallWinMenuProc() which is called by:
			HandleMenuCommand() which dispatches all menu events.
\*------------------------------------------------------------------------------*/
void winMenuPictDoc ( winHandle win, long menuResult )
{
	short			menuID;
	short			menuItem;
	

	menuID   = HiWrd(menuResult) ;
	menuItem = LoWrd(menuResult) ;
	switch ( menuID )
	{
		case mFile:
			switch ( menuItem )
			{
				case iClose:			// close an open window/document
					CallWinCloseProc( win ) ;
					break ;
				case iSave:				// save this document under the name it was opened
					winSaveCmndPictDoc( win ) ;
					break ;
				case iSaveAs:			// save this document under a new name
					winSaveAsCmndPictDoc( win ) ;
					break ;
				case iRevert:			// discard all changes made to the document since it was last saved
					winRevertCmndPictDoc( win ) ;
					break ;
				case iPageSetup:		// std page setup dialog
					winPageSetupCmndPictDoc( win ) ;
					break ;
				case iPrint:			// std print job dialog
					winPrintCmndPictDoc( win ) ;
					break ;
				case iPrintOne:			// std print job dialog
					winPrintOneCmndPictDoc( win ) ;
					break ;
			}
			break;
	}
	HiliteMenu(0);		// Unhighlight whatever MenuSelect or MenuKey hilited
}


/*------------------------------------------------------------------------------*\
	winAllocPictDoc
 *------------------------------------------------------------------------------*
		This is a AllocProcPtr for PictDoc windows.
		This routine is responsible for filling in all the needed fields of the
		winHandle structure.  This routine shouldn't be called directly.
		Instead, the code which needs to create a document of this type should
		call NewWinHandle(win,winAllocPictDoc) to envoke this function.
		This ProcPtr is envoked by NewWinHandle() which is called by:
			app_aeODOC_handler() which handles ODOC AppleEvents, and
			app_aePDOC_handler() which handles PDOC AppleEvents.
\*------------------------------------------------------------------------------*/
OSErr winAllocPictDoc ( winHandle win )
{
	OSErr			err = noErr ;
	PictDocDataHdl	data ;
	
	// set the window type
	SetWinType( win, kPictDocType ) ;
	
	// allocate data handle
	data = (PictDocDataHdl)NewHandleClear( sizeof(PictDocDataRec) ) ;
	if( data == nil ) return MemError();
	SetWinData ( win, (Handle)data ) ;
	(**data).PictMatchedNeedsUpdate =
	(**data).PictProofedNeedsUpdate =
	(**data).PictCheckedNeedsUpdate = true ;
	
	// stuff winHandle fields
	SetWinActivateProc		( win,    (ActivateProcPtr) winActivatePictDoc );
	SetWinUpdateProc		( win,      (UpdateProcPtr) winUpdatePictDoc );
	SetWinClickProc			( win,       (ClickProcPtr) winClickPictDoc );
	SetWinKeyProc			( win,         (KeyProcPtr) nil );
	SetWinResumeProc		( win,      (ResumeProcPtr) nil );
	SetWinNullProc			( win,        (NullProcPtr) nil );
	SetWinMenuProc			( win,        (MenuProcPtr) winMenuPictDoc );
	SetWinResizeProc		( win,      (ResizeProcPtr) winResizePictDoc );
	
	SetWinNewProc			( win,         (NewProcPtr) winNewPictDoc );
	SetWinOpenProc			( win,        (OpenProcPtr) winOpenPictDoc );
	SetWinCloseProc			( win,       (CloseProcPtr) winClosePictDoc );
	SetWinDisposeProc		( win,     (DisposeProcPtr) winDisposePictDoc );

	SetWinPageCountProc		( win,   (PageCountProcPtr) winPageCountPictDoc );
	SetWinPagePrintProc		( win,   (PagePrintProcPtr) winPagePrintPictDoc );
	
	return err ;	
}


OSErr winNewPictDoc ( winHandle win )
{
	return noErr;
}


/*------------------------------------------------------------------------------*\
	winOpenPictDoc
 *------------------------------------------------------------------------------*
		This is a OpenProcPtr for PictDoc windows.
		This routine is responsible for opening a PictDoc window after all the
		needed fields of the winHandle structure have been filled in by winAllocPictDoc.
		On entry the DocumentRecord must contain a valid FSSpec.
		If a PictDoc window for the FSSpec is already onen, then this routine 
		will bring it to the front instead of opening a second window.
		This routine shouldn't be called directly.  Instead, the code which needs
		to create a document of this type should call NewWinHandle(win,winAllocPictDoc),
		SetWinFSSpec(win,spec), and then CallWinOpenProc(win) to envoke this function.
		This ProcPtr is envoked by CallWinOpenProc() which is called by:
			app_aeODOC_handler() which handles ODOC AppleEvents, and
			app_aePDOC_handler() which handles PDOC AppleEvents.
\*------------------------------------------------------------------------------*/
OSErr winOpenPictDoc ( winHandle win )
{
	OSErr			err = noErr ;
	PicHandle		pict ;
	Rect			rect ;
	FSSpec			spec ;
	winHandle		existingDoc ;
	
	spec = GetWinFSSpec ( win );

	// see if we are already open
 	if ( FindWinHandle( &spec, kPictDocType, 1, 1, &existingDoc ) == 1 )
	{
		// bring it to the front
		SelectWindow( (WindowPtr) GetWinWindow(existingDoc) ) ;		
		return kWasAlreadyOpen;
	}

	// try to read the pict
	err = DoReadPICT ( spec, &pict ) ;
	if ( err ) return err ;
	
	// set the window's rect
	rect = (**pict).picFrame ;
	SetPictDocPictRect( win, rect );

	// set the window's data's pict
	SetPictDocPict( win, kOriginal, pict ) ;
	SetPictDocNeedsUpdate( win, kMatched, true ) ;
	SetPictDocNeedsUpdate( win, kProofed, true ) ;
	SetPictDocNeedsUpdate( win, kChecked, true ) ;
	
	err = DoCreatePalettes( win ) ;
	if ( err ) return err ;

	err = DoCreatePictWindow( win ) ;
	if ( err ) return err ;
	
	if( gQDGX_available ) 
	{
		gxJob		newJob ;
		gxFormat	newFormat ;
		
		err = GXNewJob(&newJob);
		if ( err ) return err ;
		SetWinGXJob( win, newJob ) ;
		
		newFormat = GXNewFormat(newJob);
		err = GXGetJobError(newJob);
		if ( err ) return err ;
		SetWinGXFormat( win, newFormat ) ;

		// instal our document specific overrides
		InstallGXOverrides( win ) ;
	}
	else
		DoCreatePrintRecord( win ) ;
	
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winDisposePictDoc
 *------------------------------------------------------------------------------*
		This is a DisposeProcPtr for PictDoc windows.
		This routine is responsible for disposing of any data that was allocated
		by the document and stored in the its data handle.
		This ProcPtr is envoked by CallWinDisposeProc() which is called by:
			DisposeWinHandle() which disposes of the entire winHandle.
\*------------------------------------------------------------------------------*/
void winDisposePictDoc ( winHandle win )
{
	PictDocDataHdl	data ;
	PicHandle		pict ;
	CMProfileRef	prof ;
	PaletteHandle	palette ;
	ListHandle		list ;

	data = (PictDocDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return;
	
	// dispose of all the PicHandles we used
	pict = GetPictDocPict( win, kOriginal ) ;
	if ( pict != nil ) DisposeHandle( (Handle)pict ) ;

	pict = GetPictDocPict( win, kProofed ) ;
	if ( pict != nil ) DisposeHandle( (Handle)pict ) ;

	pict = GetPictDocPict( win, kMatched ) ;
	if ( pict != nil ) DisposeHandle( (Handle)pict ) ;

	pict = GetPictDocPict( win, kChecked ) ;
	if ( pict != nil ) DisposeHandle( (Handle)pict ) ;

	// close of all the Profiles we used
	prof = GetPictDocProfile( win, kSrceProf ) ;
	if ( prof != nil ) CMCloseProfile( prof ) ;
	
	prof = GetPictDocProfile( win, kDestProf ) ;
	if ( prof != nil ) CMCloseProfile( prof ) ;
	
	prof = GetPictDocProfile( win, kPrevProf ) ;
	if ( prof != nil ) CMCloseProfile( prof ) ;

	// dispose of the ListHandle we used
	list = GetPictDocProfList ( win ) ;
	if ( list != nil ) LDispose(list) ;
	
	// dispose of the PictPalette we used
	palette = GetPictDocPalette( win ) ;
	if ( palette != nil ) DisposePalette( palette );

	// do we need to dispose of controls ?
}


/*------------------------------------------------------------------------------*\
	winPageCountPictDoc
 *------------------------------------------------------------------------------*
		This is a PageCountProcPtr for PictDoc windows.
		This routine is responsible for the number of pages needed to render the 
		document's data (whatever it may be) gives the bounds of a page.
		This ProcPtr is envoked by CallWinPageCountProc() which is called by:
			DoPrintLoop() which impliments an old-style printing loop, and
			DoGXPrintLoop() which impliments a GX-style printing loop.
\*------------------------------------------------------------------------------*/
short	winPageCountPictDoc	( winHandle win, Rect *page )
{
	Rect		pictRect ;
	short		horizOffset, 
				vertOffset,
				pagesWide, 		// the number of pages wide the image is
				pagesHigh, 		// the number pages high for this image
				pageWidth, pageHeight, 		// dimensions of one page
				pictWidth, pictHeight ;		// dimensions of the pict
	
	pictRect = GetPictDocPictRect( win ) ;
	
	horizOffset = -(pictRect.left) ;
	vertOffset = -(pictRect.top) ;
	
	OffsetRect( &pictRect, horizOffset, vertOffset ) ;	// this should make the origin of the rect (0,0)

	pageWidth = page->right - page->left ;
	pageHeight = page->bottom - page->top ;
	
	pictWidth  = pictRect.right - pictRect.left ;
	pictHeight = pictRect.bottom - pictRect.top ;
	
//	pagesWide = 1 + (pictWidth / pageWidth) ;		// round up the number of pages to the nearest
//	pagesHigh = 1 + (pictHeight  / pageHeight) ;	// whole page in each direction.
	
for (pagesWide=1; pagesWide*pageWidth <= pictWidth;  pagesWide++) ;
for (pagesHigh=1; pagesHigh*pageHeight<= pictHeight; pagesHigh++) ;
	
	return pagesWide * pagesHigh ;
}


/*------------------------------------------------------------------------------*\
	winPagePrintPictDoc
 *------------------------------------------------------------------------------*
		This is a PagePrintProcPtr for PictDoc windows.
		This routine is responsible for the printing a given page of document's
		data into the destination port.
		This ProcPtr is envoked by CallWinPagePrintProc() which is called by:
			DoPrintLoop() which impliments an old-style printing loop, and
			DoGXPrintLoop() which impliments a GX-style printing loop.
\*------------------------------------------------------------------------------*/
void	winPagePrintPictDoc	( winHandle win, Rect *page, GrafPtr imagingPort, short pageNum )
{
	Rect		pictRect, srcRect, dstRect;
	short		pagesWide, 		// the number of pages wide the image is
				pagesHigh, 		// the number pages high for this image
				horozTile, 		// used in the loop to denote the H tile to print from the image
				vertTile ; 		// used in the loop to denote the V tile to print from the image
	short		thisPage = 1;	// used to find the page they want us to print
	short		pictHOff,
				pictVOff,
				pictWidth, 		// the width of the win's GWorld
				pictHeight ;	// the height of the win's GWorld
	short		pageWidth, 		// the width of one page
				pageHeight ;	// the height of one page

	pictRect = GetPictDocPictRect( win );
	
	SetPort( imagingPort ) ;
	
	pictHOff   = -(pictRect.left) ;
	pictVOff   = -(pictRect.top) ;
	pictWidth  = pictRect.right - pictRect.left ;
	pictHeight = pictRect.bottom - pictRect.top ;
	
//	OffsetRect( &pictRect, pictHOff, pictVOff ) ;	// this should make the origin of the rect (0,0)

	pageWidth = page->right - page->left ;
	pageHeight = page->bottom - page->top ;
	
//	pagesWide = (1.0 + pictWidth  / pageWidth  ) ;	// round up the number of pages to the nearest
//	pagesHigh = (1.0 + pictHeight / pageHeight ) ;	// whole page in each direction.
	
for (pagesWide=1; pagesWide*pageWidth <= pictWidth;  pagesWide++) ;
for (pagesHigh=1; pagesHigh*pageHeight<= pictHeight; pagesHigh++) ;

	thisPage = 1;
	for (vertTile=0; vertTile<pagesHigh; vertTile++)
	{
		for (horozTile=0; horozTile<pagesWide; horozTile++)
		{
			// check this is the page we were called to print
			
			if( thisPage == pageNum )
			{
				srcRect = *page;
				
				OffsetRect( &srcRect,
							pictRect.left - srcRect.left,
							pictRect.top  - srcRect.top );
	
				OffsetRect( &srcRect,
							horozTile * pageWidth,
							vertTile * pageHeight );
				
				SectRect( &srcRect, &pictRect, &srcRect);
				

				// finess the rect to print, we'll assume the the top left is always correct.
				// this helps with areas of the pict that don't fit entirely on one sheet of
				// paper
				
				dstRect = srcRect;
				
				OffsetRect( &dstRect,
							page->left - srcRect.left,
							page->top  - srcRect.top );

				DoDrawPictData( win, imagingPort, &srcRect, &dstRect ); 

			}
			
			// bump the page number
			thisPage++ ;
		}
	}
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	DoDrawPictData
 *------------------------------------------------------------------------------*
		This routine draws the postion of the documents data bounded by srcRect,
		into the rectangle dstRect of the destPort.
		data into the destination port.
		This routine is called by:
			winUpdatePictDoc() which draws into the scrollable window, and
			winPagePrintPictDoc() which draws into a printing port.
\*------------------------------------------------------------------------------*/
void	DoDrawPictData ( winHandle win, GrafPtr destPort, Rect *srcRect, Rect *dstRect ) 
{	
	Rect			rect;
	PictDocDataHdl	data ;
	PicHandle		pict ;
	RgnHandle		savedClipRgn;
	short			mode, cson ;
	
	data = (PictDocDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return;
	
	rect = GetPictDocPictRect( win ) ;

	SetGWorld( (CGrafPtr)destPort, nil ) ;	// set the destPort to be the current port

	MapRect(&rect, srcRect, dstRect) ;

	savedClipRgn = NewRgn() ;
	GetClip(savedClipRgn) ;
	ClipRect(dstRect) ;
	
	cson = GetPictDocCntlValue ( win, rPictDocMatchOnOffID ) ;
	mode = GetPictDocCntlValue ( win, rPictDocMatchPopupID ) ;
	if (!cson) mode = kOriginal ;
	
	UpdatePict( win, mode ) ;
	if (mode==kDrwMchdPct)
	{
		pict = GetPictDocPict(win,kOriginal) ;
		if (pict != nil)
			NCMDrawMatchedPicture(pict, GetPictDocProfile(win,kPrevProf), &rect) ;
		else
			EraseRect( &rect ) ;	
	}
	else
	{
		pict = GetPictDocPict(win,mode) ;
		if (pict != nil)
			DrawPicture( pict, &rect ) ;
		else
			EraseRect( &rect ) ;	
	}
	
	SetClip(savedClipRgn) ;
	DisposeRgn(savedClipRgn) ;
}



/*------------------------------------------------------------------------------*\
	DoCreatePalettes
 *------------------------------------------------------------------------------*
		This routine extracts the palette, and colortable from the document's 
		PictHandle and stores them in the dPictPalette and dPictCTab fields of
		the PictDocDataHdl. 
		This routine is called by:
			winOpenPictDoc() which fills in the winHandle structure
\*------------------------------------------------------------------------------*/
OSErr DoCreatePalettes ( winHandle win )
{
	OSErr			err ;
	PicHandle		pict ;
	PictInfo		pictInfo ;
	CTabHandle		ctab ;
	PaletteHandle	pal ;
	PictDocDataHdl	data ;

	data = (PictDocDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return nilHandleErr;
	
	pict = GetPictDocPict( win, kOriginal ) ;
		
	// use the picture utilities to get the palette for the window
	err = GetPictInfo( pict, &pictInfo, returnColorTable, 256, systemMethod, 0) ;
//	err = CollectColors( pict, &pictInfo ) ;

	ctab = pictInfo.theColorTable ;
	pal = NewPalette( (**ctab).ctSize +1, ctab, pmTolerant, 0x1000 ) ;

	// set up the palette for later use
	SetPictDocPalette( win, pal ) ;

	return noErr;
}


/*------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------*/
static ListHandle BuildProfList ( winHandle win )
{
	ListHandle			theList = nil ;
	unsigned long		count ;
	CMError				cmerr ;
	PicHandle			pict ;

	pict = GetPictDocPict( win, kOriginal ) ;
	
//------------- search for all profiles
	{
		cmerr = CountProfilesInPicHandle ( pict,  &count ) ;
		if (cmerr) goto bail ;
		SetPictDocListCount ( win, count ) ;
	}
	
	
//------------- build a list
	{
		Rect		rView, rDataBnds ;
		Point		cellSize ;
		WindowPtr	window ;

		rView = GetRect ( rPictEmbdProfRectID ) ;
		rView.right -= kScrollWidth;
		SetPt( &cellSize, rView.right-rView.left, kCellHeight ) ;
		InsetRect( &rView, 1, 1) ;
		
		window = (WindowPtr) GetWinWindow ( win ) ;
		SetRect( &rDataBnds, 0,0,1,count ) ;
		theList = LNew(&rView,
						&rDataBnds,
						cellSize,
						0,						// proc ID
						window,
						false,					// draw now
						true,					// hasGrow
						false,					// scrollHoriz
						true ) ;				// scrollVert
		SetPictDocProfList( win, theList ) ;
	}


//------------- add strings to list cells
	{
		Point				theCell ;
		unsigned long		v ;
		short				dataLen ;
		Str255				name ;
		CMProfileRef		prof ;

		for ( v=0; v<count; v++ )
		{
			cmerr = GetIndexedProfileFromPicHandle ( pict, v+1, &prof, nil ) ;
			cmerr = GetProfName( prof, (StringPtr)name ) ;
			CMCloseProfile( prof ) ;
			
			SetPt(&theCell,0,v);
			dataLen = name[0] ;
			LSetCell( &(name[1]),  dataLen, theCell, theList ) ;
		}
	}
	
//-------------
bail :
	return theList ;
}


/*------------------------------------------------------------------------------*\
	DoCreatePictWindow
 *------------------------------------------------------------------------------*
		This routine actually creates the WindowPtr for a winHandle.
		It assumes that there is a valid FSSpec and PictHandle in the winHandle. 
		After creating the window, it:
			reference the window and the DocumentRecord to each other,
			makes the window the current port,
			loads and adds the scroll bar controls,
			resizes the window according to the Pict,
			attaches the winHandle's palette to the window,
			titles it according the the FSSpec, and
			makes it visible .
		This routine is called by:
			winOpenPictDoc() which fills in the winHandle structure
\*------------------------------------------------------------------------------*/
OSErr DoCreatePictWindow ( winHandle win )
{

	Rect			rect ;
	Rect			sizeRect ;
	OSErr			err = noErr;
	WindowPtr		window ;
	FSSpec			spec;
	PictDocDataHdl	data ;
	PaletteHandle	palette ;
	ListHandle		profList ;
	short			h,v ;
	
	data = (PictDocDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return nilHandleErr;
	
	// create the window
	window = GetNewCWindow(rPictDocWindID, nil, (WindowPtr)-1 );

	SetWinWindow( win, (CGrafPtr)window );	// save a reference to the window in the DocumentRecord
	SetWindowWinHandle ( window, win );		// save a reference to the DocumentRecord in the window

	SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port

	rect = GetPictDocPictRect( win );
		
	h = rect.right - rect.left ;
	v = rect.bottom - rect.top ;
	h += kScrollWidth ;
	v += kScrollWidth + kHeaderHeightBig;
	h = MAX(h, kWindowWidthMin) ;
	v = MAX(v, kWindowHeightMin+kHeaderHeightBig) ;
	
	SizeWindow(	window, h, v, false);		// we'll update everything later

	SetRect( &sizeRect, kWindowWidthMin,
						kWindowHeightMin+kHeaderHeightBig,
						h+1,v+1);
	SetWinSizeRect( win, sizeRect );
	
	// try to make list of embeded profiles
	profList = BuildProfList( win ) ;

	// set up scroll bars
	SetPictDocCntl( win, rPictDocHorzScrollID, GetNewControl(rPictDocHorzScrollID, window ) ) ;
	SetPictDocCntl( win, rPictDocVertScrollID, GetNewControl(rPictDocVertScrollID, window ) ) ;
	SetPictDocCntl( win, rPictDocMatchOnOffID, GetNewControl(rPictDocMatchOnOffID, window ) ) ;
	SetPictDocCntl( win, rPictDocMatchPopupID, GetNewControl(rPictDocMatchPopupID, window ) ) ;
	CallWinResizeProc( win ) ;

	// and if we set up the palette earlier assign it to the window				
	palette = GetPictDocPalette( win ) ;
//	if( palette != nil )
//		SetPalette ( window, palette, true );
//	ActivatePalette ( window );	
	
	// set the name of this document window
	spec = GetWinFSSpec( win ) ;
	SetWTitle( window, spec.name ) ;
	
	// make sure it is visible
	ShowWindow( window ) ;
	
	return err ;
}


/*------------------------------------------------------------------------------*\
	PictDocActionProc
 *------------------------------------------------------------------------------*
\*------------------------------------------------------------------------------*/
pascal void PictDocActionProc ( ControlHandle cntl, short cntlPart)
{
	ControlHandle	hCtl, vCtl ;
	WindowPtr		window ;
	winHandle		win ;
	short			oldCntlVal ;
	
	// get win from cntl
	window = (**cntl).contrlOwner;
	
	// get winHandle from window
	win = GetWindowWinHandle( window ) ;
	
	// get controls from winHandle's data
	hCtl = GetPictDocCntl( win, rPictDocHorzScrollID ) ;
	vCtl = GetPictDocCntl( win, rPictDocVertScrollID ) ;

	if ( cntl==hCtl	|| cntl==vCtl)
	{
		oldCntlVal = GetCtlValue( cntl ) ;
		switch (cntlPart)
		{
			case inUpButton :
					SetCtlValue( cntl, oldCntlVal - 1 );
					break;
			case inDownButton :
					SetCtlValue( cntl, oldCntlVal + 1 );
					break;
			case inPageUp :
					SetCtlValue( cntl, oldCntlVal - 5 );
					break;
			case inPageDown :
					SetCtlValue( cntl, oldCntlVal + 5 );
					break;
			case inThumb :
					break;
		}
		ScrollPane( win, cntl, oldCntlVal ) ;
	}
}


/*------------------------------------------------------------------------------*\
	UpdatePane
 *------------------------------------------------------------------------------*
\*------------------------------------------------------------------------------*/
void UpdatePane ( winHandle win, Rect updateRect )
{
	RGBColor		dkGray = {0xAAAA, 0xAAAA, 0xAAAA};
	Rect			paneRect, paneUpdateRect ;
	Rect			srcRect,pictRect,pictSrcRect;
	CGrafPtr		window ;
	
	paneRect = GetPictDocPaneRect( win ) ;
	pictRect = GetPictDocPictRect( win ) ;					// check if ok?
	window = GetWinWindow( win ) ;
	
	if (SectRect(&paneRect,&updateRect,&paneUpdateRect))
	{
		short	HorzDelta, VertDelta ;
		srcRect = paneUpdateRect ;

		HorzDelta = GetPictDocCntlValue(win,rPictDocHorzScrollID) * kScrollDelta
					+ pictRect.left ;
					
		VertDelta = GetPictDocCntlValue(win,rPictDocVertScrollID) * kScrollDelta
					+ pictRect.top 
					- kHeaderHeightBig;
					
		OffsetRect( &srcRect, HorzDelta, VertDelta ) ;
	
		// Draw the image defined by rect 
		DoDrawPictData( win, (GrafPtr)window, &srcRect, &paneUpdateRect ) ;
		
		// Draw gray for area outside pict but inside content
		if (SectRect(&pictRect,&srcRect,&pictSrcRect))
			if ( !EqualRect( &pictSrcRect, &srcRect ))
			{
				RgnHandle	paneUpdateRgn = NewRgn();
				RgnHandle	pictSrcRgn = NewRgn();
				
				OffsetRect( &pictSrcRect, -HorzDelta, -VertDelta ) ;

				RectRgn( paneUpdateRgn, &paneUpdateRect);
				RectRgn( pictSrcRgn,    &pictSrcRect);
				DiffRgn( paneUpdateRgn, pictSrcRgn, paneUpdateRgn);

				RGBForeColor( &dkGray ) ;
				PaintRgn(paneUpdateRgn);
				DisposeRgn( paneUpdateRgn ) ;
				DisposeRgn( pictSrcRgn ) ;
				
			}
	}
}


/*------------------------------------------------------------------------------*\
	ScrollPane
 *------------------------------------------------------------------------------*
\*------------------------------------------------------------------------------*/
void ScrollPane ( winHandle win, ControlHandle cntl, short oldCntlVal )
{
	short			newCntlVal ;
	short			distHorz=0, distVert=0 ;
	ControlHandle	hCtl, vCtl ;
	Rect			paneRect;
	
	newCntlVal = GetCtlValue( cntl ) ;
	if ( newCntlVal != oldCntlVal )
	{
		RgnHandle		updateRgn = NewRgn();

		hCtl = GetPictDocCntl( win, rPictDocHorzScrollID ) ;
		vCtl = GetPictDocCntl( win, rPictDocVertScrollID ) ;

		paneRect = GetPictDocPaneRect( win );		// determine scrolling rect
		if (cntl==vCtl)
			distVert = (oldCntlVal-newCntlVal)*kScrollDelta ;
		if (cntl==hCtl)
			distHorz = (oldCntlVal-newCntlVal)*kScrollDelta ;
		ScrollRect( &paneRect, distHorz, distVert, updateRgn);
		UpdatePane ( win, (**updateRgn).rgnBBox ) ;
		DisposeRgn(updateRgn);
	}
}



void UpdatePict ( winHandle win, short which )
{
	CMError		cmerr ;
	PicHandle	pictNew ;
	PicHandle	pictOrig ;
	long		ticks ;
	Boolean		needsUpdate ;
	
	if (which==kOriginal) return ;
	if (which>kChecked) return ;
	
	needsUpdate = GetPictDocNeedsUpdate( win, which ) ;
	pictNew = GetPictDocPict( win, which ) ;
	
	//if ( (needsUpdate==false) && (pictNew!=nil) ) return;
	if ( needsUpdate==false ) return;

	if (pictNew)
		DisposeHandle( (Handle)pictNew ) ;
	
	SetUpCursors(121, 153);
	
	pictOrig = GetPictDocPict(win,kOriginal) ;
	
	ticks = TickCount();

	switch (which)
	{
		case kMatched :
			cmerr = MatchPicHandle(
						pictOrig, &pictNew,
						GetPictDocProfile( win, kSrceProf ),
						GetPictDocProfile( win, kPrevProf ) ) ;
			break;
		case kProofed :
			cmerr = ProofPicHandle(
						pictOrig, &pictNew,
						GetPictDocProfile( win, kSrceProf ),
						GetPictDocProfile( win, kDestProf ),
						GetPictDocProfile( win, kPrevProf ) ) ;
			break;
		case kChecked :
			cmerr = CheckPicHandle(
						pictOrig, &pictNew,
						GetPictDocProfile( win, kSrceProf ),
						GetPictDocProfile( win, kDestProf ) ) ;
			break;
	}
							
	ticks = TickCount() - ticks;						 

	TearDownCursors();
	SetCursor(&(qd.arrow)) ;

	if (cmerr)
	{
		ticks = 0 ;
		AlertUser( cmerr ) ;
	}
	
	SetPictDocDrawTime( win, which, ticks ) ;
	SetPictDocPict( win, which, pictNew );
	SetPictDocNeedsUpdate( win, which, false ) ;
}


void ChangeCSOnOff ( winHandle win )
{
	Rect			rect;
	short			val;
	
	val = GetPictDocCntlValue ( win, rPictDocMatchOnOffID ) ;
	SetPictDocCntlValue ( win, rPictDocMatchOnOffID, !val ) ;
	
	SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port
	rect = GetPictDocPaneRect( win ) ;
	rect.bottom += kScrollWidth ;			// inval status rect
	InvalRect( &rect ) ;
}


void ChangeCSMode ( winHandle win )
{
	Rect			rect;
	short			which;
	
	which = GetPictDocCntlValue ( win, rPictDocMatchPopupID ) ;
	if ( GetPictDocPict(win,which) == nil )
		if ( GetPictDocNeedsUpdate(win,which) == false )
			SetPictDocNeedsUpdate( win, which, true) ;
			
	SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port
	rect = GetPictDocPaneRect( win ) ;
	rect.bottom += kScrollWidth ;			// inval status rect
	InvalRect( &rect ) ;
}


Boolean	SetWinContentColor ( WindowPtr window, RGBColor theColor ) 
{
	AuxWinHandle	awh ;
	Boolean			hasAuxWinRec ;
	CTabHandle		ctab ;
	short			count, i ;
	
	hasAuxWinRec = GetAuxWin( window, &awh ) ;
	if (hasAuxWinRec)
	{
		ctab = (**awh).awCTable ;
		count = (**ctab).ctSize ;
		
		for (i=0; i<count; i++)
			if ( (**ctab).ctTable[i].value == 0 )
				 (**ctab).ctTable[i].rgb = theColor ;
	}
	return hasAuxWinRec ;
}


OSErr SetPictDocEmbdProf( winHandle win, CMProfileRef prof )
{
	PicHandle	pictOld, pictNew ;
	OSErr		err = noErr ;
	Rect		rect ;
	
	if (prof==nil) return paramErr ;

	pictOld = GetPictDocPict( win, kOriginal ) ;
	
	err = PrependProfileToPicHandle ( pictOld, &pictNew, prof ) ;
	if (err) return err ;
	
	SetPictDocPict( win, kOriginal, pictNew ) ;		// set new pict
	
	DisposeHandle( (Handle)pictOld ) ;				// dispose old pict

	SetPictDocNeedsUpdate( win, kMatched, true ) ;	// update booleans
	SetPictDocNeedsUpdate( win, kProofed, true ) ;
	SetPictDocNeedsUpdate( win, kChecked, true ) ;
	
	SetWinDirty( win, true ) ;
	
	LDispose( GetPictDocProfList(win) ) ;			// dispose old list of embedded profiles 
	BuildProfList( win ) ;							// make new list of embeded profiles
	
	SetGWorld( GetWinWindow( win ), nil ) ;			// set the window to be the current port
	rect = GetWinRect( win ) ;
	//if ( GetPictDocCntlValue(win,rPictDocMatchOnOffID) )
	// rect = header rect
	InvalRect( &rect ) ;							// force window redraw

	return err ;
}
