// A profile list window
//
// David Hayward 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the menus routines for a profile list window.
// 
// 12/13/94	david	first cut


#include <PictUtil.h>
#include <Fonts.h>
#include <Files.h>
#include <QDOffScreen.h>
#include <Resources.h>
#include <GXTypes.h>

#include "appGlobals.h"
#include "appMain.h"
#include "appMenus.h"

#include "win.h"
#include "winProfList.h"
#include "winProfListGetSet.h"
#include "winProfListMenus.h"
#include "winProfile.h"
#include "winProfileGetSet.h"
#include "winPictDoc.h"

#include "ErrorHandling.h"
#include "aeSenders.h"
#include "faux3DStuff.h"
#include "colorsyncStuff.h"



/**\
|**| ==============================================================================
|**| PRIVATE DEFINES
|**| ==============================================================================
\**/
#define rProfListWindID			4000			// 'WIND' resource id
#define kWindowWidthMin			300
#define kWindowHeightMin		40
#define kHeaderHeight			20
#define kCellHeight				16


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
ListHandle	BuildProfList			( winHandle win ) ;
OSErr		DoCreateProfListWindow	( winHandle win ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	winActivateProfList
 *------------------------------------------------------------------------------*
		This is a ActivateProcPtr for ProfList windows.
		So far all this does is update menus when its window becomes active.
		Other things, such as activating/deactivation controls should also go here
		This ProcPtr is envoked by CallWinActivateProc() which is called by:
			DoActivateEvent() which dispaches activate events and
			DoOSEvent() which handles events such as Suspend/Resume.
\*------------------------------------------------------------------------------*/
void	winActivateProfList ( winHandle win, Boolean becomingActive ) 
{
	if (becomingActive)
	{
		SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port
		winAdjustMenusProfList( win ) ;
	}
}


/*------------------------------------------------------------------------------*\
	winUpdateProfList
 *------------------------------------------------------------------------------*
		This is a UpdateProcPtr for ProfList windows.  
		It handles all drawing into the window.
		For ProfList windows, this means that this routine is responible for
		drawing the grow box, the header area, and the list itself.
		This ProcPtr is envoked by CallWinUpdateProc() which is called by:
			DoUpdateEvent() which dispaches update events.
\*------------------------------------------------------------------------------*/
void	winUpdateProfList ( winHandle win, EventRecord *e )
{
	Rect			docRect ;
	Rect			headRect, headUpdateRect ;
	Rect			growRect, growUpdateRect ;
	Rect			contRect, contUpdateRect ;
	WindowPtr		window = (WindowPtr)e->message;
	Rect 			updateRect = ((**(window->visRgn)).rgnBBox);
	GrafPtr			savedPort ;
	
	GetPort(&savedPort ) ;	
	SetPort( window );

	BeginUpdate( window );


	docRect = GetWinRect( win ) ;		// check if ok?
	
// draw header	
	headRect = docRect ;
	headRect.bottom = kHeaderHeight ;
	headRect.bottom -= 3;
	if (SectRect(&headRect,&updateRect,&headUpdateRect))
	{
		faux3DFillRect ( &headRect ) ;
		ForeColor(whiteColor);
		MoveTo (headRect.left, headRect.bottom+1); LineTo (headRect.right, headRect.bottom+1);
		ForeColor(blackColor);
		MoveTo (headRect.left, headRect.bottom  ); LineTo (headRect.right, headRect.bottom  );
		MoveTo (headRect.left, headRect.bottom+2); LineTo (headRect.right, headRect.bottom+2);
	}

// update controls
	DrawControls( window );
	
// draw content
	TextSize(9);
	TextFace(0);
	TextFont(geneva);

	contRect = docRect ;
	contRect.top += kHeaderHeight ;
	contRect.bottom -= kScrollWidth ;
	contRect.right -= kScrollWidth ;
	if (SectRect(&contRect,&updateRect,&contUpdateRect))
	{
		ListHandle list ;
		list = GetProfListList( win );
		LDoDraw( true, list );
		LUpdate( window->visRgn, list);		// not sure this is the rgn
	}

// draw GrowIcon
	growRect = docRect ;
	growRect.top += kHeaderHeight ;
	if (SectRect(&growRect,&updateRect,&growUpdateRect))
	{
		RgnHandle saveRgn = NewRgn() ;
		GetClip( saveRgn ) ;
		ClipRect( &growRect ) ;
		DrawGrowIcon( window ) ;
		SetClip( saveRgn ) ;
		DisposeRgn( saveRgn ) ;
	}
	
// signal that we finished drawing
	EndUpdate( window );
	SetPort( savedPort ) ;
}


/*------------------------------------------------------------------------------*\
	winClickProfList
 *------------------------------------------------------------------------------*
		This is a ClickProcPtr for ProfList windows.
		So far, all this contains is a crude handling of the scroll controls
		Other things, such as initiating a drag-and-drop should also go here
		This ProcPtr is envoked by CallWinClickProc() which is called by:
			DoMouseDownEvent() which dispaches mouse down events 
\*------------------------------------------------------------------------------*/
void	winClickProfList ( winHandle win, EventRecord *e ) 
{
	Point				pt = e->where;
	WindowPtr			window ;
	Boolean				dblClk ;
	ListHandle			list ;
	Point				cell = {0,0} ;
	CMProfileSearchRef	search ;
	FSSpec				spec ;
	unsigned long		index ;
	OSErr				err ;
	
	window = (WindowPtr) GetWinWindow ( win ) ;
	list = GetProfListList( win ) ;
	search = GetProfListSearchResult( win ) ;
	
	SetPort (window ) ;
	GlobalToLocal( &pt );
	
	//? check to see if clist is in list
	
	dblClk = LClick( pt, e->modifiers, list ) ;
	
	if ( dblClk )
	{
		while ( LGetSelect(true, &cell, list))
		{
			index = cell.v ;
			if (index==0)
			{
				winHandle		profWin ;
				// create a new winHandle of the proper type
				err = NewWinHandle( &profWin, winAllocProfile ) ;
				FailIfErr( err ) ;
				
				SetProfileRef( profWin, nil ) ;
				SetProfileSubType( profWin, kSysProfSubType ) ;	// set subtype
				err = CallWinOpenProc( profWin ) ;
			
				if ( err != noErr )			//if an error occured
					DisposeWinHandle( profWin );
				if ( err == kWasAlreadyOpen )
					err = noErr;
			}
			else
			{
				err = CMSearchGetIndProfileFileSpec( search, index, &spec) ;
				SendOpenDoc( &spec ) ;
			}
			LNextCell( false, true, &cell, list ) ;
		}
	}
	
	dimAllMenus() ;				// dim all menus
	DoAppAdjustMenus() ;		// undim app items
	winAdjustMenusProfList( win ) ;
}


/*------------------------------------------------------------------------------*\
	winResizeProfList
 *------------------------------------------------------------------------------*
		This is a ResizeProcPtr for ProfList windows.
		This routine is responsible for re-arranging the contents of the window if
		it is resized.  On entry to this routine, the new window size can be found
		in the document's window's portRect.  At the very least, this routine 
		should call SetWinRect() to put the new window size in the winHandle.
		So far, the only other thing this routine does is repostion the scroll controls
		and issue the appropriate update events so everything is re-drawn correctly.
		This ProcPtr is envoked by CallWinResizeProc() which is called by:
			DoMouseDownEvent() which dispaches mouse down events and
			DoCreatePictWindow() which initially creates the window.
\*------------------------------------------------------------------------------*/
void	winResizeProfList ( winHandle win ) 
{
	ProfListDataHdl	data ;
	WindowPtr		window ;
	Rect			horzInval, vertInval ;
	Rect			oldDocRect, newDocRect ;
	

	oldDocRect = GetWinRect( win );

	data = (ProfListDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return;
	
	window = (WindowPtr) GetWinWindow ( win ) ;
	newDocRect = window->portRect ;
	SetWinRect( win, newDocRect );

	horzInval = vertInval = newDocRect ;
	horzInval.top  = MIN( oldDocRect.bottom, newDocRect.bottom) - kScrollWidth;
	vertInval.left = MIN( oldDocRect.right, newDocRect.right)   - kScrollWidth;
	EraseRect( &horzInval ) ;
	EraseRect( &vertInval ) ;
	
	LSize(	newDocRect.right - kScrollWidth,
			newDocRect.bottom - kHeaderHeight - kScrollWidth,
			GetProfListList(win) );

	InvalRect( &horzInval ) ;
	InvalRect( &vertInval ) ;
	
	return ;
}


/*------------------------------------------------------------------------------*\
	winCloseProfList
 *------------------------------------------------------------------------------*
		This is a CloseProcPtr for ProfList windows.
		So far, all this does is call DisposeWinHandle() and return all the menus
		to the default state by calling dimAllMenus() and DoAppAdjustMenus().
		This is needed because, if the window being closed is the only window,
		then there is not another window to trigger an activate event. 
		This win type doesn't need to be saved so ther is no reason to prompt the
		user to save the file here.
		This ProcPtr is envoked by CallWinCloseProc() which is called by:
			DoMouseDownEvent() which dispached click events (e.g. close box), and
			MenuProcPtrs which are called whenever menu events occur.
\*------------------------------------------------------------------------------*/
void	winCloseProfList ( winHandle win ) 
{
	// could do something like ask the user to save if dirty
	DisposeWinHandle( win ) ;
	
	// we have to update the menu bar just in case
	// this window is the only window
	
	dimAllMenus() ;				// dim all menus
	DoAppAdjustMenus() ;		// undim app items
	DrawMenuBar() ;
	
	return ;
}


/*------------------------------------------------------------------------------*\
	winMenuProfList
 *------------------------------------------------------------------------------*
		This is a MenuProcPtr for ProfList windows.
		This routine dispatches any menu commands that the window can handle
		to the appropriate function.
		This ProcPtr is envoked by CallWinMenuProc() which is called by:
			HandleMenuCommand() which dispatches all menu events.
\*------------------------------------------------------------------------------*/
void winMenuProfList ( winHandle win, long menuResult )
{
	short			menuID;
	short			menuItem;
	

	menuID   = HiWrd(menuResult) ;
	menuItem = LoWrd(menuResult) ;
	switch ( menuID )
	{
		case mFile:
			switch ( menuItem )
			{
				case iClose:			// close an open window/document
					CallWinCloseProc( win ) ;
					break ;
			}
			break;
			
		case mProfiles:
			switch ( menuItem )
			{
				case iMakeSrceProf:
					ProfListMakeProfileCmnd( win, kSrceProf ) ;
					break ;
				case iMakeDestProf:
					ProfListMakeProfileCmnd( win, kDestProf ) ;
					break ;
				case iMakePrevProf:
					ProfListMakeProfileCmnd( win, kPrevProf ) ;
					break ;
				case iMakeEmbdProf:
					ProfListMakeEmbededCmnd( win ) ;
					break ;
			}
			break;
	}
	HiliteMenu(0);		// Unhighlight whatever MenuSelect or MenuKey hilited
}


/*------------------------------------------------------------------------------*\
	winAllocProfList
 *------------------------------------------------------------------------------*
		This is a AllocProcPtr for ProfList windows.
		This routine is responsible for filling in all the needed fields of the
		winHandle structure.  This routine shouldn't be called directly.
		Instead, the code which needs to create a document of this type should
		call NewWinHandle(win,winAllocProfList) to envoke this function.
		This ProcPtr is envoked by NewWinHandle() which is called by:
			DoProfListCommand() which handles the Profile:CS List menu command.
\*------------------------------------------------------------------------------*/
OSErr winAllocProfList ( winHandle win )
{
	OSErr			err = noErr ;
	ProfListDataHdl	data ;
	
	// set the window type
	SetWinType( win, kProfListType ) ;
	
	// allocate data handle
	data = (ProfListDataHdl)NewHandleClear( sizeof(ProfListDataRec) ) ;	
	if ( data == nil ) return MemError();
	SetWinData ( win, (Handle)data ) ;
	
	// stuff winHandle fields
	SetWinActivateProc		( win,    (ActivateProcPtr) winActivateProfList );
	SetWinUpdateProc		( win,      (UpdateProcPtr) winUpdateProfList );
	SetWinClickProc			( win,       (ClickProcPtr) winClickProfList );
	SetWinKeyProc			( win,         (KeyProcPtr) nil );
	SetWinResumeProc		( win,      (ResumeProcPtr) nil );
	SetWinNullProc			( win,        (NullProcPtr) nil );
	SetWinMenuProc			( win,        (MenuProcPtr) winMenuProfList );
	SetWinResizeProc		( win,      (ResizeProcPtr) winResizeProfList );
	
	SetWinNewProc			( win,         (NewProcPtr) winNewProfList );
	SetWinOpenProc			( win,        (OpenProcPtr) nil );
	SetWinCloseProc			( win,       (CloseProcPtr) winCloseProfList );
	SetWinDisposeProc		( win,     (DisposeProcPtr) winDisposeProfList );
	
	SetWinPageCountProc		( win,   (PageCountProcPtr) nil );
	SetWinPagePrintProc		( win,   (PagePrintProcPtr) nil );
		
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winNewProfList
 *------------------------------------------------------------------------------*
		This is a NewProcPtr for ProfList windows.
		This routine is responsible for building a new ProfList window after all the
		needed fields of the winHandle structure have been filled in by winAllocProfList.
		If an ProfList window is already open, then this routine will bring it
		to the front instead of opening a second window.
		This routine shouldn't be called directly.  Instead, the code which needs
		to create a document of this type should call NewWinHandle(win,winAllocProfList),
		and then CallWinNewProc(win) to envoke this function.
		This ProcPtr is envoked by CallWinNewProc() which is called by:
			DoProfListCommand() which handles the Profile:CS List menu command.
\*------------------------------------------------------------------------------*/
OSErr winNewProfList ( winHandle win )
{
	OSErr			err = noErr ;
	winHandle		existingDoc ;
	
	// see if we are already open
	
 	if ( FindWinHandle ( nil, kProfListType, 1, 1, &existingDoc ) == 1 )
	{
		// bring it to the front
		SelectWindow( (WindowPtr) GetWinWindow(existingDoc) ) ;		
		return kWasAlreadyOpen ;
	}

	err = DoCreateProfListWindow( win ) ;
	if ( err ) return err ;
		
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winDisposeProfList
 *------------------------------------------------------------------------------*
		This is a DisposeProcPtr for ProfList windows.
		This routine is responsible for disposing of any data that was allocated
		by the document and stored in the its data handle.
		This ProcPtr is envoked by CallWinDisposeProc() which is called by:
			DisposeWinHandle() which disposes of the entire winHandle.
\*------------------------------------------------------------------------------*/
void winDisposeProfList ( winHandle win )
{
	ProfListDataHdl		data ;
	ListHandle			list ;
	CMProfileSearchRef	search ;

	data = (ProfListDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return;
	
	list = GetProfListList ( win ) ;
	if ( list != nil )
		LDispose(list) ;
	
	search = GetProfListSearchResult( win ) ;
	if ( search != nil )
		CMDisposeProfileSearch(search) ;
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------*/
static ListHandle BuildProfList ( winHandle win )
{
	ListHandle			theList = nil ;
	unsigned long		count ;
	CMError				cmerr ;


//------------- search for all profiles
	{
		CMSearchRecord		searchSpec ;
		CMProfileSearchRef	searchResult ;
		
		searchSpec.searchMask = cmMatchAnyProfile ;
		searchSpec.filter = nil ;
		
		cmerr = CMNewProfileSearch( &searchSpec, nil, &count, &searchResult) ;
		if (cmerr) goto bail ;
		
		SetProfListSearchResult	( win, searchResult ) ;
		SetProfListCount ( win, count ) ;
	}
	
	
//------------- build a list
	{
		Rect		rView = {	kHeaderHeight,		0,
								kWindowHeightMin,	kWindowWidthMin-kScrollWidth } ;
		Rect		rDataBnds ;
		Point		cellSize = {kCellHeight, kWindowWidthMin-kScrollWidth} ;
		WindowPtr	window ;

		window = (WindowPtr) GetWinWindow ( win ) ;
		SetRect( &rDataBnds, 0,0,1,count+1 ) ;
		theList = LNew(&rView,
						&rDataBnds,
						cellSize,
						0,						// proc ID
						window,
						false,					// draw now
						true,					// hasGrow
						false,					// scrollHoriz
						true ) ;				// scrollVert
		SetProfListList( win, theList ) ;
	}


//------------- 
	{
		Point				theCell ;
		unsigned long		v ;
		short				dataLen ;
		Str255				name ;
		CMProfileRef		prof ;

		cmerr = GetProfName( nil, (StringPtr)name ) ;
		SetPt(&theCell,0,0);
		dataLen = name[0] ;
		LSetCell( &(name[1]), dataLen, theCell, theList ) ;
		
		for ( v=1; v<=count; v++ )
		{
			cmerr = CMSearchGetIndProfile( GetProfListSearchResult(win), v, &prof) ;
			cmerr = GetProfName( prof, (StringPtr)name ) ;
			CMCloseProfile( prof ) ;
			SetPt(&theCell,0,v);
			dataLen = name[0] ;
			LSetCell( &(name[1]), dataLen, theCell, theList ) ;
		}
	}
	
//-------------
bail :
	return theList ;
}


/*------------------------------------------------------------------------------*\
	DoCreateProfListWindow
 *------------------------------------------------------------------------------*
		This routine actually creates the WindowPtr for a winHandle.
		After creating the window, it:
			reference the window and the DocumentRecord to each other,
			makes the window the current port,
			loads and adds the scroll bar controls,
			resizes the window according to the #defined constants,
			makes it visible .
		This routine is called by:
			winOpenProfList() which fills in the winHandle structure
\*------------------------------------------------------------------------------*/
OSErr DoCreateProfListWindow ( winHandle win )
{

	Rect			sizeRect ;
	OSErr			err = noErr;
	WindowPtr		window ;
	ProfListDataHdl	data ;
	short			h,v ;
	ListHandle		profList ;
	unsigned long	count ;
	
	data = (ProfListDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return nilHandleErr;
	
	// create the window
	window = GetNewCWindow(rProfListWindID, nil, (WindowPtr)-1 );

	SetWinWindow( win, (CGrafPtr)window );	// save a reference to the window in the DocumentRecord
	SetWindowWinHandle ( window, win );		// save a reference to the DocumentRecord in the window

	SetGWorld( GetWinWindow(win), nil ) ;	// set the window to be the current port

	// try to read the ColorSync Profiles Folder
	profList = BuildProfList( win ) ;
	count = GetProfListCount( win ) + 1 ;	// add one for the "System Profile" at top of list
	
	h = kWindowWidthMin ;
	v = MIN(count,20) * kCellHeight ;
	v += kScrollWidth + kHeaderHeight;
	v = MAX(v, kWindowHeightMin+kHeaderHeight) ;
	
	SizeWindow(	window, h, v, false);		// we'll update everything later

	SetRect( &sizeRect, kWindowWidthMin,
					    kWindowHeightMin+kHeaderHeight,
					    h+1, v+1);
	SetWinSizeRect( win, sizeRect );
	
	CallWinResizeProc( win ) ;
	
	// make sure it is visible
	ShowWindow( window ) ;
	
	return err ;
}


