// A profile window
//
// David Hayward 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the menus routines for a profile window.
// 
// 12/13/94	david	first cut


#include <QuickDraw.h>
#include <Icons.h>
#include <Fonts.h>
#include <Files.h>
#include <TextUtils.h>
#include <QDOffScreen.h>
#include <Resources.h>

#include "stringStuff.h"

#include "appGlobals.h"
#include "appMain.h"
#include "appMenus.h"

#include "win.h"
#include "winProfile.h"
#include "winProfileGetSet.h"
#include "winProfileMenus.h"
#include "winPictDoc.h"

#include "colorsyncStuff.h"
#include "fileStuff.h"
#include "resourceStuff.h"


/**\
|**| ==============================================================================
|**| PRIVATE TYPEDEFS
|**| ==============================================================================
\**/
typedef struct OSTypeList
{
	short	count ;			// 1-based count of structs to follow 
	struct
	{
		OSType	type ;
		Str27	string ;
	} entry[];
} OSTypeList, *OSTypeListPtr, **OSTypeListHdl ;


typedef struct FlagsList
{
	short	shift ;			// how many bits to shift right 
	short	count ;			// 1-based count of structs to follow
	struct
	{
		Str27	string ;
	} entry[];
} FlagsList, *FlagsListPtr, **FlagsListHdl ;


/**\
|**| ==============================================================================
|**| PRIVATE DEFINES
|**| ==============================================================================
\**/
#define rProfileWindID				1000			// 'WIND' resource id
#define rProfileLabelsStringID		1000			// 'STR ' resource id
#define rProfileVarsStringID		1001			// 'STR ' resource id
#define rProfileWhereVarStringID	1002			// 'STR ' resource id
#define rProfileIconID				1003			// 'WIND' resource id


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
void		DoDrawProfData			( winHandle win ) ;
void		MungeProfileHeader1		( CMProfileRef prof, Handle text ) ;
void		MungeProfileHeader2		( CMProfileRef prof, Handle text ) ;
OSErr		RendIntent2String		( unsigned long intent, StringPtr dest ) ;
OSErr		ColorSpace2String		( OSType spaceType, StringPtr dest ) ;
OSErr		ProfileClass2String		( OSType classType, StringPtr dest ) ;
OSErr		DoCreateProfWindow		( winHandle win ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	winActivateProfile
 *------------------------------------------------------------------------------*
		This is a ActivateProcPtr for Profile windows.
		So far all this does is update menus when its window becomes active.
		Other things, such as activating/deactivation controls should also go here
		This ProcPtr is envoked by CallWinActivateProc() which is called by:
			DoActivateEvent() which dispaches activate events and
			DoOSEvent() which handles events such as Suspend/Resume.
\*------------------------------------------------------------------------------*/
void	winActivateProfile ( winHandle win, Boolean becomingActive ) 
{
	if (becomingActive)
	{
		SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port
		winAdjustMenusProfile( win ) ;
	}
}


/*------------------------------------------------------------------------------*\
	winUpdateProfile
 *------------------------------------------------------------------------------*
		This is a UpdateProcPtr for Profile windows.  
		It handles all drawing into the window.
		For ProfList windows, this means that this routine is responible for
		drawing the the basic CS1 and CS2 profile info into the window.
		This ProcPtr is envoked by CallWinUpdateProc() which is called by:
			DoUpdateEvent() which dispaches update events.
\*------------------------------------------------------------------------------*/
void	winUpdateProfile ( winHandle win, EventRecord *e )
{
	WindowPtr		window = (WindowPtr)e->message;
	GrafPtr			savedPort ;
	
	GetPort(&savedPort ) ;	
	SetPort( window );

	BeginUpdate( window );

	DoDrawProfData( win ) ;
	
	// signal that we finished drawing
	EndUpdate( window );
	SetPort( savedPort ) ;
}


/*------------------------------------------------------------------------------*\
	winCloseProfile
 *------------------------------------------------------------------------------*
		This is a CloseProcPtr for Profile windows.
		So far, all this does is call DisposeWinHandle() and return all the menus
		to the default state by calling dimAllMenus() and DoAppAdjustMenus().
		This is needed because, if the window being closed is the only window,
		then there is not another window to trigger an activate event. 
		This win type doesn't need to be saved so ther is no reason to prompt the
		user to save the file here.
		This ProcPtr is envoked by CallWinCloseProc() which is called by:
			DoMouseDownEvent() which dispached click events (e.g. close box), and
			MenuProcPtrs which are called whenever menu events occur.
\*------------------------------------------------------------------------------*/
void	winCloseProfile ( winHandle win ) 
{
	// could do something like ask the user to save if dirty
	DisposeWinHandle( win ) ;
	
	// we have to update the menu bar just in case
	// this window is the only window
	
	dimAllMenus() ;				// dim all menus
	DoAppAdjustMenus() ;		// undim app items
	DrawMenuBar() ;
	
	return ;
}


/*------------------------------------------------------------------------------*\
	winMenuProfile
 *------------------------------------------------------------------------------*
		This is a MenuProcPtr for Profile windows.
		This routine dispatches any menu commands that the window can handle
		to the appropriate function.
		This ProcPtr is envoked by CallWinMenuProc() which is called by:
			HandleMenuCommand() which dispatches all menu events.
\*------------------------------------------------------------------------------*/
void winMenuProfile ( winHandle win, long menuResult )
{
	short			menuID;
	short			menuItem;
	

	menuID   = HiWrd(menuResult) ;
	menuItem = LoWrd(menuResult) ;
	switch ( menuID )
	{
		case mFile:
			switch ( menuItem )
			{
				case iClose:			// close an open window/document
					CallWinCloseProc( win ) ;
					break ;
			}
			break;
			
		case mProfiles:
			switch ( menuItem )
			{
				case iMakeSrceProf:
					ProfMakeProfileCmnd( win, kSrceProf ) ;
					break ;
				case iMakeDestProf:
					ProfMakeProfileCmnd( win, kDestProf ) ;
					break ;
				case iMakePrevProf:
					ProfMakeProfileCmnd( win, kPrevProf ) ;
					break ;
				case iMakeEmbdProf:
					ProfMakeEmbededCmnd( win ) ;
					break ;
			}
			break;
	}
	HiliteMenu(0);		// Unhighlight whatever MenuSelect or MenuKey hilited
}


/*------------------------------------------------------------------------------*\
	winAllocProfile
 *------------------------------------------------------------------------------*
		This is a AllocProcPtr for Profile windows.
		This routine is responsible for filling in all the needed fields of the
		winHandle structure.  This routine shouldn't be called directly.
		Instead, the code which needs to create a document of this type should
		call NewWinHandle(win,winAllocProfile) to envoke this function.
		This ProcPtr is envoked by NewWinHandle() which is called by:
			app_aeODOC_handler() which handles ODOC AppleEvents, and
			app_aePDOC_handler() which handles PDOC AppleEvents.
\*------------------------------------------------------------------------------*/
OSErr winAllocProfile ( winHandle win )
{
	OSErr			err = noErr ;
	ProfileDataHdl	data ;
		
	// set the window type
	SetWinType( win, kProfileType ) ;
	
	// stuff winHandle fields
	SetWinActivateProc		( win,    (ActivateProcPtr) winActivateProfile );
	SetWinUpdateProc		( win,      (UpdateProcPtr) winUpdateProfile );
	SetWinClickProc			( win,       (ClickProcPtr) nil );
	SetWinKeyProc			( win,         (KeyProcPtr) nil );
	SetWinResumeProc		( win,      (ResumeProcPtr) nil );
	SetWinNullProc			( win,        (NullProcPtr) nil );
	SetWinMenuProc			( win,        (MenuProcPtr) winMenuProfile );
	SetWinResizeProc		( win,      (ResizeProcPtr) nil );
	
	SetWinNewProc			( win,         (NewProcPtr) nil );
	SetWinOpenProc			( win,        (OpenProcPtr) winOpenProfile );
	SetWinCloseProc			( win,       (CloseProcPtr) winCloseProfile );
	SetWinDisposeProc		( win,     (DisposeProcPtr) winDisposeProfile );

	SetWinPageCountProc		( win,   (PageCountProcPtr) nil );
	SetWinPagePrintProc		( win,   (PagePrintProcPtr) nil );
	
	// allocate data handle
	data = (ProfileDataHdl)NewHandleClear( sizeof(ProfileDataRec) ) ;	
	if( data == nil ) return MemError();
	SetWinData( win, (Handle)data ) ;
	
	// stuff ProfileDataHdl fields
	SetProfileSubType( win, kFileSubType ) ;	// set default subtype
	
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winOpenProfile
 *------------------------------------------------------------------------------*
		This is a OpenProcPtr for Profile windows.
		This routine is responsible for opening a Profile window after all the
		needed fields of the winHandle structure have been filled in by winAllocProfile.
		On entry the DocumentRecord must contain a valid FSSpec.
		If a Profile window for the FSSpec is already onen, then this routine 
		will bring it to the front instead of opening a second window.
		This routine shouldn't be called directly.  Instead, the code which needs
		to create a document of this type should call NewWinHandle(win,winAllocProfile),
		SetWinFSSpec (win,spec), and then CallWinOpenProc(win) to envoke this function.
		This ProcPtr is envoked by CallWinOpenProc() which is called by:
			app_aeODOC_handler() which handles ODOC AppleEvents, and
			app_aePDOC_handler() which handles PDOC AppleEvents.
\*------------------------------------------------------------------------------*/
OSErr winOpenProfile ( winHandle win )
{
	OSErr				err = noErr ;
	winHandle			existingWin ;
	FSSpec				spec ;
	CMProfileRef 		prof ;
	CMProfileLocation	profLoc ;
	OSType				subtype ;	// this is the subtype of window ('file' 'embd' or 'sysp')
	unsigned long		index ;		// if embeded subtype then this tells us which one
	Boolean				alreadyOpen = false;
	short				count, i ;
	
	spec = GetWinFSSpec( win );
	subtype = GetProfileSubType( win ) ;
	index = GetProfileIndex( win ) ;

	// see if already open
	switch (subtype)
	{
		case kFileSubType :
		 	if ( FindWinHandle( &spec, kProfileType, 1, 1, &existingWin ) == 1 )
		 		alreadyOpen = true ;
		 	break ;
		 	
		case kEmbededSubType :
			count = FindWinHandle( &spec, kProfileType, 0, 0, nil ) ;
			for (i=1; i<=count && !alreadyOpen; i++)
			{
				if ( FindWinHandle( &spec, kProfileType, i, 1, &existingWin ) == 1 )
					if ( GetProfileSubType(existingWin) == subtype )
						if ( GetProfileIndex(existingWin) == index )
							alreadyOpen = true ;
			}
		 	break ;
		 	
		case kSysProfSubType :
			count = FindWinHandle( nil, kProfileType, 0, 0, nil ) ;
			for (i=1; i<=count && !alreadyOpen; i++)
			{
				if ( FindWinHandle( nil, kProfileType, i, 1, &existingWin ) == 1 )
					if ( GetProfileSubType(existingWin) == subtype )
						alreadyOpen = true ;
			}
		 	break ;
	}
 	
 	if (alreadyOpen)
	{
		// bring it to the front
		SelectWindow( (WindowPtr) GetWinWindow(existingWin) ) ;		
		return kWasAlreadyOpen ;
	}

	switch (subtype)
	{
		case kFileSubType :
			err = OpenProfileFSSpec( spec, &prof ) ;
			if ( err ) return err;
			SetProfileRef( win, prof ) ;			// set the window's data's profile ref
			break ;
			
		case kEmbededSubType :
			//? the prof is valid
			//? the spec is the spec of the pict file
			break ;
			
		case kSysProfSubType :
			err = CMGetSystemProfile( &prof ) ;
			err = CMGetProfileLocation( prof, &profLoc) ;
			if ( err ) return err ;
			if ( profLoc.locType != cmFileBasedProfile )
				return fnfErr ;						//? should also close prof
			spec = profLoc.u.fileLoc.spec ;
			SetWinFSSpec( win, &spec ) ;
			SetProfileRef( win, prof ) ;			// this should be nil already
			break ;
	}
	
	err = DoCreateProfWindow( win ) ;
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winDisposeProfile
 *------------------------------------------------------------------------------*
		This is a DisposeProcPtr for Profile windows.
		This routine is responsible for disposing of any data that was allocated
		by the document and stored in the its data handle.
		This ProcPtr is envoked by CallWinDisposeProc() which is called by:
			DisposeWinHandle() which disposes of the entire winHandle.
\*------------------------------------------------------------------------------*/
void winDisposeProfile ( winHandle win )
{
	ProfileDataHdl	data ;
	CMProfileRef	prof ;

	data = (ProfileDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return;

	prof = GetProfileRef( win ) ;
	if ( prof != nil )
		CMCloseProfile( prof ) ;
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


void DoDrawProfData ( winHandle win ) 
{	
	long				length ;
	Rect				labelRect, varsRect, whereRect, iconRect ; // = {7,53,39,85} ;
	Handle				labelText, varText,  whereText ;
	CMProfileRef		prof ;	
	CMError				cmerr ;
	OSType				subtype ;
	unsigned long		vers ;
	Str255				name, path ;
	FSSpec				spec ;

// get rects from resources
	labelRect = GetRect ( rProfileLabelsStringID ) ;
	varsRect = GetRect ( rProfileVarsStringID ) ;
	whereRect = GetRect ( rProfileWhereVarStringID ) ;
	iconRect = GetRect ( rProfileIconID ) ;

// draw icon
	PlotIconID( &iconRect, atNone, ttNone, rProfileIconID ) ;

// draw labels
	TextSize(9);
	TextFace(bold);
	TextFont(geneva);

	labelText = GetResource( 'TEXT', rProfileLabelsStringID ) ;
	
	HLock( labelText ) ;
	length = GetHandleSize( labelText ) ;
	TETextBox( *labelText, length, &labelRect, teFlushRight);
	HUnlock( labelText ) ;
	ReleaseResource( labelText ) ;

	TextSize(9);
	TextFace(0);
	TextFont(geneva);

// get the profile's vital stats
	subtype = GetProfileSubType( win ) ;
	prof = GetProfileRef( win ) ;
	spec = GetWinFSSpec( win ) ;
	
// get the profile's name element
	cmerr = GetProfName ( prof, (StringPtr)&name ) ;
	if (cmerr) return ;

// draw the profile's name
	MoveTo (varsRect.left,varsRect.top-19) ;
	DrawString( name ) ;

// get profile version
	cmerr = GetProfVersion ( prof, &vers) ;
	if (cmerr) return ;

// set up the profile info text
	varText = GetResource( 'TEXT', rProfileVarsStringID ) ;
	if (vers == cmCS1ProfileVersion)
		MungeProfileHeader1( prof, varText ) ;
	else if (vers >= cmCS2ProfileVersion)
		MungeProfileHeader2( prof, varText ) ;
	else return ;							// need a better error code here

// set up the "where" info text
	whereText = GetResource( 'TEXT', rProfileWhereVarStringID ) ;
	FSSpecToString( GetWinFSSpec(win), path, (subtype==kEmbededSubType) ) ;
	MyReplaceText ( whereText, "\pWhere", path ) ;
	MyReplaceText ( whereText, "\p:", "\p:\0" ) ;

// draw the profile info
	HLock( varText ) ;
	length = GetHandleSize( varText ) ;
	TETextBox( *varText, length, &varsRect, teFlushLeft);
	HUnlock( varText ) ;
	ReleaseResource( varText ) ;	

// draw the "where" information
	HLock( whereText ) ;
	length = GetHandleSize( whereText ) ;
	TETextBox( *whereText, length, &whereRect, teFlushLeft);
	HUnlock( whereText ) ;
	ReleaseResource( whereText ) ;
}


/*------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------*/
void MungeProfileHeader1 ( CMProfileRef prof, Handle text )
{
	CMAppleProfileHeader	head ;
	CMHeader				cm1 ;
	Str255					myStr ;
	CMError					cmerr ;
	
// get the profile's header
	cmerr = CMGetProfileHeader(prof, &head) ;
	if (cmerr) return ;
	cm1 = head.cm1 ;
	
// Vers
	MyReplaceText( text,  "\pVers", "\p1.0" ) ;
// Embed
// Size
	FormatLong( cm1.size, rBytesFMAT, myStr) ;
	MyReplaceText( text,  "\pSize", myStr ) ;
// ColorSpace
	OSTypeToString( cm1.dataType, myStr) ;
	ColorSpace2String ( cm1.dataType, myStr ) ;
	MyReplaceText( text,  "\pColorSpace", myStr ) ;
// CMM
	OSTypeToString( cm1.CMMType, myStr) ;
	MyReplaceText( text,  "\pCMM", myStr ) ;
// Class
	OSTypeToString( cm1.deviceType, myStr) ;
	ProfileClass2String ( cm1.deviceType, myStr ) ;
	MyReplaceText( text,  "\pClass", myStr ) ;
// Manufacturer
	OSTypeToString( cm1.deviceManufacturer, myStr) ;
	MyReplaceText( text,  "\pManufacturer", myStr ) ;
// DeviceModel
	LongHexToString ( cm1.deviceModel, myStr) ;
	MyReplaceText( text,  "\pDeviceModel", myStr ) ;
// Attributes
	LongHexToString ( cm1.deviceAttributes[0], myStr) ;
	MyReplaceText( text,  "\pAttributes0", myStr ) ;
	LongHexToString ( cm1.deviceAttributes[1], myStr) ;
	MyReplaceText( text,  "\pAttributes1", myStr ) ;
// Flags
	LongHexToString ( cm1.flags, myStr) ;
	MyReplaceText( text,  "\pFlags", myStr ) ;
// Intent
	RendIntent2String ( cm1.options, myStr ) ;
	MyReplaceText( text,  "\pIntent", myStr ) ;
// White
	MyReplaceText( text,  "\pWhite", "\pX, Y, Z" ) ;
	FormatSmallFract( cm1.white.X, rDotThreeFMAT, myStr) ;
	MyReplaceText( text,  "\pX", myStr ) ;
	FormatSmallFract( cm1.white.Y, rDotThreeFMAT, myStr) ;
	MyReplaceText( text,  "\pY", myStr ) ;
	FormatSmallFract( cm1.white.Z, rDotThreeFMAT, myStr) ;
	MyReplaceText( text,  "\pZ", myStr ) ;
// Created
	MyReplaceText( text,  "\pCreated", "\p--" ) ;
}


/*------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------*/
void MungeProfileHeader2 ( CMProfileRef prof, Handle text )
{
	CMAppleProfileHeader	head ;
	CM2Header				cm2 ;
	Str255					myStr ;
	CMError					cmerr ;
	unsigned long			rawSecs ;

// get the profile's header
	cmerr = CMGetProfileHeader(prof, &head) ;
	if (cmerr) return ;
	cm2 = head.cm2 ;
	
// Vers
	VersionToString( cm2.profileVersion, myStr) ;
	MyReplaceText( text,  "\pVers", myStr ) ;
// Embed
// Size
	FormatLong( cm2.size, rBytesFMAT, myStr) ;
	MyReplaceText( text,  "\pSize", myStr ) ;
// ColorSpace
	OSTypeToString( cm2.dataColorSpace, myStr) ;
	ColorSpace2String ( cm2.dataColorSpace, myStr ) ;
	MyReplaceText( text,  "\pColorSpace", myStr ) ;
// CMM
	OSTypeToString( cm2.CMMType, myStr) ;
	MyReplaceText( text,  "\pCMM", myStr ) ;
// Class
	OSTypeToString( cm2.profileClass, myStr) ;
	ProfileClass2String ( cm2.profileClass, myStr ) ;
	MyReplaceText( text,  "\pClass", myStr ) ;
// Manufacturer
	OSTypeToString( cm2.deviceManufacturer, myStr) ;
	MyReplaceText( text,  "\pManufacturer", myStr ) ;
// DeviceModel
	LongHexToString ( cm2.deviceModel, myStr) ;
	MyReplaceText( text,  "\pDeviceModel", myStr ) ;
// Attributes
	LongHexToString ( cm2.deviceAttributes[0], myStr) ;
	MyReplaceText( text,  "\pAttributes0", myStr ) ;
	LongHexToString ( cm2.deviceAttributes[1], myStr) ;
	MyReplaceText( text,  "\pAttributes1", myStr ) ;
// Flags
	LongHexToString ( cm2.flags, myStr) ;
	MyReplaceText( text,  "\pFlags", myStr ) ;
// Intent
	RendIntent2String ( cm2.renderingIntent, myStr ) ;
	MyReplaceText( text,  "\pIntent", myStr ) ;
// White
	MyReplaceText( text,  "\pWhite", "\pX, Y, Z" ) ;
	FormatFixed( cm2.white.X, rDotThreeFMAT, myStr) ;
	MyReplaceText( text,  "\pX", myStr ) ;
	FormatFixed( cm2.white.Y, rDotThreeFMAT, myStr) ;
	MyReplaceText( text,  "\pY", myStr ) ;
	FormatFixed( cm2.white.Z, rDotThreeFMAT, myStr) ;
	MyReplaceText( text,  "\pZ", myStr ) ;
// Created
	MyReplaceText( text,  "\pCreated", "\pDate, Time" ) ;
	Date2Secs( (DateTimeRec*)&(cm2.dateTime), &rawSecs);
	IUDateString( rawSecs, abbrevDate, myStr);
	MyReplaceText( text,  "\pDate", myStr ) ;
	IUTimeString( rawSecs, false, myStr);
	MyReplaceText( text,  "\pTime", myStr ) ;
}


OSErr RendIntent2String ( unsigned long intent, StringPtr dest )
{
	short			i, count ;
	FlagsListHdl		list ;
	
//	LongHexToString ( intent, dest) ;
//	pStrCat (dest, "\p = ", 255) ;

	list = (FlagsListHdl) GetResource('FLG#', 128) ;
	if (list==nil) return ResError();
	
	count = (**list).count ;
	for (i=0; i<count; i++ )
		if ( (intent>>(**list).shift) == i )
		//	pStrCat (dest, (**list).entry[i].string, 255) ;
			StringToString( (**list).entry[i].string, dest) ;
	ReleaseResource( (Handle)list ) ;
	return noErr ;
}

OSErr ColorSpace2String ( OSType spaceType, StringPtr dest )
{
	short			i, count ;
	OSTypeListHdl	list ;
	
	list = (OSTypeListHdl) GetResource('OST#', 129) ;
	if (list==nil) return ResError();
	
	count = (**list).count ;
	for (i=0; i<count; i++ )
		if ((**list).entry[i].type == spaceType )
			StringToString( (**list).entry[i].string, dest) ;
	ReleaseResource( (Handle)list ) ;
	return noErr ;
}

OSErr ProfileClass2String ( OSType classType, StringPtr dest )
{
	short			i, count ;
	OSTypeListHdl	list ;
	
	list = (OSTypeListHdl) GetResource('OST#', 128) ;
	if (list==nil) return ResError();

	count = (**list).count ;
	for (i=0; i<count; i++ )
		if ((**list).entry[i].type == classType )
			StringToString( (**list).entry[i].string, dest) ;
	ReleaseResource( (Handle)list ) ;
	return noErr ;
}





/*------------------------------------------------------------------------------*\
	DoCreateProfWindow
 *------------------------------------------------------------------------------*
		This routine actually creates the WindowPtr for a winHandle.
		After creating the window, it:
			reference the window and the DocumentRecord to each other,
			makes the window the current port,
			resizes the window according to the WIND resource,
			titles it according the the FSSpec, and
			makes it visible .
		This routine is called by:
			winOpenProfList() which fills in the winHandle structure
\*------------------------------------------------------------------------------*/
OSErr DoCreateProfWindow ( winHandle win )
{
	OSErr			err = noErr ;
	WindowPtr		window ;
	FSSpec			spec ;
	CMProfileRef 	prof ;
	OSType			subtype ;
	Str255			winTitle="\p" ;
		
	// create the window
	window = GetNewCWindow(rProfileWindID, nil, (WindowPtr)-1 );
			
	SetWinWindow( win, (CGrafPtr)window );	// save a reference to the window in the winRecord
	SetWindowWinHandle ( window, win );		// save a reference to the winRecord in the window

	SetGWorld( GetWinWindow( win ), nil ) ;	// set the window to be the current port

	SetWinRect( win, ((CGrafPtr)window)->portRect ) ;

	spec = GetWinFSSpec( win ) ;
	prof = GetProfileRef( win ) ;
	subtype = GetProfileSubType( win ) ;

	// set the name of this document window
	switch (subtype)
	{
		case kFileSubType :
			SetWTitle( window, spec.name ) ;
			break ;
			
		case kEmbededSubType :
			NumToString( GetProfileIndex(win), winTitle ) ;
			pStrIns( winTitle, "\p:", 255 ) ;
			pStrIns( winTitle, spec.name, 255 ) ;
			SetWTitle( window, winTitle ) ;
			break ;
			
		case kSysProfSubType :
			err = GetProfName( nil, winTitle ) ;
			SetWTitle( window, winTitle ) ;
			break ;
	}
	
	// make sure it is visible
	ShowWindow( window ) ;
	
	return err ;
}

	


